package com.uoc.footballteam.business;

import java.sql.Time;
import java.util.ArrayList;

import com.uoc.footballteam.integration.DEntrenaments;
import com.uoc.footballteam.integration.DJugadors;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;

/**
 * Classe que gestiona els entrenaments.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public class NEntrenaments {
	//Fitxer de configuraci de MyFrameUOC que s'utilitza.
	private String file		= null;
	
	/**
	 * Constructor.
	 * 
	 * @param file - Fixer de configuraci de MyFrameUOC que s'utilitza.
	 */
	public NEntrenaments(String file){
		this.file = file;
	}

	/**
	 * Permet obtenir una llista dels entrenaments que hi ha disponibles.
	 * 
	 * @return Un <code>ArrayList</code> que cont els entrenaments disponibles.
	 * 
	 * @throws MyFrameUOCException Si no es poden obtenir els entrenaments. 
	 */
	public ArrayList<DEntrenaments> consultaLlistatEntrenaments() throws MyFrameUOCException{
		ArrayList<DEntrenaments> entrenaments		= null;
		DEntrenaments dentrenaments					= null;
		
		dentrenaments = new DEntrenaments();
		entrenaments = dentrenaments.consultaLlistatEntrenaments(this.file);
		
		return (entrenaments);
	}
	
	/**
	 * Permet obtenir un entrenament que est emmagatzemat en la base de dades.
	 * 
	 * @param idEntrenament - Identificador de l'entrenament que es vol recuperar de la base de dades.
	 * 
	 * @return Un <code>DEntrenaments</code> que cont l'entrenament que es recupera de la base de dades.
	 * 
	 * @throws MyFrameUOCException Si es produeix algn error al recuperar l'entrenament de la base de dades. 
	 */
	public DEntrenaments consultaEntrenament(String idEntrenament) throws MyFrameUOCException{
		DEntrenaments entrenament	= null;
		
		entrenament = new DEntrenaments();
		entrenament = entrenament.consultaEntrenament(this.file, idEntrenament);
		
		return (entrenament);
	}
	
	/**
	 * Permet donar d'alta un nou entrenament en la base de dades.
	 * 
	 * @param idEntrenament - Identificador de l'entrenament.
	 * @param nom - Nom de l'entrenament.
	 * @param descripcio - Descripci del pla de l'entrenament.
	 * @param instruccions - Instruccions o passos del pla de l'entrenament.
	 * @param sessions - Nmeo de sessions de qu consta l'entrenament.
	 * @param durada - Durada de cada sessi.
	 * 
	 * @throws MyFrameUOCException Si no es pot donar d'alta l'entrenament. 
	 */
	public void altaEntrenament(String idEntrenament, String nom, String descripcio, String instruccions, int sessions, Time durada) throws MyFrameUOCException{
		DEntrenaments entrenament		= null;
		
		entrenament = new DEntrenaments(idEntrenament, nom, descripcio, instruccions, sessions, durada);
		entrenament.altaEntrenament(this.file);
	}
	
	/**
	 * Permet esborrar un entrenament.
	 * 
	 * @param idEntrenament - Identificador de l'entrenament.
	 * 
	 * @throws Exception Si no es pot esborrar l'entrenament. 
	 */
	public void baixaEntrenament(String idEntrenament) throws Exception{
		DEntrenaments entrenament		= null;
		
		entrenament = new DEntrenaments();
		entrenament = entrenament.consultaEntrenament(this.file, idEntrenament);
		entrenament.baixaEntrenament(this.file);
	}
	
	/**
	 * Actualitza un entrenament en la base de dades.
	 * 
	 * @param entrenament - Entrenament que es vol actualitzar en la base de dades.
	 * 
	 * @throws Exception Si no es pot actualitzar l'entrenament en la base de dades. 
	 */
	public void actualitzaEntrenament(DEntrenaments entrenament) throws Exception{
		if (entrenament != null){
			entrenament.actualitzaEntrenament(file);
		}
	}
	
	/**
	 * Permet obtenir una llista dels jugadors que no han seguit un determinat entrenament.
	 * 
	 * @throws MyFrameUOCException Si no es pot obtenir la llista de jugadors que no han seguit l'entrenament. 
	 */
	public ArrayList<DJugadors> consultaJugadorsSenseEntrenament(DEntrenaments entrenament) throws MyFrameUOCException{
		ArrayList<DJugadors> jugadors		= null;
		
		if (entrenament != null){
			jugadors = entrenament.consultaJugadorsSenseEntrenament(this.file);
		}
		return (jugadors);
	}
	
	/**
	 * Afegeix un jugador en un entrenament.
	 * 
	 * @param DNI - DNI del jugador que es vol afegir a l'entrenament
	 * @param entrenament - Entrenament a on es vol afegir el jugador.
	 * 
	 * @throws Exception Si no es pot afegir el jugador a l'entrenament. 
	 */
	public void incloureJugador(char[] DNI, DEntrenaments entrenament) throws Exception{
		if (entrenament != null){
			entrenament.incloureJugador(this.file, DNI);
		}
	}
	
	/**
	 * Treu un jugador d'un entrenament
	 * 
	 * @param DNI - DNI del jugador que es vol treure de l'entrenament
	 * @param entrenament - Entrenament del qual es vol treure el jugador.
	 * 
	 * @throws Exception Si no es pot treure el jugador de l'entrenament 
	 */
	public void treureJugador(char[] DNI, DEntrenaments entrenament) throws Exception{
		if (entrenament != null){
			entrenament.treureJugador(this.file, DNI);
		}
	}
	
	/**
	 * Permet obtenir una llista dels entrenaments en els quals no est incls un jugador.
	 * 
	 * @param jugador - El jugador del qual es vol obtenir el llistat dels entrenamentes en els quals no est incls.
	 * 
	 * @return Un <code>ArrayList</code> que cont els entrenaments en els quals no participa el jugador.
	 * 
	 * @throws MyFrameUOCException Si no es pot obtenir la llista dels entrenaments en els quals no est incls el jugador. 
	 */
	public ArrayList<DEntrenaments> consultaLlistatEntrenamentsNoJugador(DJugadors jugador) throws MyFrameUOCException{
		ArrayList<DEntrenaments> entrenaments		= null;
		ArrayList<DEntrenaments> entrenamentsNo		= null;
		DEntrenaments dentrenaments					= null;
		
		if (jugador != null){
			dentrenaments = new DEntrenaments();
			entrenaments = dentrenaments.consultaLlistatEntrenaments(this.file);
			if (entrenaments != null && entrenaments.size() > 0){
				entrenamentsNo = new ArrayList<DEntrenaments>();
				for (DEntrenaments ent: entrenaments){
					if (!jugador.conteEntrenament(ent.getIdEntrenament())){
						entrenamentsNo.add(ent);
					}
				}
			}
		}
		return (entrenamentsNo);
	}
}
