package com.uoc.footballteam.integration;

import java.math.BigDecimal;
import java.util.ArrayList;

import com.uoc.myframeuoc.annotations.*;
import com.uoc.myframeuoc.environment.Session;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;

/**
 * Classe que representa un equip.
 *  
 * @author Jordi Garcia Casanova.
 *
 */
@Entity(Table = "Equip")
public class DEquips {
	@PrimaryKey(Name = "PK_Equip", Field = "campIdEquip")
	private int idEquip;
	
	@Column(Field = "campNom", Length="50", Required = true)
	private String nom;
	
	@Column(Field = "campEstadi", Length="50")
	private String estadi;
	
	@Column(Field = "campPressupost", Precision = "20", Scale = "2")
	private BigDecimal pressupost;
	
	@OneToMany(Table = "EquipJugador", NameFK_To = "FK1_EquipJugador", NameFK_From = "FK2_EquipJugador", Fields_To = "jugador-DNI", Fields_From = "equip-idEquip")
	ArrayList<DJugadors> jugadors;
	
	/**
	 * Constructor.
	 */
	public DEquips(){
		
	}
	
	/**
	 * Constructor.
	 * 
	 * @param idEquip - Identificador de l'equip.
	 * @param nom - Nom de l'equip.
	 * @param estadi - Nom de l'estadi.
	 * @param pressupost - Pressupost de l'equip.
	 */
	public DEquips(int idEquip, String nom, String estadi, BigDecimal pressupost) {
		this.idEquip = idEquip;
		this.nom = nom;
		this.estadi = estadi;
		this.pressupost = pressupost;
	}

	//Getters i setters.
	public int getIdEquip() {
		return idEquip;
	}

	public void setIdEquip(int idEquip) {
		this.idEquip = idEquip;
	}

	public String getNom() {
		return nom;
	}

	public void setNom(String nom) {
		this.nom = nom;
	}

	public String getEstadi() {
		return estadi;
	}

	public void setEstadi(String estadi) {
		this.estadi = estadi;
	}

	public BigDecimal getPressupost() {
		return pressupost;
	}

	public void setPressupost(BigDecimal pressupost) {
		this.pressupost = pressupost;
	}
	
	public ArrayList<DJugadors> getJugadors() {
		return jugadors;
	}

	public void setJugadors(ArrayList<DJugadors> jugadors) {
		this.jugadors = jugadors;
	}

	public void afegirJugador(DJugadors jugador){
		if (this.jugadors == null){
			this.jugadors = new ArrayList<DJugadors>();
		}
		this.jugadors.add(jugador);
	}
	
	public void treureJugador(DJugadors jugador){
		boolean trobat		= false;
		int i;
		int numJugadors;
		DJugadors jug		= null;
		
		if (this.jugadors != null){
			numJugadors = this.jugadors.size();
			i = 0;
			trobat = false;
			while (!trobat && i < numJugadors){
				jug = this.jugadors.get(i);
				if (String.valueOf(jug.getDNI()).equals(String.valueOf(jugador.getDNI()))){
					trobat = true;
				}
				else{
					i++;
				}
				if (trobat){
					this.jugadors.remove(i);
				}
			}
		}
	}
	
	/**
	 * Emmagatzema un equip en la base de dades.
	 * 
	 * @param file - Fitxer de configuraci de MyFrameUOC que s'utilitza.
	 * 
	 * @throws MyFrameUOCException Si no es pot emmagatzemar l'equip en la base de dades. 
	 */
	public void altaEquip(String file) throws MyFrameUOCException{
		Session session 	= null;
		
		try{
			session = Session.getSession(file);
			session.insert(this);
		}
		finally{
			if (session != null){
				session.closeNoEx();
			}
		}
	}
	
	/**
	 * Permet obtenir un equip de la base de dades.
	 * 
	 * @param file - Fitxer de configuraci de MyFrameUOC que s'utilitza.
	 * @param idEquip - Identificador de l'equip que es vol consultar.
	 * 
	 * @return Un <code>DEquips</code> que cont l'equip que es recupera de la base de dades.
	 * 
	 * @throws MyFrameUOCException Si no es podrueix algun problema en la recuperaci de l'equip de la base de dades. 
	 */
	public DEquips consultaEquip(String file, int idEquip) throws MyFrameUOCException{
		DEquips equip				= null;
		Session session				= null;
		ArrayList<Object> pk		= null;
		
		try{
			pk = new ArrayList<Object>();
			pk.add(idEquip);
			session = Session.getSession(file);
			equip = (DEquips) session.read(DEquips.class.getCanonicalName(), pk);
		}
		finally{
			if (session != null){
				session.closeNoEx();
			}
		}
		return (equip);
	}
	
	/**
	 * Permet obtenir una llista de tots els equips que estan emmagatzemats en la base de dades.
	 * 
	 * @param file - Fitxer de configuraci de MyFrameUOC que s'utilitza.
	 * 
	 * @return Un <code>ArrayList</code> que cont tots els equips que estan emmagatzemats en la base de dades.
	 * 
	 * @throws MyFrameUOCException Si no es pot recuperar la llista d'equips de la base de dades. 
	 */
	public ArrayList<DEquips> consultaLlistatEquips(String file) throws MyFrameUOCException{
		ArrayList<DEquips> equips		= null;
		ArrayList<Object> llista		= null;
		Session session					= null;
		String queryUOCQL				= null;
		DEquips equip					= null;
		
		try{
			session = Session.getSession(file);
			queryUOCQL = "FROM " + DEquips.class.getCanonicalName() + " WHERE idEquip IS NOT NULL";
			llista = session.executeUOCQL(queryUOCQL);
			if (llista != null && llista.size() > 0){
				equips = new ArrayList<DEquips>();
				for (Object obj: llista){
					equip = (DEquips) obj;
					equips.add(equip);
				}
			}
		}
		finally{
			if (session != null){
				session.closeNoEx();
			}
		}
		
		return (equips);
	}
	
	/**
	 * Elimina un equip de la base de dades.
	 * 
	 * @param file - Fitxer de configuraci de MyFrameUOC que s'utilitza.
	 * 
	 * @throws Exception Si no es pot eliminar l'equip de la base de dades. 
	 */
	public void baixaEquip(String file) throws Exception{
		Session session					= null;
		DJugadors djugadors				= null;
		DJugadors jug					= null;
		
		try{
			session = Session.getSession(file);
			session.delete(this);
			djugadors = new DJugadors();
			if (this.jugadors != null && this.jugadors.size() > 0){
				for (DJugadors jugador: this.jugadors){
					jug = djugadors.consultaJugador(file, jugador.getDNI());
					if (jug == null){
						session.insert(jugador);
					}
				}
			}
		}
		finally{
			if (session != null){
				session.closeNoEx();
			}
		}
	}
	
	/**
	 * Actualiza un equip en la base de dades.
	 * 
	 * @param file - Fitxer de configuraci de MyFrameUOC que s'utilitza.
	 * 
	 * @throws MyFrameUOCException Si no es pot actualitzar l'equip en la base de dades. 
	 */
	public void actualitzaEquip(String file) throws MyFrameUOCException{
		Session session					= null;
		ArrayList<DEquips> equips	 	= null;
		
		try{
			session = Session.getSession(file);
			
			equips = this.consultaLlistatEquips(file);
			if (equips != null && equips.size() > 0){
				for (DEquips eq: equips){
					session.delete(eq);
				}
				
				for (DEquips eq: equips){
					if (this.getIdEquip() == eq.getIdEquip()){
						session.insert(this);
					}
					else{
						session.insert(eq);
					}
				}
			}
		}
		finally{
			if (session != null){
				session.closeNoEx();
			}
		}
	}
	
	/**
	 * Permet obtenir una llista dels jugadors que no tenen equip.
	 * 
	 * @param file - Fitxer de configuraci de MyFrameUOC que s'utilitza.
	 * 
	 * @return Un <code>ArrayList</code> que cont la llista de jugadors que no tenen equip.
	 * 
	 * @throws MyFrameUOCException Si no es pot obtenir la llista de jugadors que no tenen equip. 
	 */
	public ArrayList<DJugadors> consultaJugadorsSenseEquip(String file) throws MyFrameUOCException{
		ArrayList<DJugadors> jugadors				= null;
		ArrayList<DJugadors> jugadorsSenseEquip		= null;
		ArrayList<DEquips> equips					= null;
		DJugadors jugador							= null;
		boolean trobat;
		int numJugadors;
		int numEquips;
		int j;
		DEquips equip								= null;
		
		
		jugador = new DJugadors();
		jugadors = jugador.consultaLlistatJugadors(file);

		if (jugadors != null){
			equips = this.consultaLlistatEquips(file);
			if (equips != null){
				numJugadors = jugadors.size();
				numEquips = equips.size();
				jugadorsSenseEquip = new ArrayList<DJugadors>();

				for (int i = 0; i < numJugadors; i++){
					jugador = jugadors.get(i);
					trobat = false;
					j = 0;
					while (!trobat && j < numEquips){
						equip = equips.get(j);
						if (equip.conteJugador(jugador)){
							trobat = true;
						}
						else{
							j++;
						}
					}
					if (!trobat){
						jugadorsSenseEquip.add(jugador);
					}
				}
			}
		}

		return (jugadorsSenseEquip);
	}
	
	/*
	 * Permet conixer si un jugador pertany a un equip.
	 */
	private boolean conteJugador(DJugadors jugador){
		boolean conte 	= false;
		int i;
		int numJugadors;
		DJugadors jug	= null;
		
		if (jugador != null){
			if (this.jugadors != null){
				numJugadors = this.jugadors.size();
				i = 0;
				while (!conte && i < numJugadors){
					jug = this.jugadors.get(i);
					if (String.valueOf(jug.getDNI()).equals(String.valueOf(jugador.getDNI()))){
						conte = true;
					}
					else{
						i++;
					}
				}
			}
		}
		
		return (conte);
	}
	
	/**
	 * Afegeix un jugador a l'equip.
	 * 
	 * @param file - Fitxer de configuraci de MyFrameUOC que s'utilitza.
	 * @param DNI - DNI del jugador que es vol afegir a l'equip.
	 * 
	 * @throws Exception Si no es pot afegir el jugador a l'equip. 
	 */
	public void incloureJugador(String file, char[] DNI) throws Exception{
		DJugadors jugador						= null;
		Session session							= null;
		ArrayList<Object> pk					= null;
		ArrayList<DEntrenaments> entrenaments	= null;
		DEntrenaments dentrenaments				= null;
		ArrayList<DEquips> equips				= null;
		DEquips dequips							= null;
		DEntrenaments entrenament				= null;
		
		try{
			session = Session.getSession(file);
			dentrenaments = new DEntrenaments();
			entrenaments = dentrenaments.consultaLlistatEntrenaments(file);
			
			dequips = new DEquips();
			equips = dequips.consultaLlistatEquips(file);
			
			pk = new ArrayList<Object>();
			pk.add(DNI);
			jugador = (DJugadors) session.read(DJugadors.class.getCanonicalName(), pk);
			if (jugador != null){
				this.afegirJugador(jugador);
				if (entrenaments != null && entrenaments.size() > 0){
					for (DEntrenaments ent: entrenaments){
						ent.baixaEntrenament(file);
					}
				}
				
				if (equips != null && equips.size() > 0){
					for (DEquips eq: equips){
						if (eq.getIdEquip() != this.idEquip){
							eq.actualitzaEquip(file);
						}
						else{
							session.update(this);
						}
					}
				}
				
				if (entrenaments != null && entrenaments.size() > 0){
					for (DEntrenaments ent: entrenaments){
						entrenament = dentrenaments.consultaEntrenament(file, ent.getIdEntrenament());
						if (entrenament == null){
							ent.altaEntrenament(file);
						}
					}
				}
			}
		}
		finally{
			if (session != null){
				session.closeNoEx();
			}
		}
	}
	
	/**
	 * Treu un jugador de l'equip.
	 * 
	 * @param file - Fitxer de configuraci de MyFrameUOC que s'utilitza.
	 * @param DNI - DNI del jugador que es vol treure de l'equip.
	 * 
	 * @throws Exception Si no es pot treure el jugador de l'equip. 
	 */
	public void treureJugador(String file, char[] DNI) throws Exception{
		DJugadors jugador						= null;
		Session session							= null;
		ArrayList<Object> pk					= null;
		DJugadors jug							= null;
		ArrayList<DEquips> equips				= null;
		DEquips dequips							= null;
		ArrayList<DEntrenaments> entrenaments	= null;
		DEntrenaments dentrenaments				= null;
		DEntrenaments entrenament				= null;
		
		try{
			session = Session.getSession(file);
			
			dentrenaments = new DEntrenaments();
			entrenaments = dentrenaments.consultaLlistatEntrenaments(file);
			
			dequips = new DEquips();
			equips = dequips.consultaLlistatEquips(file);
			
			pk = new ArrayList<Object>();
			pk.add(DNI);
			jugador = (DJugadors) session.read(DJugadors.class.getCanonicalName(), pk);
			if (jugador != null){
				this.treureJugador(jugador);
				if (entrenaments != null && entrenaments.size() > 0){
					for (DEntrenaments ent: entrenaments){
						ent.baixaEntrenament(file);
					}
				}
				
				if (equips != null && equips.size() > 0){
					for (DEquips eq: equips){
						if (eq.getIdEquip() != this.idEquip){
							session.update(eq);
						}
						else{
							session.update(this);
							jug = (DJugadors) session.read(DJugadors.class.getCanonicalName(), pk);
							if (jug == null){
								session.insert(jugador);
							}
						}
					}
				}
				
				if (entrenaments != null && entrenaments.size() > 0){
					for (DEntrenaments ent: entrenaments){
						entrenament = dentrenaments.consultaEntrenament(file, ent.getIdEntrenament());
						if (entrenament == null){
							ent.altaEntrenament(file);
						}
					}
				}
			}
		}
		finally{
			if (session != null){
				session.closeNoEx();
			}
		}
	}
	
	/**
	 * Elimina la taula associada als equips.
	 * 
	 * @param file - Fitxer de configuraci de MyFrameUOC que s'utilitza.
	 * 
	 * @throws MyFrameUOCException Si no es pot eliminar la taula dels equips. 
	 */
	public void drop(String file) throws MyFrameUOCException{
		Session session		= null;
		
		try{
			session = Session.getSession(file);
			session.drop(DEquips.class.getCanonicalName());
		}
		finally{
			if (session != null){
				session.closeNoEx();
			}
		}
	}
}
