package com.uoc.myframeuoc.database;

import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Time;
import java.util.ArrayList;
import java.util.HashMap;

import com.uoc.myframeuoc.exceptions.DataBaseMsg;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.utils.MyFrameUOC;

/**
 * Classe que representa una connexi amb la base de dades.
 * 
 * @author Jordi Garcia Casanova
 *
 */
public abstract class Connection {

	//Connexi a la base de dades.
	protected java.sql.Connection connection				= null;
	
	//Tipus de dades mapejats.
	protected HashMap<String, ArrayList<TypeMap>> typeMaps	= null;
	
	//Restricci utilitzada per a definir una columna com a obligatria.
	protected String notNullConstraint						= null;
	
	//Restricci utilitzada per a definir una columna com no repetible.
	protected String uniqueConstraint						= null;
	
	/**
	 * Constructor.
	 */
	public Connection() {
	}
	
	@Override
	protected void finalize() throws Throwable {
		super.finalize();
		
		if (this.connection != null){
			this.connection.close();
		}
	}

	/**
	 * Executa una consulta SQL de tipus DDL cap a la base de dades.
	 * 
	 * @param query - Consulta SQL de tipus DDL que es llanar cap a la base de dades.
	 * 
	 * @throws MyFrameUOCException Si no es pot executar la sentncia SQL.
	 */
	public void executeSQLDDL(String query) throws MyFrameUOCException{
		Statement stm		= null;
		
		try{
			if (this.connection == null){
				//Error => No hi ha una connexi oberta amb la base de dades.
				throw new MyFrameUOCException(DataBaseMsg.NO_CONNEXIO_OBERTA);
			}
			if (query != null && !"".equals(query)){
				stm = this.connection.createStatement();
				stm.executeUpdate(query);
			}
		}
		catch (Exception e){
			//No s'ha pogut executar la constula SQL sobre la base de dades.
			throw new MyFrameUOCException(DataBaseMsg.NO_EXECUTAR_CONSULTA, e.getMessage());
		}
		finally{
			if (stm != null){
				try {
					stm.close();
				} 
				catch (SQLException e) {
				}
			}
		}
	}
	
	/**
	 * Executa una sentncia SQL DDL preparada.
	 * 
	 * @param preparedQuery - Sentncia SQL DDL preparada.
	 * @param obj - Objecte que cont els valors a substituir.
	 * 
	 * @throws MyFrameUOCException Si no es pot executar la sentncia SQL DDL preparada. 
	 */
	@SuppressWarnings("unchecked")
	public void executeSQLDDL(String preparedQuery, Object obj) throws MyFrameUOCException{
		String query						= null;
		ArrayList<Object> values			= null;
		Object[] preparedStatementValues	= null;
		PreparedStatement stm				= null;
		int elements;
		Object value						= null;
		
		
		try{	
			if (preparedQuery == null || "".equals(preparedQuery)){
				//Error => La sentncia SQL est buida.
				throw new MyFrameUOCException(DataBaseMsg.EXECUTESQLDDL_SENSE_SENTENCIA_PREPARADA);
			}
			if (obj == null){
				//Error => L'objecte del qual s'obtenen els valors s nul.
				throw new MyFrameUOCException(DataBaseMsg.EXECUTSQLDDL_SENSE_OBJECTE);
			}
			
			preparedStatementValues = this.prepareStatementValues(preparedQuery, obj);
						
			if (preparedStatementValues != null && preparedStatementValues.length == 2){
				query = (String) preparedStatementValues[0];
				values = (ArrayList<Object>) preparedStatementValues[1];
				stm = this.connection.prepareStatement(query);
				stm.clearParameters();
				elements = values.size();
				for (int i = 0; i < elements; i++){
					value = values.get(i);
					this.setValuePreparedStatement(stm, i + 1, value);
				}
				stm.executeUpdate();
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'execuci de la sentncia SQL preaprada.
			throw new MyFrameUOCException(DataBaseMsg.EXECUTESQLDDL_ERROR, e.getMessage());
		}
		finally{
			if (stm != null){
				try {
					stm.close();
				} 
				catch (SQLException e) {
				}
			}
		}
	}
	
	/**
	 * Executa una sentncia SQL DDL preparada.
	 * 
	 * @param preparedQuery - Sentncia SQL DDL preparada.
	 * @param objCollection - Objecte d'una collecci de l'objecte 'obj' i qu cont els valors a substituir.
	 * @param obj - Objecte que cont els valors a substituir.
	 * 
	 * @throws MyFrameUOCException Si no es pot executar la sentncia SQL DDL preparada. 
	 */
	@SuppressWarnings("unchecked")
	public void executeSQLDDL(String preparedQuery, Object objCollection, Object obj) throws MyFrameUOCException{
		String query						= null;
		ArrayList<Object> values			= null;
		Object[] preparedStatementValues	= null;
		PreparedStatement stm				= null;
		int elements;
		Object value						= null;
		
		
		try{	
			if (preparedQuery == null || "".equals(preparedQuery)){
				//Error => La sentncia SQL est buida.
				throw new MyFrameUOCException(DataBaseMsg.EXECUTESQLDDL_SENSE_SENTENCIA_PREPARADA);
			}
			if (obj == null){
				//Error => L'objecte del qual s'obtenen els valors s nul.
				throw new MyFrameUOCException(DataBaseMsg.EXECUTSQLDDL_SENSE_OBJECTE);
			}
			if (objCollection == null){
				//Error => L'objecte del qual s'obtenen els valors s nul.
				throw new MyFrameUOCException(DataBaseMsg.EXECUTSQLDDL_SENSE_OBJECTE);
			}
			
			preparedStatementValues = this.prepareStatementValues(preparedQuery, objCollection, obj);
						
			if (preparedStatementValues != null && preparedStatementValues.length == 2){
				query = (String) preparedStatementValues[0];
				values = (ArrayList<Object>) preparedStatementValues[1];
				stm = this.connection.prepareStatement(query);
				stm.clearParameters();
				elements = values.size();
				for (int i = 0; i < elements; i++){
					value = values.get(i);
					this.setValuePreparedStatement(stm, i + 1, value);
				}
				stm.executeUpdate();
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'execuci de la sentncia SQL preaprada.
			throw new MyFrameUOCException(DataBaseMsg.EXECUTESQLDDL_ERROR, e.getMessage());
		}
		finally{
			if (stm != null){
				try {
					stm.close();
				} 
				catch (SQLException e) {
				}
			}
		}
	}
	
	/**
	 * Executa una sentncia SQL preparada.
	 * 
	 * @param preparedQuery - Sentncia SQL preparada.
	 * @param values - Llista de valors per a posar en la restricci de la sentncia SQL preparada.
	 * 
	 * @return Un <code>ResultSet</code> amb el resultat de l'execuci de la sentncia SQL.
	 * 
	 * @throws MyFrameUOCException Si no es pot executar la sentncia SQL preparada. 
	 */
	public ResultSet executeSQL(String preparedQuery, ArrayList<Object> values) throws MyFrameUOCException{
		PreparedStatement stm		= null;
		int elements;
		ResultSet rs				= null;
		
		
		
		try{
			if (preparedQuery == null || "".equals(preparedQuery)){
				//Error => La sentncia preparada est buida.
				throw new MyFrameUOCException(DataBaseMsg.EXECUTESQL_SENSE_SENTENCIA);
			}
			if (values == null || values.size() == 0){
				//Error => No hi han valors per a la restricci.
				throw new MyFrameUOCException(DataBaseMsg.EXECUTESQL_SENSE_VALORS);
			}
			
			stm = this.connection.prepareStatement(preparedQuery);
			elements = values.size();
			for (int i = 0; i < elements; i++){
				this.setValuePreparedStatement(stm, i + 1, values.get(i));
			}
			rs = stm.executeQuery();
			
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'execuci de la sentncia SQL preaprada.
			throw new MyFrameUOCException(DataBaseMsg.EXECUTESQLDDL_ERROR, e.getMessage());
		}
		return (rs);
	}
	
	/*
	 * Obt la sentncia SQL preparada amb els valors a substituir.
	 * 
	 * @param queryPrepared - Sentncia SQL preparada.
	 * @param objCollection - Objecte d'una collecci de l'objecte 'obj' d'on s'obtindran els valors a substituir en la sentncia SQL.
	 * @param obj - Objcecte d'on s'obtindran els valors a substituir en la sentncia SQL.
	 */
	private Object[] prepareStatementValues(String queryPrepared, Object objCollection, Object obj) throws MyFrameUOCException{
		String[] params				= null;
		int elements;
		String queryPreparedFinal	= null;
		ArrayList<Object> values	= null;
		Object[] res				= null;
		
		try{
			if (queryPrepared != null && !"".equals(queryPrepared)){
				params = queryPrepared.split(MyFrameUOC.PARAM_STATEMENT);
				if (params != null && params.length > 1){
					elements = params.length;
					values = new ArrayList<Object>();
					queryPreparedFinal = new String(queryPrepared);
					for (int i = 1; i < elements; i = i + 2){
						queryPreparedFinal = queryPreparedFinal.replace(MyFrameUOC.PARAM_STATEMENT + params[i] + MyFrameUOC.PARAM_STATEMENT, "?");
						values.add(this.getObjectValue(params[i], objCollection, obj));
					}
					res = new Object[2];
					res[0] = queryPreparedFinal;
					res[1] = values;
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'obtenci de la sentncia i els valors a substituir.
			throw new MyFrameUOCException(DataBaseMsg.PREPARAR_SENTENCIA_VALORS_ERROR, e.getMessage());
		}	
		return (res);
	}
	
	/*
	 * Obt la sentncia SQL preparada amb els valors a substituir.
	 * 
	 * @param queryPrepared - Sentncia SQL preparada.
	 * @param obj - Objcecte d'on s'obtindrn els valors a substituir en la sentncia SQL.
	 */
	private Object[] prepareStatementValues(String queryPrepared, Object obj) throws MyFrameUOCException{
		String[] params				= null;
		int elements;
		String queryPreparedFinal	= null;
		ArrayList<Object> values	= null;
		Object[] res				= null;
		
		try{
			if (queryPrepared != null && !"".equals(queryPrepared)){
				params = queryPrepared.split(MyFrameUOC.PARAM_STATEMENT);
				if (params != null && params.length > 1){
					elements = params.length;
					values = new ArrayList<Object>();
					queryPreparedFinal = new String(queryPrepared);
					for (int i = 1; i < elements; i = i + 2){
						queryPreparedFinal = queryPreparedFinal.replace(MyFrameUOC.PARAM_STATEMENT + params[i] + MyFrameUOC.PARAM_STATEMENT, "?");
						values.add(this.getObjectValue(params[i], obj));
					}
					res = new Object[2];
					res[0] = queryPreparedFinal;
					res[1] = values;
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'obtenci de la sentncia i els valors a substituir.
			throw new MyFrameUOCException(DataBaseMsg.PREPARAR_SENTENCIA_VALORS_ERROR, e.getMessage());
		}	
		return (res);
	}
	
	/*
	 * Permet obtenir l'objecte a partir d'una ruta.
	 * 
	 * @param url - Ruta a on cal anar a cercar els valors d'un objecte.
	 * @param objCollection - Objecte d'una collecci de 'obj' d'on s'onbtindran els valors.
	 * @param obj - Objecte d'on s'obtindran els valors.
	 */
	private Object getObjectValue(String url, Object objCollection, Object obj) throws MyFrameUOCException{
		Object res				= null;
		String[] params			= null;
		int elements;
		Class<?> c				= null;
		Method method			= null;
		Class<?>[] methodArgs	= null;
		Object[] invokeArgs		= null;
		String getter			= null;
		
		try{
			if (obj != null && url != null && !"".equals(url)){
				params = url.split(MyFrameUOC.SEPARATOR_PARAM_STATEMENT);
				if (params != null && params.length > 0){
					elements = params.length;
					if (elements == 2){
						c = Class.forName(params[0]);
						methodArgs = new Class[]{};
						getter = "get" + String.valueOf(params[1].charAt(0)).toUpperCase() + params[1].substring(1, params[1].length());
						method = c.getMethod(getter, methodArgs);
						invokeArgs = new Object[]{};
						res = method.invoke(obj, invokeArgs);
					}
					else if (elements == 3){
						c = Class.forName(params[1]);
						methodArgs = new Class[]{};
						getter = "get" + String.valueOf(params[2].charAt(0)).toUpperCase() + params[2].substring(1, params[2].length());
						method = c.getMethod(getter, methodArgs);
						invokeArgs = new Object[]{};
						res = method.invoke(objCollection, invokeArgs);
					}
					else if (elements == 4){
						c = Class.forName(params[0]);
						methodArgs = new Class[]{};
						getter = "get" + String.valueOf(params[1].charAt(0)).toUpperCase() + params[1].substring(1, params[1].length());
						method = c.getMethod(getter, methodArgs);
						res = method.invoke(obj, invokeArgs);
						
						if (res != null){
							c = Class.forName(params[2]);
							getter = "get" + String.valueOf(params[3].charAt(0)).toUpperCase() + params[3].substring(1, params[3].length());
							method = c.getMethod(getter, methodArgs);
							res = method.invoke(res, invokeArgs);
						}
					}
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en obtenir els valors d'un objecte.
			throw new MyFrameUOCException(DataBaseMsg.OBTENIR_OBJECTE_ERROR, e.getMessage());
		}
		return (res);
	}
	
	/*
	 * Permet obtenir l'objecte a partir d'una ruta.
	 * 
	 * @param url - Ruta a on cal anar a cercar els valors d'un objecte.
	 * @param obj - Objecte d'on s'obtindran els valors.
	 */
	private Object getObjectValue(String url, Object obj) throws MyFrameUOCException{
		Object res				= null;
		String[] params			= null;
		int elements;
		Class<?> c				= null;
		Method method			= null;
		Class<?>[] methodArgs	= null;
		Object[] invokeArgs		= null;
		String getter			= null;
		
		try{
			if (obj != null && url != null && !"".equals(url)){
				params = url.split(MyFrameUOC.SEPARATOR_PARAM_STATEMENT);
				if (params != null && params.length > 0){
					elements = params.length;
					if (elements == 2){
						c = Class.forName(params[0]);
						methodArgs = new Class[]{};
						getter = "get" + String.valueOf(params[1].charAt(0)).toUpperCase() + params[1].substring(1, params[1].length());
						method = c.getMethod(getter, methodArgs);
						invokeArgs = new Object[]{};
						res = method.invoke(obj, invokeArgs);
					}
					else if (elements == 4){
						c = Class.forName(params[0]);
						methodArgs = new Class[]{};
						getter = "get" + String.valueOf(params[1].charAt(0)).toUpperCase() + params[1].substring(1, params[1].length());
						method = c.getMethod(getter, methodArgs);
						res = method.invoke(obj, invokeArgs);
						
						if (res != null){
							c = Class.forName(params[2]);
							getter = "get" + String.valueOf(params[3].charAt(0)).toUpperCase() + params[3].substring(1, params[3].length());
							method = c.getMethod(getter, methodArgs);
							res = method.invoke(res, invokeArgs);
						}
					}
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en obtenir els valors d'un objecte.
			throw new MyFrameUOCException(DataBaseMsg.OBTENIR_OBJECTE_ERROR, e.getMessage());
		}
		return (res);
	}
	
	/*
	 * Insereix un valor en la sentncia preparada.
	 * 
	 * @param stm - Sentncia preparada.
	 * @param pos - Posici en la qual s'inserir el valor.
	 * @param value - El valor que s''inserir en la sentncia preparada.
	 */
	private void setValuePreparedStatement(PreparedStatement stm, int pos, Object value) throws MyFrameUOCException{
		String type				= null;
		
		try{
			if (stm == null){
				//Error => La sentncia preparada est buida.
				throw new MyFrameUOCException(DataBaseMsg.INSERTAR_VALOR_SENSE_SENTENCIA);
			}
			
			if (value == null){
				stm.setNull(pos, java.sql.Types.NULL);
				return;
				
			}
			type = value.getClass().getCanonicalName();
			if (JavaType.String.getType().equals(type)){
				stm.setString(pos, (String) value);
			}
			else if (JavaType.char_.getType().equals(type)){
				stm.setString(pos, String.valueOf(value));
			}
			else if (JavaType.Character.getType().equals(type)){
				stm.setString(pos, String.valueOf(value));
			}
			else if (JavaType.char_array.getType().equals(type)){
				stm.setString(pos, String.copyValueOf((char[]) value));
			}
			else if (JavaType.int_.getType().equals(type)){
				stm.setInt(pos, Integer.valueOf(String.valueOf(value)));
			}
			else if (JavaType.Integer.getType().equals(type)){
				stm.setInt(pos, (Integer) value);
			}
			else if (JavaType.float_.getType().equals(type)){
				stm.setFloat(pos, Float.valueOf(String.valueOf(value)));
			}
			else if (JavaType.Float.getType().equals(type)){
				stm.setFloat(pos, (Float) value);
			}
			else if (JavaType.double_.getType().equals(type)){
				stm.setDouble(pos, Double.valueOf(String.valueOf(value)));
			}
			else if (JavaType.Double.getType().equals(type)){
				stm.setDouble(pos, (Double) value);
			}
			else if (JavaType.BigDecimal.getType().equals(type)){
				stm.setBigDecimal(pos, (BigDecimal) value);
			}
			else if (JavaType.Date.getType().equals(type)){
				stm.setDate(pos, (Date) value);
			}
			else if (JavaType.byte_.getType().equals(type)){
				stm.setByte(pos, Byte.valueOf(String.valueOf(value)));
			}
			else if (JavaType.Byte.getType().equals(type)){
				stm.setByte(pos, (Byte) value);
			}
			else if (JavaType.short_.getType().equals(type)){
				stm.setShort(pos, Short.valueOf(String.valueOf(value)));
			}
			else if (JavaType.Short.getType().equals(type)){
				stm.setShort(pos, (Short) value);
			}
			else if (JavaType.long_.getType().equals(type)){
				stm.setLong(pos, Long.valueOf(String.valueOf(value)));
			}
			else if (JavaType.Long.getType().equals(type)){
				stm.setLong(pos, (Long) value);
			}
			else if (JavaType.boolean_.getType().equals(type)){
				stm.setBoolean(pos, Boolean.valueOf(String.valueOf(value)));
			}
			else if (JavaType.Boolean.getType().equals(type)){
				stm.setBoolean(pos, (Boolean) value);
			}
			else if (JavaType.Time.getType().equals(type)){
				stm.setTime(pos, (Time) value);
			}
			else if (JavaType.BigInteger.getType().equals(type)){
				stm.setString(pos, String.valueOf(value));
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la inserci del valors en una sentncia preparada.
			throw new MyFrameUOCException(DataBaseMsg.INSERTAR_VALOR_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Permet obtenir els tipus de dades mapejats que es poden utilitzar
	 * en MyFrameUOC i que depenen del sistema de gesti de base
	 * de dades utilitzat.
	 * 
	 * @return Un <code>HashMap</code> amb els tipus de dades mapejats
	 * que es poden utilitzar en MyFrameUOC.
	 */
	public HashMap<String, ArrayList<TypeMap>> getTypeMaps(){
		return (this.typeMaps);
	}
	
	/**
	 * Inicialitza una transacci en la base de dades.
	 *  
	 * @throws MyFrameUOCException Si no es pot inicialitzar una transacci.
	 */
	public void beginTransaction() throws MyFrameUOCException{
		if (this.connection != null){
			try {
				this.connection.setAutoCommit(false);
				
			} 
			catch (SQLException e) {
				//Error => No s'ha pogut inicialitzar la transacci.
				throw new MyFrameUOCException(DataBaseMsg.NO_INICIAR_TRANSACCIO, e.getMessage());
			}
		}
		else{
			//Error => No hi ha una connexi oberta amb la base de dades.
			throw new MyFrameUOCException(DataBaseMsg.NO_CONNEXIO_OBERTA);
		}
	}
	
	/**
	 * Confirma una transacci oberta.
	 * 
	 * @throws MyFrameUOCException Si no es pot confirmar una transacci. 
	 */
	public void commit() throws MyFrameUOCException{
		if (this.connection != null){
			try {
				this.connection.commit();
				this.connection.setAutoCommit(true);
			} 
			catch (SQLException e) {
				//Error => No s'ha pogut confirmar la transacci.
				throw new MyFrameUOCException(DataBaseMsg.NO_CONFIRMAR_TRANSACCIO, e.getMessage());
			}
		}
		else{
			//Error => No hi ha una connexi oberta amb la base de dades.
			throw new MyFrameUOCException(DataBaseMsg.NO_CONNEXIO_OBERTA);
		}
	}
	
	/**
	 * Cancella una transacci.
	 * 
	 * @throws MyFrameUOCException Si no es pot cancellar una transacci.
	 */
	public void rollback() throws MyFrameUOCException{
		if (this.connection != null){
			try {
				this.connection.rollback();
				this.connection.setAutoCommit(true);
			} 
			catch (SQLException e) {
				//Error => No s'ha pogut cancellar la transacci.
				throw new MyFrameUOCException(DataBaseMsg.NO_CANCELAR_TRANSACCIO, e.getMessage());
			}
		}
		else{
			//Error => No hi ha una connexi oberta amb la base de dades.
			throw new MyFrameUOCException(DataBaseMsg.NO_CONNEXIO_OBERTA);
		}
	}
	
	/**
	 * Permet verificar si existeix una taula en la base de dades.
	 * 
	 * @param table - Nom de la taula que es vol verificar si existeix en
	 * la base de dades.
	 * 
	 * @return <code>True</code> si la taula existeix en la base de dade. <code>False</code>
	 * en qualsevol altre cas.
	 * 
	 * @throws MyFrameUOCException Si no es pot verificar si existeix una taula en la base de dades.
	 */
	public abstract boolean existsTable(String table) throws MyFrameUOCException;
	
	/**
	 * Permet obtenir la connexi amb la base de dades.
	 * 
	 * @return Un <code>java.sql.Connection</code> que cont la connexi amb la base de dades.
	 */
	public java.sql.Connection getConnection(){
		return (this.connection);
	}
	
	/**
	 * Permet obtenir el mapeig de tipus que li correspn a un tipus de Java determinat i amb
	 * unes propietats determinades (precisi, escala i mida).
	 * 
	 * @param typeJava - El tipus de dades en Java del qual es vol conixer el seu mapeig segons
	 * el sistema de gesti de bases de dades que s'utilitzi.
	 * @param precision - Precisi del tipus de dades numric.
	 * @param scale - Escala del tipus de dades numric.
	 * @param length - Mida en carcters o bytes del tipus de dades.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.database.TypeMap TypeMap}</code> que cont
	 * el mapeig del tipus de dades en Java segons el sistema de gesti de bases de dades 
	 * que s'utilitzi.
	 * 
	 * @throws MyFrameUOCException Si no es pot obtenir el tipus de dades de Java mapejat. 
	 */
	public TypeMap getTypeMap(String javaType, Integer precision, Integer scale, Integer length) throws MyFrameUOCException{
		TypeMap typeMap				= null;
		ArrayList<TypeMap> types	= null;
		boolean foundPrecision		= false;
		boolean foundScale			= false;
		boolean foundLength			= false;
		int elements				= 0;
		int i;
		Integer maxPrecision		= null;
		Integer maxScale			= null;
		Integer minScale			= null;
		Integer maxLength			= null;
		
		try{
			if (javaType != null && !"".equals(javaType)){
				if (this.typeMaps == null || this.typeMaps.size() == 0){
					//Error => No hi han tipus mapejats definits.
					throw new MyFrameUOCException(DataBaseMsg.NO_TIPUS_MAPEJATS);
				}
				if (!this.typeMaps.containsKey(javaType)){
					//Error => El tipus de dades de Java demanat no est mapejat.
					throw new MyFrameUOCException(DataBaseMsg.NO_EXISTEIX_TIPUS_JAVA);
				}
				//Validar els valors dels parmetres del tipus de dades.
				this.validateTypeParameters(javaType, precision, scale, length);
				types = this.typeMaps.get(javaType);
				elements = types.size();
				i = 0;
				while (i < elements && !foundPrecision && !foundScale && !foundLength){
					typeMap = types.get(i);
					maxPrecision = typeMap.getMaxPrecision();
					maxScale = typeMap.getMaxScale();
					minScale = typeMap.getMinScale();
					maxLength = typeMap.getMaxLength();
					
					if (precision != null && !"".equals(precision)){
						//La precisi est informada.
						if (maxPrecision != null && !"".equals(maxPrecision)){
							if (precision <= maxPrecision){
								foundPrecision = true;
							}
						}
					}
					else{
						//La precisi no est informada.
						foundPrecision = true;
					}
					
					
					if (scale != null && !"".equals(scale)){
						//L'escala est informada.
						if (maxScale != null && !"".equals(maxScale)){
							if (minScale != null && !"".equals(minScale)){
								if (scale <= maxScale && scale >= minScale){
									foundScale = true;
								}
							}
							else{
								if (scale <= maxScale){
									foundScale = true;
								}
							}
						}
						else if (minScale != null && !"".equals(minScale)){
							if (scale >= minScale){
								foundScale = true;
							}
						}
					}
					else{
						//L'escala no est informada.
						foundScale = true;
					}
					
					if (length != null && !"".equals(length)){
						//La mida est informada.
						if (maxLength != null && !"".equals(maxLength)){
							if (length <= maxLength){
								foundLength = true;
							}
						}
					}
					else{
						foundLength = true;
					}
					
					if (!(foundPrecision && foundScale && foundLength)){
						i++;
						foundPrecision = false;
						foundScale = false;
						foundLength = false;
					}
				}
				if (!(foundPrecision && foundScale && foundLength)){
					//Error => El tipus de dades que es demana no est mapejat.
					throw new MyFrameUOCException(DataBaseMsg.NO_EXISTEIX_TIPUS_JAVA);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'obtenci del mapeig del tipus de dades.
			throw new MyFrameUOCException(DataBaseMsg.GETTYPEMAP_ERROR, e.getMessage());
		}
		return (typeMap);
	}
	
	/*
	 * Valida els valors dels parmetres del tipus de dades.
	 *  
	 * @param javaType - Nom del tipus de dades en Java.
	 * @param precision - Precisi del tipus de dades numric.
	 * @param scale - Escala del tipus de dades numric.
	 * @param length - Mida en carcters o bytes del tipus de dades.
	 */
	private void validateTypeParameters(String javaType, Integer precision, Integer scale, Integer length) throws MyFrameUOCException{
		ArrayList<String> tipusNumerics		= null;
		ArrayList<String> tipusText			= null;
		
		tipusNumerics = new ArrayList<String>();
		tipusNumerics.add(JavaType.int_.getType());
		tipusNumerics.add(JavaType.Integer.getType());
		tipusNumerics.add(JavaType.float_.getType());
		tipusNumerics.add(JavaType.Float.getType());
		tipusNumerics.add(JavaType.double_.getType());
		tipusNumerics.add(JavaType.Double.getType());
		tipusNumerics.add(JavaType.BigDecimal.getType());
		tipusNumerics.add(JavaType.short_.getType());
		tipusNumerics.add(JavaType.Short.getType());
		tipusNumerics.add(JavaType.long_.getType());
		tipusNumerics.add(JavaType.Long.getType());
		tipusNumerics.add(JavaType.BigInteger.getType());
		
		tipusText = new ArrayList<String>();
		tipusText.add(JavaType.String.getType());
		tipusText.add(JavaType.char_array.getType());
		
		if (javaType != null && !"".equals(javaType)){
			if (!tipusNumerics.contains(javaType)){
				if (precision != null || scale != null){
					//Error => El tipus de dada no pot tenir ni precisi ni escala.
					throw new MyFrameUOCException(DataBaseMsg.DADA_AMB_PRECISIO_ESCALA);
				}
				if (!tipusText.contains(javaType)){
					if (length != null){
						//Error => El tipus de dada no pot tenir una mida.
						throw new MyFrameUOCException(DataBaseMsg.DADA_AMB_MIDA);
					}
				}
			}
			else{
				if (length != null){
					//Error => El tipus de dada no pot tenir una mida.
					throw new MyFrameUOCException(DataBaseMsg.DADA_AMB_MIDA);
				}
				if (JavaType.BigDecimal.getType().equals(javaType)){
					if ((precision == null && scale != null) || (precision != null && scale == null)){
						//Error => El tipus de dada ha de tenir precisi i escala.
						throw new MyFrameUOCException(DataBaseMsg.DADA_SENSE_PRECISIO_O_ESCALA);
					}
				}
				else if (JavaType.BigInteger.getType().equals(javaType)){
					if ((precision == null)){
						//Error => Un BigInteger ha de tenir indicat una precisi.
						throw new MyFrameUOCException(DataBaseMsg.BIGINTEGER_SENSE_PRECISIO);
					}
				}
			}
		}
	}

	/**
	 * Permet obtenir la definici de la restricci d'una columna obligatria.
	 * 
	 * @return Un <code>String</code> amb la definici de la restricci d'una columna obligatria.
	 */
	public String getNotNullConstraint() {
		return notNullConstraint;
	}

	/**
	 * Permet obtenir la definici de la restricci d'una columna nica.
	 * 
	 * @return Un <code>String</code> amb la definici de la restricci d'una columna nica.
	 */
	public String getUniqueConstraint() {
		return uniqueConstraint;
	}
	
	/**
	 * Tanca la connexi amb la base de dades.
	 * 
	 * @throws MyFrameUOCException Si no es pot tancar la connexi amb la base de dades. 
	 */
	public void close() throws MyFrameUOCException{
		try{
			if (this.connection != null){
				if (!this.connection.isClosed()){
					this.connection.close();
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error al tancar la connexi amb la base de dades.
			throw new MyFrameUOCException(DataBaseMsg.CLOSE_ERROR, e.getMessage());
		}
	}
}
