package com.uoc.myframeuoc.database;

import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import com.uoc.myframeuoc.environment.Configuration;
import com.uoc.myframeuoc.exceptions.DataBaseMsg;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.utils.MyFrameUOC;

/**
 * Classe que hereta de <code>{@link com.uoc.myframeuoc.database.Connection}</code> i
 * que serveix per a gestionar una connexi en MySQL.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public class ConnectionMySQL extends Connection {

	/**
	 *Constructor.
	 * 
	 * @param configuration - Un <code>{@link com.uoc.myframeuoc.environment.Configuration}</code>
	 * amb la configuraci definida en el fitxer de configuraci de MyFrameUOC.
	 * @throws MyFrameUOCException 
	 */
	public ConnectionMySQL(Configuration configuration) throws MyFrameUOCException {
		super();
		
		if (configuration == null){
			//Error => No es pot crear una connexi sense una configuraci.
			throw new MyFrameUOCException(DataBaseMsg.CONNEXIO_SENSE_CONFIGURACIO);
		}
		
		//Obtenir el mapeig dels tipus de dades per a MySQL.
		this.mapTypes();
		
		//Obrir una connexi amb la BD de MySQL.
		this.getConnection(configuration);	
		
		//Definir les restriccions que utilitza.
		this.notNullConstraint = "NOT NULL";
		this.uniqueConstraint = "UNIQUE";
	}
	
	/*
	 * Obt el mapeig dels tipus de dades en Java amb els tipus de dades en MySQL.
	 */
	private void mapTypes(){
		TypeMap typeMap										= null;
		HashMap<String, ArrayList<TypeMap>> typeMaps		= null;
		ArrayList<TypeMap> types							= null;
		
		typeMaps = new HashMap<String, ArrayList<TypeMap>>();
		
		//java.lang.String
		types = new ArrayList<TypeMap>();
		
		//java.lang.String si no s'indica mida.
		typeMap = new TypeMap(JavaType.String, "VARCHAR(255)");
		types.add(typeMap);
		
		//java.lang.String si 0 > mida <= 255
		typeMap = new TypeMap(JavaType.String, "VARCHAR(" + MyFrameUOC.PARAM_LENGTH + ")", 255);
		types.add(typeMap);
				
		//java.lang.String si 255 > mida <= 65535
		typeMap = new TypeMap(JavaType.String, "TEXT", 65535);
		types.add(typeMap);
		
		typeMaps.put(JavaType.String.getType(), types);
		
		//char.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.char_, "CHAR");
		types.add(typeMap);
		typeMaps.put(JavaType.char_.getType(), types);
		
		//char[] si 0 > mida <= 255
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.char_array, "CHAR(" + MyFrameUOC.PARAM_LENGTH + ")", 255);
		types.add(typeMap);
		typeMaps.put(JavaType.char_array.getType(), types);
		
		//java.lang.Character.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.Character, "CHAR");
		types.add(typeMap);
		typeMaps.put(JavaType.Character.getType(), types);
		
		//int.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.int_, "INTEGER");
		types.add(typeMap);
		typeMaps.put(JavaType.int_.getType(), types);
		
		//java.lang.Integer.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.Integer, "INTEGER");
		types.add(typeMap);
		typeMaps.put(JavaType.Integer.getType(), types);
		
		//float.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.float_, "FLOAT");
		types.add(typeMap);
		typeMaps.put(JavaType.float_.getType(), types);
		
		//java.lang.Float.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.Float, "FLOAT");
		types.add(typeMap);
		typeMaps.put(JavaType.Float.getType(), types);
		
		//double.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.double_, "DOUBLE");
		types.add(typeMap);
		typeMaps.put(JavaType.double_.getType(), types);
		
		//java.lang.Double.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.Double, "DOUBLE");
		types.add(typeMap);
		typeMaps.put(JavaType.Double.getType(), types);
		
		//java.math.BigDecimal.
		types = new ArrayList<TypeMap>();
		
		//java.math.BigDecimal amb precisi i escala.
		typeMap = new TypeMap(JavaType.BigDecimal, "NUMERIC(" + MyFrameUOC.PARAM_PRECISION + ", " + MyFrameUOC.PARAM_SCALE + ")", 64, 30, null, null);
		types.add(typeMap);
		
		typeMaps.put(JavaType.BigDecimal.getType(), types);
		
		//java.sql.Date.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.Date, "TIMESTAMP");
		types.add(typeMap);
		typeMaps.put(JavaType.Date.getType(), types);
		
		//byte.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.byte_, "TINYINT");
		types.add(typeMap);
		typeMaps.put(JavaType.byte_.getType(), types);
		
		//java.lang.Byte.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.Byte, "TINYINT");
		types.add(typeMap);
		typeMaps.put(JavaType.Byte.getType(), types);
		
		//short.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.short_, "SMALLINT");
		types.add(typeMap);
		typeMaps.put(JavaType.short_.getType(), types);
		
		//java.lang.Short.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.Short, "SMALLINT");
		types.add(typeMap);
		typeMaps.put(JavaType.Short.getType(), types);
		
		//long.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.long_, "BIGINT");
		types.add(typeMap);
		typeMaps.put(JavaType.long_.getType(), types);
		
		//java.lang.Long.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.Long, "BIGINT");
		types.add(typeMap);
		typeMaps.put(JavaType.Long.getType(), types);
		
		//boolean.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.boolean_, "TINYINT(1)");
		types.add(typeMap);
		typeMaps.put(JavaType.boolean_.getType(), types);
		
		//java.lang.Boolean
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.Boolean, "TINYINT(1)");
		types.add(typeMap);
		typeMaps.put(JavaType.Boolean.getType(), types);
		
		//java.sql.Time.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.Time, "TIME");
		types.add(typeMap);
		typeMaps.put(JavaType.Time.getType(), types);
			
		//java.math.BigInteger si 1 > mida <= 65535.
		types = new ArrayList<TypeMap>();
		typeMap = new TypeMap(JavaType.BigInteger, "TEXT(" + MyFrameUOC.PARAM_PRECISION + ")", 65535, null, null, null);
		types.add(typeMap);
		typeMaps.put(JavaType.BigInteger.getType(), types);
		
		this.typeMaps = typeMaps;
	}
	
	/*
	 * Crea una connexi amb el SGBD de MySQL.
	 */
	private void getConnection(Configuration configuration) throws MyFrameUOCException{
		java.sql.Connection connection		= null;
		
		if (configuration == null){
			//Error => Si no existeix una configuraci no es pot obrir una connexi.
			throw new MyFrameUOCException(DataBaseMsg.NO_OBRIR_CONNEXIO_CONFIG);
		}
		try {
			Class.forName("com.mysql.jdbc.Driver");
		} 
		catch (ClassNotFoundException e) {
			//Error => No s'ha pogut registrar la classe 'com.mysql.jdbc.Driver'.
			throw new MyFrameUOCException(DataBaseMsg.NO_REGISTRE_MYSQL, e.getMessage());
		}

		try {
			connection = DriverManager.getConnection(configuration.getUrl(), configuration.getUser(), configuration.getPassword());
			this.connection = connection;
		} 
		catch (SQLException e) {
			//Error => No s'ha pogut obrir una connexi amb MySQL.
			throw new MyFrameUOCException(DataBaseMsg.NO_OBRIR_CONNEXIO, e.getMessage());
		}
	}

	/**
	 * Permet conixer si existeix una taula en la base de dades.
	 * 
	 * @return <code>True</code> si eixsteixl a taula. <code>False</code> en qualsevol altre cas.
	 */
	@Override
	public boolean existsTable( String table) throws MyFrameUOCException {
		boolean res						= false;
		String sql						= null;
		ResultSet rs					= null;
		Statement stm					= null;
		java.sql.Connection connection	= null;
		
		try{
			if (table != null && !"".equals(table)){
				connection = this.getConnection();
				if (connection != null){
					stm = connection.createStatement();
					//Tamb es pot fer amb: sql = "SELECT COUNT(*) FROM information_schema.tables WHERE table_name = '" + table + "'";
					sql = "SHOW TABLES LIKE '" + table + "'";
					rs = stm.executeQuery(sql);
					if (rs != null && rs.next()){
						res = true;
					}
				}
				else{
					//Error => No hi ha una connexi oberta amb la base de dades.
					throw new MyFrameUOCException(DataBaseMsg.NO_CONNEXIO_OBERTA);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en verificar l'existncia de la taula.
			throw new MyFrameUOCException(DataBaseMsg.ERR_EXISTEIX_TAULA, e.getMessage());
		}
		finally{
			if (stm != null){
				try {
					stm.close();
				} 
				catch (SQLException e) {
				}
			}
			if (rs != null){
				try {
					rs.close();
				} 
				catch (SQLException e) {
				}
			}
		}
		return (res);
	}	
}
