package com.uoc.myframeuoc.database;

import com.uoc.myframeuoc.exceptions.DataBaseMsg;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.utils.MyFrameUOC;

/**
 * Classe que representa el mapeig entre un tipus de dades en Java i un tipus de dades en un sistema de gesti de bases de dades.
 * 
 * @author Jordi Garcia Casanova
 *
 */
public class TypeMap {

	//Tipus de dades en Java.
	private JavaType javaType	= null;
	
	//Tipus de dades mapejat segons el SGBD utilitzat.
	private String mappingType	= null;
	
	//Mxima precisi permesa pel SGBD.
	Integer maxPrecision		= null;
	
	//Mxima escala permesa pel SGBD.
	Integer maxScale			= null;
	
	//Mnima escala permes pel SGBD.
	Integer minScale			= null;
	
	//Mxim nmero de carcters o bytes permessos pel SGBD.
	Integer maxLength			= null;
		
	/**
	 * Constructor.
	 * 
	 * @param javaType - Un <code>{@link com.uoc.myframeuoc.database.JavaType JavaType}</code>
	 * amb el tipus de dades de Java que es mapeja.
	 * @param mappingType - Tipus de dades mapejat segons el sistema de gesti de base de dades.
	 * @param maxPrecision - Precisi mxima permesa pel sistema de gesti de base de dades.
	 * @param maxScale - Escala mxima (dgits decimals) permesa pel sistema de gesti de base de dades.
	 * @param minScale - Escala mnima (dgits decimals) permesa pel sistema de gesti de base de dades.
	 * @param maxLength - Longitud mxima (en bytes o carcters) permesa pel sistema de gesti de base de dades.
	 */
	public TypeMap(JavaType javaType, String mappingType, Integer maxPrecision, Integer maxScale, Integer minScale, Integer maxLength) {
		this.javaType = javaType;
		this.mappingType = mappingType;
		this.maxPrecision = maxPrecision;
		this.maxScale = maxScale;
		this.minScale = minScale;
		this.maxLength = maxLength;
	}

	/**
	 * Constructor.
	 * 
	 * @param javaType - Un <code>{@link com.uoc.myframeuoc.database.JavaType JavaType}</code>
	 * amb el tipus de dades de Java que es mapeja.
	 * @param mappingType - Tipus de dades mapejat segons el sistema de gesti de base de dades.
	 */
	public TypeMap(JavaType javaType, String mappingType){
		this.javaType = javaType;
		this.mappingType = mappingType;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param javaType - Un <code>{@link com.uoc.myframeuoc.database.JavaType JavaType}</code>
	 * amb el tipus de dades de Java que es mapeja.
	 * @param mappingType - Tipus de dades mapejat segons el sistema de gesti de base de dades.
	 * @param maxLength - Longitud mxima (en bytes o carcters) permesa pel sistema de gesti de base de dades.
	 */
	public TypeMap(JavaType javaType, String mappingType, Integer maxLength){
		this.javaType = javaType;
		this.mappingType = mappingType;
		this.maxLength = maxLength;
	}
	
	/**
	 * Permet obtenir el tipus de dades de Java que representa el tipus mapejat.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.database.JavaType JavaTypes}</code>
	 * amb el tipus de dades de Java que es mapeja.
	 */
	public JavaType getJavaType() {
		return javaType;
	}

	/**
	 * Permet obtenir el mapeig segons la base de dades.
	 * 
	 * @return Un <code>String</code> amb el tipus de dades mapejat.
	 */
	public String getMappingType() {
		return mappingType;
	}

	/**
	 * Permte obtenir el mapeig segons la base de dades i tenint en compte la
	 * precisi, l'escala i la mida indicadades.
	 *  
	 * @param precision - Precisi informada.
	 * @param scale - Escala informada.
	 * @param length - Mida informada.
	 * 
	 * @return Un <code>String</code> amb el tipus de dades mapejat.
	 * 
	 * @throws MyFrameUOCException Si no es pot mapejar el tipus de dades. 
	 */
	public String getMappingType (Integer precision, Integer scale, Integer length) throws MyFrameUOCException{
		String mappingType		= null;
				
		mappingType = this.mappingType;
		if (mappingType != null){
			if (mappingType.contains(MyFrameUOC.PARAM_LENGTH) && length == null){
				//Error => No s'ha informat la mida de l'atribut.
				throw new MyFrameUOCException(DataBaseMsg.MIDA_NO_INDICADA);
			}
			if (mappingType.contains(MyFrameUOC.PARAM_PRECISION) && precision == null){
				//Error => No s'ha informat la precisi de l'atribut.
				throw new MyFrameUOCException(DataBaseMsg.PRECISIO_NO_INDICADA);
			}
			if (mappingType.contains(MyFrameUOC.PARAM_SCALE) && scale == null){
				//Error => No s'ha informat l'escala de l'atribut.
				throw new MyFrameUOCException(DataBaseMsg.ESCALA_NO_INDICADA);
			}
			mappingType = mappingType.replace(MyFrameUOC.PARAM_LENGTH, String.valueOf(length));
			mappingType = mappingType.replace(MyFrameUOC.PARAM_PRECISION, String.valueOf(precision));
			mappingType = mappingType.replace(MyFrameUOC.PARAM_SCALE, String.valueOf(scale));
		}
		return (mappingType);
	}
	
	/**
	 * Permet obtenir la precisi mxima (nmero de dgits d'un nmero) 
	 * que permet el sistema de gesti de base de dades.
	 * 
	 * @return Un <code>int</code> amb la precisi mxima permesa pel sistema
	 * de gesti de base de dades.
	 */
	public Integer getMaxPrecision(){
		return (this.maxPrecision);
	}
	
	/**
	 * Permet obtenr l'escala mxima (nmero de decimals d'un nmero)
	 * que permet el sistema de gesti de base de dades.
	 * 
	 * @return Un <code>int</code> amb l'escala mxima permes pel sistema
	 * de gesti de base de dades.
	 */
	public Integer getMaxScale(){
		return (this.maxScale);
	}
	
	/**
	 * Permet obtenir el nmero mxim de carcters o de bytes que permet
	 * el sistema de gesti de base de dades.
	 * 
	 * @return Un <code>int</code> amb el nmero mxim de carcters o de
	 * bytes que permet el sistema de gesti de base de dades.
	 */
	public Integer getMaxLength(){
		return (this.maxLength);
	}
	
	/**
	 * Permet obtenir l'escala mnima que permet el sistema de gesti de base
	 * de dades.
	 * 
	 * @return Un <code>int</code> amb l'escala mnima que permet el sistema
	 * de gesti de base de dades. 
	 */
	public Integer getMinScale(){
		return (this.minScale);
	}
}
