package com.uoc.myframeuoc.environment;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;

import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;

import com.uoc.myframeuoc.exceptions.EnvironmentMsg;
import com.uoc.myframeuoc.exceptions.GeneralMsg;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.utils.MyFrameUOC;

/**
 * Class que s'encarrega de gestionar la configuraci de l'entorn
 * de MyFrameUOC.
 * 
 * @author Jordi Garcia Casanova
 *
 */
public class Configuration {

	//Tipus de sistema de gesti de bases de dades utilitzat. 
	private final SGBDType SGBD;
	
	//Direcci url per a connecatr-se a la base de dades.
	private final String url;
	
	//Usuari per a connectar-se a la base de dades.
	private final String user;
	
	//Password utilitzat per a connectar-se a la base de dades.
	private final String password;
	
	/**
	 * Constructor. 
	 * 
	 * @param file - Nom del fitxer XML que cont la configuraci.
	 * @throws IOException 
	 * @throws JDOMException 
	 * @throws FileNotFoundException 
	 * @throws MyFrameUOCException 
	 */
	public Configuration(String file) throws MyFrameUOCException {
		SAXBuilder saxBuilder 	= null;
		Document document		= null;
		Element element			= null;
		List<Element> nodes		= null;
		String nameNode			= null;
		String value			= null;
		SGBDType[] sgbdTypes	= null;
		SGBDType sgbd			= null;
		boolean trobat;
		int i;
		
		
		saxBuilder = new SAXBuilder();
		
		try {
			//document = saxBuilder.build(new FileInputStream(MyFrameUOC.XML_DIRECTORY + MyFrameUOC.XML_FILE_NAME));
			document = saxBuilder.build(new FileInputStream(file));
		} 
		catch (FileNotFoundException e) {
			//Error => No s'ha pogut obrir el fitxer XML de configuraci.
			throw new MyFrameUOCException(EnvironmentMsg.NO_OBRIR_FITXERCONFIG);
		} 
		catch (JDOMException e) {
			//Error => No s'ha pogut obrir el fitxer XML de configuraci.
			throw new MyFrameUOCException(EnvironmentMsg.NO_OBRIR_FITXERCONFIG, e.getMessage());
		} 
		catch (IOException e) {
			//Error => No s'ha pogut obrir el fitxer XML de configuraci.
			throw new MyFrameUOCException(EnvironmentMsg.NO_OBRIR_FITXERCONFIG, e.getMessage());
		}
		
		
		//Obtenir el node arrel.
		element = document.getRootElement();
		if (element != null){
			nameNode = element.getName();
		}
		else{
			//Error => No s'ha pogut obtenir el node arrel del fitxer XML de configuraci.
			throw new MyFrameUOCException(EnvironmentMsg.NO_NODE_MYFRAMEUOC);
		}
		
		//Verificar que el node arrel s <MyFrameUOC>.
		if (!MyFrameUOC.ROOT_NODE.equals(nameNode)){
			//Error => El node arrel s incorrecte.
			throw new MyFrameUOCException(EnvironmentMsg.NO_NODE_MYFRAMEUOC);
		}
		
		//Obtenir els nodes fills de <MyFrameUOC>.
		nodes = element.getChildren();
		if (nodes != null && !nodes.isEmpty()){
			if (nodes.size() != 1){
				//Error => Estructrua del fitxer de configuraci s incorrecta.
				throw new MyFrameUOCException(EnvironmentMsg.FITXERCONFIG_INCORRECTE);
			}
			element = nodes.get(0);
			if (!MyFrameUOC.PROPERTIES_NODE.equals(element.getName())){
				//Error => El node <properties> no s'ha trobat.
				throw new MyFrameUOCException(EnvironmentMsg.NO_NODE_PROPIETATS);
			}
			
			nodes = element.getChildren();
			if (nodes != null && !nodes.isEmpty()){
				if (nodes.size() != 4){
					//Error => Estructura del fitxer de configuraci s incorrecta.
					throw new MyFrameUOCException(EnvironmentMsg.FITXERCONFIG_INCORRECTE);
				}
				//Obtenir el node <SGBD>.
				element = nodes.get(0);
				if (!MyFrameUOC.SGBD_NODE.equals(element.getName())){
					//Error => El node <SGBD> no s'ha trobat.
					throw new MyFrameUOCException(EnvironmentMsg.NO_NODE_SGBD);
				}
				else{
					value = element.getValue();
					if (value == null){
						//Error => No s'ha informat un SGBD.
						throw new MyFrameUOCException(EnvironmentMsg.NO_INFORMAT_SGBD);
					}
					else{
						value = value.trim();
						if ("".equals(value)){
							//Error => No s'ha informat un SGBD.
							throw new MyFrameUOCException(EnvironmentMsg.NO_INFORMAT_SGBD);
						}
					}
					
					//Verificar que el SGBD informat s algn dels que MyFrameUOC permet.
					sgbdTypes = SGBDType.values();
					if (sgbdTypes != null){
						trobat = false;
						i = 0;
						while (!trobat && i < sgbdTypes.length){
							if (sgbdTypes[i].name().equals(value)){
								trobat = true;
								sgbd = sgbdTypes[i];
							}
							else{
								i++;
							}
						}
						if (!trobat){
							//Error => El SGBD que s'ha informat s incorrecte perqu no est suportat per MyFrameUOC.
							throw new MyFrameUOCException(EnvironmentMsg.SGBD_INCORRECTE);
						}
						else{
							this.SGBD = sgbd;
							//Obtenir el node <url>.
							element = nodes.get(1);
							if (!MyFrameUOC.URL_NODE.equals(element.getName())){
								//Error => El node <url> no s'ha trobat.
								throw new MyFrameUOCException(EnvironmentMsg.NO_NODE_URL);
							}
							value = element.getValue();
							if (value == null){
								//Error => No s'ha informat la url per accedir a la base de dades.
								throw new MyFrameUOCException(EnvironmentMsg.NO_INFORMAT_URL);
							}
							else{
								value = value.trim();
								if ("".equals(value)){
									//Error => No s'ha informat la url per accedir a la base de dades.
									throw new MyFrameUOCException(EnvironmentMsg.NO_INFORMAT_URL);
								}
							}
							this.url = value;
							
							//Obtenir el node <user>.
							element = nodes.get(2);
							if (!MyFrameUOC.USER_NODE.equals(element.getName())){
								//Error => El node <user> no s'ha trobat.
								throw new MyFrameUOCException(EnvironmentMsg.NO_NODE_USER);
							}
							value = element.getValue();
							if (value == null){
								//Error => No s'ha informat l'usuari.
								throw new MyFrameUOCException(EnvironmentMsg.NO_INFORMAT_USER);
							}
							else{
								value = value.trim();
								if ("".equals(value)){
									//Error => No s'ha informat l'usuari.
									throw new MyFrameUOCException(EnvironmentMsg.NO_INFORMAT_USER);
								}
							}
							this.user = value;
							
							//Obtenir el node <password>.
							element = nodes.get(3);
							if (!MyFrameUOC.PASSWORD_NODE.equals(element.getName())){
								//Error => El node <password> no s'ha trobat.
								throw new MyFrameUOCException(EnvironmentMsg.NO_NODE_PASSWORD);
							}
							this.password = element.getValue();;
						}
					}
					else{
						//Error => No hi ha definit cap tipus de SGBD (aix no hauria de passar mai perqu hi ha definits dos tipus de SGBD).
						throw new MyFrameUOCException(GeneralMsg.NO_SGBD);
					}
				}
			}
			else{
				//Error => el node SGBD no s'ha trobat.
				throw new MyFrameUOCException(EnvironmentMsg.NO_NODE_SGBD);
			}
		}
		else{
			//Error => el node properties no s'ha trobat.
			throw new MyFrameUOCException(EnvironmentMsg.NO_NODE_PROPIETATS);
		}
	}

	/**
	 * Permet obtenir el tipus de sistema de gesti de bases de dades definit en el
	 * fitxer de configuraci.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.environment.SGBDType SGBDTypes}</code>
	 * amb el sistema de gesti de bases de dades definit en el fitxer de configuraci.
	 */
	public SGBDType getSGBD() {
		return SGBD;
	}

	/**
	 * Permet obtenir la direcci url de la base de dades que s'ha definit en el
	 * fitxer de configuraci.
	 * 
	 * @return Un <code>String</code> amb la direcci url de la base de dades
	 * definida en el fitxer de configuraci.
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * Permet obtenir el nom d'usuari per accedir a la base de dades
	 * i que s'ha definit en el fitxer de configuraci.
	 * 
	 * @return Un <code>String</code> amb el nom d'usuari per accedir a la
	 * base de dades i que s'ha definit en el fitxer de configuraci.
	 */
	public String getUser() {
		return user;
	}

	/**
	 * Permet obtenr la contrasenya per accedir a la base de dades
	 * i que s'ha definit en el fitxer de configuraci.
	 * 
	 * @return Un <code>String</code> amb la contrasenya per accedir a la
	 * base de dades i que s'ha definit en el fitxer de configuraci.
	 */
	public String getPassword() {
		return password;
	}
	
	
}
