package com.uoc.myframeuoc.environment;

import java.util.HashMap;
import com.uoc.myframeuoc.exceptions.EnvironmentMsg;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.mapping.RegisteredEntity;

/**
 * Classe que gestiona les entitats d'una sessi de MyFrameUOC.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public class EntityManager {
	
	//Llista d'entitats que ja estan registrades en memria.
	private HashMap<String, RegisteredEntity> entities		= null;
	
	/**
	 * Constructor.
	 */
	public EntityManager() {
	}
	
	/**
	 * Permet carregar i obtenir una entitat per a poder-la utilitzar en una sessi.
	 * 
	 * @param nameEntity - Nom de l'entitat que es vol carregar.
	 * @param session - Sessi de MyFrameUOC que utilitza l'usuari.
	 * @param createTables - <code>True</code> per a crear les taules i <code>False</code> per a no crerar-les.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.mapping.RegisteredEntity}</code> amb l'entitat que s'obt i 
	 * que est carregada a memria. 
	 * 
	 * @throws MyFrameUOCException Si no es pot registrar l'entitat. 
	 */
	public RegisteredEntity registerEntity(String nameEntity, Session session, boolean createTables) throws MyFrameUOCException{
		RegisteredEntity entity		= null;
		
		if (nameEntity == null || "".equals(nameEntity)){
			//Error => No s'ha indicat cap entitat per a carregar.
			throw new MyFrameUOCException(EnvironmentMsg.NO_NOM_ENTITAT);
		}
		
		if (session == null){
			//Error => No hi ha una sessi oberta.
			throw new MyFrameUOCException(EnvironmentMsg.NO_SESSIO_OBERTA);
		}
		
		if (this.entities != null && this.entities.size() > 0){
	
			entity = this.entities.get(nameEntity);

			if (entity == null){
				//L'entitat encara no ha estat registrada => cal registrar-la.
				entity = new RegisteredEntity(nameEntity, session);
				if (createTables){
					entity.createEntityTables(session);
				}
			}
		}
		else{
			entity = new RegisteredEntity(nameEntity, session);
			if (createTables){
				entity.createEntityTables(session);
			}
		}
		
		return (entity);
	}
	
	/**
	 * Afegeix una nova entitat com a entitat registrada.
	 * 
	 * @param entity - Un <code>{@link com.uoc.myframeuoc.mapping.RegisteredEntity Entity}</code> amb
	 * l'entitat que es vol afegir.
	 * 
	 * @throws MyFrameUOCException Si no es pot afegir l'entitat com a entitat registrada. 
	 */
	public void addEntity(RegisteredEntity entity) throws MyFrameUOCException{
		if (entity == null){
			//Error => No es pot registrar una entitat buda.
			throw new MyFrameUOCException(EnvironmentMsg.REGISTRE_ENITAT_BUIDA);
		}
		
		if (this.entities == null){
			this.entities = new HashMap<String, RegisteredEntity>();
		}
		this.entities.put(entity.getName(), entity);
	}
	
	/**
	 * Elimina una entitat de les entitats registrades.
	 * 
	 * @param entity - Un <code>{@link com.uoc.myframeuoc.mapping.RegisteredEntity Entity}</code> amb
	 * l'entitat que es vol esborrar. Si es passa un nul, no s'esborra res.
	 */
	public void dropEntity(RegisteredEntity entity){
		if (entity != null){
			if (this.entities != null){
				this.entities.remove(entity.getName());
			}
		}
	}
	
	/**
	 * Permet obtenir una entitat que estuigui registrada en memria. Si no est
	 * registrada en memria s'obt un objecte nul.
	 * 
	 * @param nameEntity - Nom de l'entitat que es vol obtenir.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.mapping.RegisteredEntity RegisteredEntity}</code>
	 * que cont l'entitat registrada. Un <code>null</code> si l'entitat no est registrada.
	 */
	public RegisteredEntity getEntity(String nameEntity){
		RegisteredEntity entity		= null;
		
		if (this.entities != null && this.entities.size() > 0){
			entity = this.entities.get(nameEntity);
		}
		return (entity);
		
	}
}
