package com.uoc.myframeuoc.environment;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import com.uoc.myframeuoc.database.Connection;
import com.uoc.myframeuoc.database.ConnectionMySQL;
import com.uoc.myframeuoc.database.ConnectionOracle;
import com.uoc.myframeuoc.exceptions.EnvironmentMsg;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.mapping.ColumnMap;
import com.uoc.myframeuoc.mapping.PrimaryKeyMap;
import com.uoc.myframeuoc.mapping.RegisteredEntity;
import com.uoc.myframeuoc.querymanagement.Restriction;
import com.uoc.myframeuoc.querymanagement.Select;

/**
 * Classe que gestiona una sessi de MyFrameUOC. Permet que l'usuari
 * interactu amb el framework de persistncia.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public class Session {

	//Connexi amb la base de dades.
	private final Connection connection;
	
	//Indicador de transacci activa.
	private boolean inTransaction				= false;
	
	//Configuraci utilitzada en la sessi.
	private final Configuration configuration;
	
	//Gestor d'entitats utilitzat en la sessi.
	private final EntityManager entityManager;
	
	/*
	 * Constructor. Es fa privat perqu l'usuari no el pugui cridar.
	 * Per a obtenir una sessi, l'usuari ha d'utilitzar el mtode
	 * 'getSession(String)'.
	 * 
	 * @param file - Nom del fitxer XML que cont la configuraci.
	 */
	private Session(String file) throws MyFrameUOCException {
		
		//Obtenir la configuraci que s'utilitzar en la sessi.
		this.configuration = new Configuration(file);
		
		if (this.configuration == null){
			//Error => La sessi no t una configuraci.
			throw new MyFrameUOCException(EnvironmentMsg.SESSIO_SENSE_CONFIGURACIO);
		}
		
		//Obtenir la connexi amb la base de dades que s'utilitzar en la sessi.
		this.connection = createConnection(this);
		
		if (this.connection == null){
			//Error => La sessi no t una connexi oberta amb la base de dades.
			throw new MyFrameUOCException(EnvironmentMsg.SESSIO_SENSE_CONNEXIO);
		}
		
		//Inicialitzar el gestor d'entitats que s'utilitzar en la sessi.
		this.entityManager = new EntityManager();
		
		if (this.entityManager == null){
			//Error => La sessi no t un gestor d'entitats.
			throw new MyFrameUOCException(EnvironmentMsg.SESSIO_SENSE_GESTOR_ENTITATS);
		}
	}

	/**
	 * Permet obtenir una sessi de MyFrameUOC que l'usuari podr
	 * utilitzar per a treballar amb el framework de persistncia. 
	 * 
	 * @param file - Nom del fitxer XML que cont la configuraci.
	 * 
	 * @return Un <code>com.uoc.myframeuoc.environment.Session</code> amb
	 * la sessi de MyFrameUOC que l'usuari podr utilitzar per a treballar
	 * amb el framework de persistncia.
	 * 
	 * @throws MyFrameUOCException Si no es pot obtenir una sessi de MyFrameUOC.
	 */
	public static Session getSession(String file) throws MyFrameUOCException{
		Session session		= null;
		
		if (file == null || "".equals(file)){
			//Error => No s'ha indicat un fitxer.
			throw new MyFrameUOCException(EnvironmentMsg.FITXER_CONFIGURACIO_SENSE_INFORMAR);
		}
		
		session = new Session(file);
		
		return (session);
	}
	
	/*
	 * Crea una connexi segns el sistema de gesti de base de dades que hi hagi definit
	 * en la configuraci de la sessi.
	 * 
	 * @param session - Sessi de MyFrameUOC sobre la que est definida la configuraci.
	 */
	private static Connection createConnection(Session session) throws MyFrameUOCException{
		String SGBD					= null;
		Connection connection		= null;
		
		if (session == null){
			//Error => No s'ha pogut crear una connexi amb la base de dades perqu la sessi no est incialitzada.
			throw new MyFrameUOCException(EnvironmentMsg.NO_SESSION_INICIALITZADA);
		}
		SGBD = session.configuration.getSGBD().name();
		
		if (SGBDType.Oracle.name().equals(SGBD)){
			connection = new ConnectionOracle(session.configuration);
		}
		else if (SGBDType.MySQL.name().equals(SGBD)){
			connection = new ConnectionMySQL(session.configuration);
		}
		return (connection);
	}
	
	/**
	 * Permet inicialitzar una transacci.
	 * 
	 * @throws MyFrameUOCException Si no es pot inicialitzar una transacci.
	 */
	public void beginTransaction() throws MyFrameUOCException{
		
		if (this.inTransaction){
			//Error => Ja hi ha una transacci inicialitzada.
			throw new MyFrameUOCException(EnvironmentMsg.TRANSACCIO_ACTIVA);
		}
		this.connection.beginTransaction();
		
		try {
			if (!this.connection.getConnection().getAutoCommit()){
				this.inTransaction = true;
			}
		} 
		catch (SQLException e) {
			//Error => No s'ha pogut iniciar la transacci.
			throw new MyFrameUOCException(EnvironmentMsg.NO_INICIAR_TRANSACCIO, e.getMessage());
		}
	}

	/**
	 * Permet confirmar la transacci que estigui activa, si n'hi ha.
	 * 
	 * @throws MyFrameUOCException Si no es pot confirmar la transacci activa. 
	 */
	public void commit() throws MyFrameUOCException{
		try{
			if (!this.inTransaction){
				//Error => No hi ha una transacci activa.
				throw new MyFrameUOCException(EnvironmentMsg.TRANSACCI_NO_ACTIVA);
			}
			this.connection.commit();
			this.inTransaction = false;
		}
		catch (Exception e){
			//Error => S'ha produt un erro en confirmar una transacci.
			throw new MyFrameUOCException(EnvironmentMsg.COMMIT_ERROR, e.getMessage());
		}
		
	}
	
	/**
	 * Permet cancellar la transacci que estuigui activa, si n'hi ha.
	 * 
	 * @throws MyFrameUOCException Si no es pot cancellar la transacci activa.
	 */
	public void rollback() throws MyFrameUOCException{
		try{
			if (!this.inTransaction){
				//Error => No hi ha una transacci activa.
				throw new MyFrameUOCException(EnvironmentMsg.TRANSACCI_NO_ACTIVA);
			}
			this.connection.rollback();
			this.inTransaction = false;
		}
		catch (Exception e){
			//Error => S'ha produt un error en la cancellaci d'una transacci.
			throw new MyFrameUOCException(EnvironmentMsg.ROLLBACK_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Permet emmagatzemar un objecte d'una classe en la base de dades. Si 
	 * l'objecte ja existeix en la base de dades, llavors es retornar una excepci
	 * de tipus <code>{@link com.uoc.myframeuoc.exceptions.TypeException#ENVIRONMENT TypeException.ENVIRONMENT}</code>
	 * i amb codi 20.
	 * 
	 * @param obj - Objecte que s'emmagatzemar en la base de dades.
	 * 
	 * @throws MyFrameUOCException Si no es pot inserir l'objecte en la base de dades.
	 */
	public void insert(Object obj) throws MyFrameUOCException{
		RegisteredEntity entity		= null;
		
		try{
			if (obj == null){
				//Error => No es pot inserir un objecte nul en la base de dades.
				throw new MyFrameUOCException(EnvironmentMsg.INSERT_SENSE_OBJECTE);
			}
			entity = this.entityManager.registerEntity(obj.getClass().getCanonicalName(), this, true);
			if (entity == null){
				//Error => No s'ha pogut obtenir l'entitat de l'objecte.
				throw new MyFrameUOCException(EnvironmentMsg.ENTITAT_NO_TROBADA);
			}
			entity.insert(this, obj);
		}
		catch (Exception e){
			//Error => S'ha produt un error en la inserci d'un objecte en la BD.
			throw new MyFrameUOCException(EnvironmentMsg.INSERT_OBJECT_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Permet llegir una objecte de la base de dades.
	 * 
	 * @param nameEntity - Nom de l'entitat de l'objecte que es vol recuperar de la base de dades.
	 * @param values - Valors per a cadascuna de les claus primries de l'objecte que es vol recuperar. 
	 * L'ordre d'aquests valors ha de ser el mateix que el d'aparici o de definici de les claus 
	 * primries en l'entitat.
	 * 
	 * @return Un <code>Object</code> que cont la instncia de l'entitat recuperada
	 * de la base de dades.
	 * 
	 * @throws MyFrameUOCException Si no es pot llegir l'objecte de la base de dades. 
	 */
	public Object read(String nameEntity, ArrayList<Object> values) throws MyFrameUOCException{
		Object res					= null;
		RegisteredEntity entity		= null;
		
		try{
			entity = this.entityManager.registerEntity(nameEntity, this, true);
			if (entity == null){
				//Error => No s'ha pogut obtenir l'entitat.
				throw new MyFrameUOCException(EnvironmentMsg.ENTITAT_NO_TROBADA);
			}
			res = entity.read(this, values);
		}
		catch (Exception e){
			//Error => S'ha produt un error en la lectura d'un objecte de la BD.
			throw new MyFrameUOCException(EnvironmentMsg.READ_OBJECT_ERROR, e.getMessage());
		}
		return (res);
	}
	
	/**
	 * Permet acutalitzar un objecte en la base de dades.
	 * 
	 * @param obj - Objecte que cont els valors que cal passar a la base de dades.
	 * 
	 * @throws MyFrameUOCException Si no es pot actualitzar l'objecte en la base de dades. 
	 * 
	 */
	public void update(Object obj) throws MyFrameUOCException{
		RegisteredEntity entity		= null;

		try{
			if (obj != null){
				entity = this.entityManager.registerEntity(obj.getClass().getCanonicalName(), this, true);
				if (entity == null){
					//Error => No s'ha pogut obtenir l'entitat.
					throw new MyFrameUOCException(EnvironmentMsg.ENTITAT_NO_TROBADA);
				}
				entity.update(this, obj);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'actualitzaci d'un objecte en la base de dades.
			throw new MyFrameUOCException(EnvironmentMsg.UPDATE_OBJECT_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Permet esborrar un objecte de la base de dades.
	 * 
	 * @param obj - Objecte de l'entitat que es vol esborrar.
	 * 
	 * @throws MyFrameUOCException Si no es pot esborrar l'objecte de la base de dades. 
	 */
	public void delete(Object obj) throws MyFrameUOCException{
		RegisteredEntity entity		= null;
		
		try{
			if (obj != null){
				entity = this.entityManager.registerEntity(obj.getClass().getCanonicalName(), this, true);
				if (entity == null){
					//Error => No s'ha pogut obtenir l'entitat.
					throw new MyFrameUOCException(EnvironmentMsg.ENTITAT_NO_TROBADA);
				}
				entity.delete(this, obj);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error al esborrar un objecte de la base de dades.
			throw new MyFrameUOCException(EnvironmentMsg.DELETE_OBJECT_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Permet executar una consulta UOCQL i obtenir un conjunt d'objectes de la base de dades.
	 * 
	 * @param queryUOCQL - Consulta UOCQL que es desitja executar.
	 * 
	 * @return Un <code>ArrayList</code> que cont els objectes que s'ha recuperat de la base de dades.
	 * 
	 * @throws MyFrameUOCException Si no es pot executar la consutla UOCQL. 
	 */
	public ArrayList<Object> executeUOCQL(String queryUOCQL) throws MyFrameUOCException{
		ArrayList<Object> list				= null;
		String query						= null;
		ArrayList<Object> pk				= new ArrayList<Object>();
		ResultSet rs						= null;
		Statement stm						= null;
		java.sql.Connection connection		= null;
		ArrayList<Object> convert			= null;
		RegisteredEntity entity				= null;
		ArrayList<PrimaryKeyMap> pkms		= null;
		Object readObj						= null;
		
		try{
			convert = this.convertUOCQL(queryUOCQL);
			entity = (RegisteredEntity) convert.get(0);
			query = String.valueOf(convert.get(1));
			connection = this.connection.getConnection();
			stm = connection.createStatement();
			rs = stm.executeQuery(query);
			if (rs != null){
				list = new ArrayList<Object>();
				if (!entity.hasPrimaryKeyMaps()){
					//Error => L'entitat no t claus primries.
					throw new MyFrameUOCException(EnvironmentMsg.EXECUTEUOCQL_ENTITAT_SENSE_PK);
				}
				pkms = entity.getPrimaryKeyMaps();
				while (rs.next()){
					pk.clear();
					for (PrimaryKeyMap pkm: pkms){
						pk.add(rs.getObject(pkm.getField()));
					}
					readObj = entity.read(this, pk);
					list.add(readObj);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en executar la consulta UOCQL.
			throw new MyFrameUOCException(EnvironmentMsg.EXECUTEUOCQL_ERROR, e.getMessage());
		}

		return (list);
	}
	
	/*
	 * Tradueix una consulta UOCQL en una consulta SQL.
	 * 
	 * @param queryUOCQL - Consulta UOCQL.
	 * 
	 * @return Un AarrayList que cont l'entitat de la consulta i la consulta.
	 */
	private ArrayList<Object> convertUOCQL(String queryUOCQL) throws MyFrameUOCException{
		String query						= null;
		RegisteredEntity entity				= null;
		String nameEntity					= null;
		String[] where						= null;
		String[] from						= null;
		ArrayList<String[]> conditions		= null;
		Select select						= null;
		String[] condition					= null;
		int elements;
		Restriction restriction				= null;
		ArrayList<Object> res				= new ArrayList<Object>();
		
		try{
			if (queryUOCQL != null && !"".equals(queryUOCQL)){
				where = queryUOCQL.split("WHERE");
				if (where != null && where.length == 2){
					from = where[0].split("FROM");
					if (from != null && from.length == 2){
						nameEntity = from[1].trim();
						entity = this.getEntityManager().registerEntity(nameEntity, this, true);
						res.add(entity);
						conditions = this.getUOCQLConditions(where[1]);
						conditions = this.convertToFieldsUOCQLConditions(conditions, entity);
						select = entity.getPreparedSqlRead();
						restriction = select.getRestriction();
						select.removeRrestriction();
						if (conditions != null && conditions.size() > 0){
							condition = conditions.get(0);
							query = select.getQuery() + " WHERE " + condition[1];
							elements = conditions.size();
							for (int i = 1; i < elements; i++){
								condition = conditions.get(i);
								query += " " + condition[0] + " " + condition[1]; 
							}
						}
						res.add(query);
						select.addRestriction(restriction);
					}
					else{
						//Error => La consulta UOCQL est mal construda.
						throw new MyFrameUOCException(EnvironmentMsg.CONVERTUOCQL_UOCQL_INCORRECTA);
					}
				}
				else{
					//Error => La consulta UOCQL est mal construda.
					throw new MyFrameUOCException(EnvironmentMsg.CONVERTUOCQL_UOCQL_INCORRECTA);
				}
			}
			else{
				//Error => La consulta UOCQL est buida.
				throw new MyFrameUOCException(EnvironmentMsg.CONVERTUOCQL_UOCQL_BUIDA);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la conversi de la sentncia UOCQL en una sentncia SQL.
			throw new MyFrameUOCException(EnvironmentMsg.CONVERTUOCQL_ERROR, e.getMessage());
		}
		return (res);
	}
	
	/*
	 * Converteix els atributs de la consulta UOCQL en els camps de la taula de l'entitat.
	 * 
	 * @param conditions - Condicions de la consulta UOCQL.
	 * @param entity - Entitat que participa en la consulta UOCQL.
	 */
	private ArrayList<String[]> convertToFieldsUOCQLConditions(ArrayList<String[]> conditions, RegisteredEntity entity) throws MyFrameUOCException{
		String[] parts					= null;
		ArrayList<String[]> convert		= new ArrayList<String[]>();
		String[] convertPair			= null;
		
		if (entity != null){
			if (conditions != null && conditions.size() > 0){
				for (String[] pair: conditions){
					convertPair = new String[2];
					convertPair[0] = pair[0];
					parts = pair[1].split(">=");
					if (parts != null && parts.length > 1){
						convertPair[1] = this.convertToField(parts[0].trim(), entity) + " >= " + parts[1];
						convert.add(convertPair);
					}
					else{
						parts = pair[1].split("<=");
						if (parts != null && parts.length > 1){
							convertPair[1] = this.convertToField(parts[0].trim(), entity) + " <= " + parts[1];
							convert.add(convertPair);
						}
						else{
							parts = pair[1].split("<>");
							if (parts != null && parts.length > 1){
								convertPair[1] = this.convertToField(parts[0].trim(), entity) + " <> " + parts[1];
								convert.add(convertPair);
							}
							else{
								parts = pair[1].split(">");
								if (parts != null && parts.length > 1){
									convertPair[1] = this.convertToField(parts[0].trim(), entity) + " > " + parts[1];
									convert.add(convertPair);
								}
								else{
									parts = pair[1].split("<");
									if (parts != null && parts.length > 1){
										convertPair[1] = this.convertToField(parts[0].trim(), entity) + " < " + parts[1];
										convert.add(convertPair);
									}
									else{
										parts = pair[1].split("=");
										if (parts != null && parts.length > 1){
											convertPair[1] = this.convertToField(parts[0].trim(), entity) + " = " + parts[1];
											convert.add(convertPair);
										}
										else{
											parts = pair[1].split("IS NOT NULL");
											if (parts != null && parts.length > 0){
												convertPair[1] = this.convertToField(parts[0].trim(), entity) + " IS NOT NULL";
												convert.add(convertPair);
											}
											else{
												parts = pair[1].toUpperCase().split("IS NULL");
												if (parts != null && parts.length > 0){
													convertPair[1] = this.convertToField(parts[0].trim(), entity) + " IS NULL";
													convert.add(convertPair);
												}
												else{
													//Error => L'operador relacional utilitzat no est perms.
													throw new MyFrameUOCException(EnvironmentMsg.CONVERTUOCQL_OPERADOR_RELACIONAL_ERRONI);
												}
											}
										}
										
									}
								}
							}
						}
					}
				}
			}
		}
		return (convert);
	}
	
	/*
	 * Mapeja un atribut amb el camp que li correspon en la taula de la base de dades.
	 * 
	 * @param property - Nom de l'atribut.
	 * @param entity - Entitat a la que pertany l'atribut.
	 * 
	 * @return Un String amb el nom del camp de la taula que li correspon a l'atribut.
	 */
	private String convertToField(String property, RegisteredEntity entity) throws MyFrameUOCException{
		int elements;
		ArrayList<PrimaryKeyMap> pkms	= null;
		ArrayList<ColumnMap> cms		= null;
		int i;
		PrimaryKeyMap pkm				= null;
		ColumnMap cm					= null;
		boolean found;
		String field					= null;
		
		if (property != null && !"".equals(property) && entity != null){
			if (entity.hasPrimaryKeyMaps()){
				pkms = entity.getPrimaryKeyMaps();
				elements = pkms.size();
				found = false;
				i = 0;
				while (!found && i < elements){
					pkm = pkms.get(i);
					if (pkm.getProperty().equals(property)){
						found = true;
						field = pkm.getField();
					}
					else{
						i++;
					}
				}
				if (!found){
					if (entity.hasColumnMaps()){
						cms = entity.getColumnMaps();
						elements = cms.size();
						found = false;
						i = 0;
						while (!found && i < elements){
							cm = cms.get(i);
							if (cm.getProperty().equals(property)){
								found = true;
								field = cm.getField();
							}
							else{
								i++;
							}
						}
						if (!found){
							//Error => L'atribut indicat en la sentncia UOCQL no existeix.
							throw new MyFrameUOCException(EnvironmentMsg.CONVERTUOCQL_ATRIBUT_NO_EXISTEIX);
						}
					}
				}
			}
		}
		return (field);
	}
	
	/*
	 * Permet obtenir les condicions d'una restricci d'una consulta UOCQL.
	 * 
	 * @param restriction - Restricci de la consulta UOCQL de la qual s'obtindran les condicions.
	 */
	private ArrayList<String[]> getUOCQLConditions(String restriction){
		ArrayList<String[]> conditions		= new ArrayList<String[]>();
		String[] parts						= null;
		String condition					= null;
		String[] pair;
		
		restriction = restriction.trim();
		if (restriction != null){
			parts = restriction.split(" ");
			if (parts != null && parts.length > 1){
				condition = "";
				pair = new String[2];
				pair[0] = "";
				for (String element: parts){
					if ("AND".equals(element.toUpperCase())){
						pair[1] = condition;
						conditions.add(pair);
						condition = "";
						pair = new String[2];
						pair[0] = "AND";
					}
					else if ("OR".equals(element.toUpperCase())){
						pair[1] = condition;
						conditions.add(pair);
						condition = "";
						pair = new String[2];
						pair[0] = "OR";
					}
					else{
						condition += element + " ";
					}
				}
				pair[1] = condition;
				conditions.add(pair);
			}
		}
		return (conditions);
	}
	
	/**
	 * Permet obtenir la connexi oberta amb la base de dades que s'utilitza en la sessi.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.database.Connection Connection}</code> que
	 * cont la connexi oberta amb la base de dades que s'utilitza en la sessi.
	 */
	public Connection getConnection(){
		return (this.connection);
	}
	
	/**
	 * Permet obtenir la configuraci que s'utilitza en la sessi.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.environment.Configuration Configuration}</code>
	 * que cont la configuraci que s'utilitza en la sessi.
	 */
	public Configuration getConfiguration(){
		return (this.configuration);
	}
	
	/**
	 * Permet obtenir el gestor d'entitats que s'utilitza en la sessi.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.environment.EntityManager EntityManager}</code>
	 * que cont el gestor d'entitats que s'utilitza en la sessi.
	 */
	public EntityManager getEntityManager(){
		return (this.entityManager);
	}
	
	/**
	 * Permet esborrar la taula d'una entitat i totes les taules que estiguin relacionades.
	 * 
	 * @param nameEntity - Nom de l'entitat de la qual es vol esborrar la taula.
	 * 
	 * @throws MyFrameUOCException Si no es pot esborrar l'entitat. 
	 */
	public void drop(String nameEntity) throws MyFrameUOCException{
		RegisteredEntity entity		= null;
		
		try{
			entity = this.entityManager.registerEntity(nameEntity, this, true);
			if (entity != null){
				entity.drop(this, null);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en esborrar la taula d'una entitat de la base de dades.
			throw new MyFrameUOCException(EnvironmentMsg.DROP_ENTITY_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Tanca una sessi i la seva connexi amb la base de dades.
	 * 
	 * @throws MyFrameUOCException Si no es pot tancar la connexi amb la base de dades. 
	 */
	public void close() throws MyFrameUOCException{
		try{
			if (this.connection != null){
				this.connection.close();
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error al tancar la connexi amb la base de dades.
			throw new MyFrameUOCException(EnvironmentMsg.CLOSE_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Tanca una sessi i la seva connexi amb la base de dades, per sense llenar
	 * cap excepci.
	 */
	public void closeNoEx(){
		try{
			if (this.connection != null){
				this.connection.close();
			}
		}
		catch (Exception e){
		}
	}
}
