package com.uoc.myframeuoc.mapping;

import java.util.ArrayList;

import com.uoc.myframeuoc.environment.Session;
import com.uoc.myframeuoc.exceptions.MappingMsg;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.querymanagement.CreateTable;
import com.uoc.myframeuoc.querymanagement.Delete;
import com.uoc.myframeuoc.querymanagement.Drop;
import com.uoc.myframeuoc.querymanagement.Insert;
import com.uoc.myframeuoc.querymanagement.Select;
import com.uoc.myframeuoc.querymanagement.Update;
import com.uoc.myframeuoc.utils.MyFrameUOC;

/**
 * Classe que representa i gestiona les relacions binries entre entitats.
 * 
 * @author Jordi Garcia Casanova
 *
 */
public class BinaryAssociationMap extends AssociationMap {

	//Nom de la clau forana cap a la taula de l'entitat dest.
	private String nameFKTo							= null;
	
	//Nom de la clau forana cap a la taula de l'entitat origen.
	private String nameFKFrom						= null;
	
	//Correspondncies entre els camps de la clau forana cap a l'entitat dest i els atributs de l'entitat dest.
	private String fieldsTo							= null;
	
	//Correspondncies entre els camps de la clau forana cap a l'entitat origen i els atributs de l'entitat origen.
	private String fieldsFrom						= null;
	
	//Sentncia per a crear la taula de la relaci.
	private CreateTable preparedSqlCreateTable		= null;
	
	//Sentncia per a inserir un registre en la taula de la relaci.
	private Insert preparedSqlInsert				= null;
	
	//Sentncia per a llegir un registre de la taula de la relaci.
	private Select preparedSqlRead					= null;
	
	//Sentncia per a actualitzar un registre de la taula de la relaci.
	private Update preparedSqlUpdate				= null;
	
	//Sentncia per a esborrar un registre de la taula de la relaci.
	private Delete preparedSqlDelete				= null;
	
	//Sentncia que serveix per a esborrar la taula que s'utilitza per a implementar la relaci.
	private Drop preparedSqlDrop					= null;
	
	//Noms dels camps de la clau forana de la relaci cap a l'entitat dest.
	private ArrayList<String> fieldsFKTo			= null;
	
	//Noms dels atributs de l'entitat dest als quals es fa referncia en la clau forana de la relaci.
	private ArrayList<String> propertiesFKTo		= null;
	
	//Noms dels camps de la clau forana de la relaci cap a l'entitat origen.
	private ArrayList<String> fieldsFKFrom			= null;
	
	//Noms dels atributs de l'entitat origen als quals es fa referncia en la clau forana de la relaci.
	private ArrayList<String> propertiesFKFrom		= null;
	
	/**
	 * Constructor.
	 * 
	 * @param table - Nom de la taula que s'utilitza per a implementar la relaci.
	 * @param property - Nom de l'atribut de l'objecte de l'entitat que s relaci cap a una altra entitat.
	 * @param nameEntity - Nom de l'entitat que cont la relaci.
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param nameFKTo - Nom de la clau forana cap a la taula de l'entitat dest.
	 * @param nameFKFrom - Nom de la clau forana cap a la taula de l'entitat origen.
	 * @param fieldsTo - Correspondncies entre els camps de la clau forana i els atributs de l'entitat dest.
	 * @param fieldsFrom - Correspondncies entre els camps de la clau forana i els atributs de l'entitat origen.
	 * 
	 * @throws MyFrameUOCException Si no es pot crear una relaci binria.
	 */
	public BinaryAssociationMap(String table, String property,
			String nameEntity, Session session, String nameFKTo, String nameFKFrom, String fieldsTo, String fieldsFrom) throws MyFrameUOCException {
		super(table, property, nameEntity, session);
		
		if (nameFKTo == null || "".equals(nameFKTo)){
			//Error => No s'ha indicat el nom de la clau forana cap a la taula de l'entitat dest.
			throw new MyFrameUOCException(MappingMsg.BINARYASSOCIATIONMAP_SENSE_NAMEFK);
		}
		if (nameFKFrom == null || "".equals(nameFKFrom)){
			//Error => No s'ha indicat el nom de la clau forana cap a la taula de l'entitat origen.
			throw new MyFrameUOCException(MappingMsg.BINARYASSOCIATIONMAP_SENSE_NAMEFROM);
		}
		if (fieldsTo == null || "".equals(fieldsTo)){
			//Error => No s'ha indicat la correspondncia entre els camps de la clau forana i els atributs de l'entitat dest.
			throw new MyFrameUOCException(MappingMsg.BINARYASSOCIATIONMAP_SENSE_FIELDSTO);
		}
		if (fieldsFrom == null || "".equals(fieldsFrom)){
			//Error => No s'ha indicat la correspondncia entre els camps de la clau forana i els atributs de l'entitat origen.
			throw new MyFrameUOCException(MappingMsg.BINARYASSOCIATIONMAP_SENSE_FIELDSFROM);
		}
		this.nameFKTo = nameFKTo;
		this.nameFKFrom = nameFKFrom;
		this.fieldsTo = fieldsTo;
		this.fieldsFrom = fieldsFrom;
		
		this.buildFieldsPropiertiesTo();
		this.buildFieldsPropertiesFrom();
	}

	/**
	 * Permet obtenir la sentncia CREATE TABLE que serveix per a crear la taula
	 * que s'utilitza per a implementar la relaci.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.CreateTable CreateTable}</code>
	 * que cont la sentncia CREATE TABLE que permet crear la taula
	 * que s'utilitza per a implementar la relaci.
	 */
	public CreateTable getPreparedSqlCreateTable() {
		return preparedSqlCreateTable;
	}

	/**
	 * Defineix la sentncia CREATE TABLE que s'utilitzar per a crear la 
	 * taula de la relaci.
	 * 
	 * @param preparedSqlCreateTable - Un <code>{@link com.uoc.myframeuoc.querymanagement.CreateTable CreateTable}</code>
	 * que cont la sentncia CREATE TABLE que permet crear la taula de la relaci.
	 */
	public void setPreparedSqlCreateTable(CreateTable preparedSqlCreateTable) {
		this.preparedSqlCreateTable = preparedSqlCreateTable;
	}

	/**
	 * Permet obtenir la sentncia INSERT que serveix per a inserir un registre
	 * en la taula que s'utilitza per a implementar la relaci.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.Insert Insert}</code> que
	 * cont la sentncia INSERT que serveix per a inserir un registre en la taula
	 * de la relaci.
	 */
	public Insert getPreparedSqlInsert() {
		return preparedSqlInsert;
	}

	/**
	 * Defineix la sentncia INSERT que s'utilitzar per a inserir un registre en la taula
	 * de la relaci.
	 * 
	 * @param preparedSqlInsert - Un <code>{@link com.uoc.myframeuoc.querymanagement.Insert Insert}</code> que
	 * cont la sentncia INSERT que serveix per a inserir un registre en la taula
	 * de la relaci.
	 */
	public void setPreparedSqlInsert(Insert preparedSqlInsert) {
		this.preparedSqlInsert = preparedSqlInsert;
	}

	/**
	 * Permet obtenir la sentncia SELECT que serveix per a llegir un registre
	 * de la taula que s'utilitza per a implementar la relaci.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.Select Select}</code> que
	 * cont la sentncia SELECT que serveix per a llegir un registre de la taula
	 * de la relaci.
	 */
	public Select getPreparedSqlRead() {
		return preparedSqlRead;
	}

	/**
	 * Defineix la sentncia SELECT que s'utilitzar per a llegir un registre de 
	 * la taula de la relaci.
	 * 
	 * @param preparedSqlRead - Un <code>{@link com.uoc.myframeuoc.querymanagement.Select Select}</code> que
	 * cont la sentncia SELECT que serveix per a llegir un registre de la taula
	 * de la relaci.
	 */
	public void setPreparedSqlRead(Select preparedSqlRead) {
		this.preparedSqlRead = preparedSqlRead;
	}

	/**
	 * Permet obtenir la sentncia UPDATE que serveix per a actualitzar un registre
	 * de la taula que s'utilitza per a implementar la relaci.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.Update Update}</code> que
	 * cont la sentncia UPDATE que serveix per a actualitzar un registre de la taula
	 * de la relaci.
	 */
	public Update getPreparedSqlUpdate() {
		return preparedSqlUpdate;
	}

	/**
	 * Defineix la sentncia UPDATE que s'utilitzar per a actualitzar un regitre
	 * de la taula de la relaci.
	 * 
	 * @param preparedSqlUpdate - Un <code>{@link com.uoc.myframeuoc.querymanagement.Update Update}</code> que
	 * cont la sentncia UPDATE que serveix per a actualitzar un registre de la taula
	 * de la relaci.
	 */
	public void setPreparedSqlUpdate(Update preparedSqlUpdate) {
		this.preparedSqlUpdate = preparedSqlUpdate;
	}

	/**
	 * Permet obtenir la sentncia DELETE que serveix per a esborrar un registre de la
	 * taula que s'utilitza per a implementar la relaci.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.Delete Delete}</code> que
	 * cont la sentncia DELETE que serveix per a esborrar un registre de la taula
	 * de la relaci.
	 */
	public Delete getPreparedSqlDelete() {
		return preparedSqlDelete;
	}

	/**
	 * Defineix la sentncia DELETE que serveix per a esborrar un registre de la taula
	 * de la relaci.
	 * 
	 * @param preparedSqlDelete - Un <code>{@link com.uoc.myframeuoc.querymanagement.Delete Delete}</code> que
	 * cont la sentncia DELETE que serveix per a esborrar un registre de la taula
	 * de la relaci.
	 */
	public void setPreparedSqlDelete(Delete preparedSqlDelete) {
		this.preparedSqlDelete = preparedSqlDelete;
	}

	/**
	 * Permet obtenir el nom de la clau forana cap a la taula de l'entitat dest.
	 * 
	 * @return Un <code>String</code> amb el nom de la clau forana cap a la taula de l'entitat dest.
	 */
	public String getNameFKTo() {
		return nameFKTo;
	}

	/**
	 * Permet obtenir el nom de la clau forana cap a la taula de l'entitat origen.
	 * 
	 * @return Un <code>String</code> amb el nom de la clau forana cap a la taula de l'entitat origen.
	 */
	public String getNameFKFrom() {
		return nameFKFrom;
	}

	/**
	 * Permet obtenir la correspondncia entre els camps de la clau forana i els atributs de l'entitat dest.
	 * 
	 * @return Un <code>String</code> amb la correspondncia entre els camps de la clau frona i els atributs de l'entitat dest.
	 */
	public String getFieldsTo() {
		return fieldsTo;
	}

	/**
	 * Permet obtenir la correspondncia entre els camps de la clau forana i els atributs de l'entitat origen.
	 * 
	 * @return Un <code>String</code> amb la correspondncia entre els camps de la clau forana i els atributs de l'entitat origen.
	 */
	public String getFieldsFrom() {
		return fieldsFrom;
	}
	
	/*
	 * Construeix els arrays fieldsFKTo i propertiesFKTo.
	 */
	private void buildFieldsPropiertiesTo() throws MyFrameUOCException{
		String[] params			= null;
		String[] param			= null;
		
		this.fieldsFKTo = new ArrayList<String>();
		this.propertiesFKTo = new ArrayList<String>();
		
		params = this.fieldsTo.split(MyFrameUOC.SEPARATOR_FIELDS);
		if (params != null && params.length > 0){
			for (int i = 0; i < params.length; i++){
				param = params[i].split(MyFrameUOC.SEPARATOR_FIELD);
				if (param != null && param.length > 0){
					this.fieldsFKTo.add(param[0]);
					this.propertiesFKTo.add(param[1]);
				}
				else{
					//Error => La definici de la correspondncia entre els camps de la clau forana i els atributs destt s incorrecta.
					throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
				}
			}
		}
		else{
			//Error => La definici de la correspondncia entre els camps de la clau forana i els atributs destt s incorrecta.
			throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
		}
	}
	
	/*
	 * Construeix els arrays fieldsFKFrom i propertiesFKFrom.
	 */
	private void buildFieldsPropertiesFrom() throws MyFrameUOCException{
		String[] params			= null;
		String[] param			= null;
		
		this.fieldsFKFrom = new ArrayList<String>();
		this.propertiesFKFrom = new ArrayList<String>();
		
		params = this.fieldsFrom.split(MyFrameUOC.SEPARATOR_FIELDS);
		if (params != null && params.length > 0){
			for (int i = 0; i < params.length; i++){
				param = params[i].split(MyFrameUOC.SEPARATOR_FIELD);
				if (param != null && param.length > 0){
					this.fieldsFKFrom.add(param[0]);
					this.propertiesFKFrom.add(param[1]);
				}
				else{
					//Error => La definici de la correspondncia entre els camps de la clau forana i els atributs origen s incorrecta.
					throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
				}
			}
		}
		else{
			//Error => La definici de la correspondncia entre els camps de la clau forana i els atributs origen s incorrecta.
			throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
		}
	}

	/**
	 * Permet obtenir els camps de la clau forana de l'entitat dest.
	 * 
	 * @return Un <code>ArrayList</code> amb els camps de la clau foranas de 
	 * l'entitat dest.
	 */
	public ArrayList<String> getFieldsFKTo() {
		return fieldsFKTo;
	}

	/**
	 * Permet obtenir el nom dels atributs de l'entitat dest als quals es fa 
	 * referncia en la clau forana.
	 * 
	 * @return Un <code>ArrayList</code> amb el nom dels atributs de l'entitat dest
	 * als quals es fa referncia en la clau forana.
	 */
	public ArrayList<String> getPropertiesFKTo() {
		return propertiesFKTo;
	}

	/**
	 * Permet obtenir els camps de la clau forana de l'entitat origen.
	 * 
	 * @return Un <code>ArrayList</code> amb els camps de la clau forana de l'entitat origen.
	 */
	public ArrayList<String> getFieldsFKFrom() {
		return fieldsFKFrom;
	}

	/**
	 * Permet obtenir el noms dels atributs de l'entitat origen als quals es fa referncia
	 * en la clau forana.
	 * 
	 * @return Un <code>ArrayList</code> amb el nom dels atributs de l'entitat origen als quals
	 * es fa referncia en la clau forana.
	 */
	public ArrayList<String> getPropertiesFKFrom() {
		return propertiesFKFrom;
	}
	
	/**
	 * Permet conixer si hi han camps de clau forana definits cap a l'entitat dest.
	 * 
	 * @return <code>True</code> si hi han camps de clau forana definits cap a l'entitat dest.
	 * <code>False</code> en qualsevol altre cas.
	 */
	public boolean hasFieldsFKTo(){
		boolean has = false;
		
		if (this.fieldsFKTo != null && this.fieldsFKTo.size() > 0){
			has = true;
		}
		return (has);
	}
	
	/**
	 * Permet conixer si hi han atributs de l'entitat dest definits en la clau forana.
	 * 
	 * @return <code>True</code> si hi han atributs de l'entitat dest definits en la clau forana.
	 * <code>False</code> en qualsevol altre cas.
	 */
	public boolean hasPropertiesFKTo(){
		boolean has = false;
		
		if (this.propertiesFKTo != null && this.propertiesFKTo.size() > 0){
			has = true;
		}
		return (has);
	}
	
	/**
	 * Permet conixer si hi han camps de clau forana definits cap a l'entitat origen.
	 * 
	 * @return <code>True</code> si hi han camps de clau forana definits cap a l'entitat origen
	 * <code>False</code> en qualsevol altre cas.
	 */
	public boolean hasFieldsFKFrom(){
		boolean has = false;
		
		if (this.fieldsFKFrom != null && this.fieldsFKFrom.size() > 0){
			has = true;
		}
		return (has);
	}
	
	/**
	 * Permet conixer si hi han atributs de l'entitat origen definits en la clau forana.
	 * 
	 * @return <code>True</code> si hi han atributs de l'entitat origen definits en la clau forana.
	 * <code>False</code> en qualsevol altre cas.
	 */
	public boolean hasPropertiesFKFrom(){
		boolean has = false;
		
		if (this.propertiesFKTo != null && this.propertiesFKTo.size() > 0){
			has = true;
		}
		return (has);
	}

	/**
	 * Permet obtenir la sentncia DROP TABLE que serveix per a eliminar la taula que s'utilitza per a
	 * implementar la relaci.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.Drop Drop}</code> que cont
	 * la sentncia DROP TABLE que s'utilitzar per a eliminar la taula de la relaci.
	 */
	public Drop getPreparedSqlDrop() {
		return preparedSqlDrop;
	}

	/**
	 * Defineix la sentncia DROP TABLE que s'utilitzar per a eliminar la taula de la relaci.
	 * 
	 * @param preparedSqlDrop - Un <code>{@link com.uoc.myframeuoc.querymanagement.Drop Drop}</code> que cont
	 * la sentncia DROP TABLE que s'utilitzar per a eliminar la taula de la relaci.
	 */
	public void setPreparedSqlDrop(Drop preparedSqlDrop) {
		this.preparedSqlDrop = preparedSqlDrop;
	}
	
	
}
