package com.uoc.myframeuoc.mapping;

import com.uoc.myframeuoc.database.TypeMap;
import com.uoc.myframeuoc.exceptions.MappingMsg;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;

/**
 * Classe que represente i gestiona un objecte mapejat d'una entitat.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public class FieldMap {
	//Nom de l'atribut de l'entitat que es mapeja.
	private String property			= null;
	
	//Nom del camp de la taula amb el que es vincula l'atribut.
	private String field			= null;
	
	//Tipus de dades mapejat al que pertany l'atribut.
	private TypeMap typeMap			= null;
	
	//Precisi del camp numric de la taula.
	private Integer precision		= -1;
	
	//Escala del camp numric de la taula.
	private Integer scale			= -1;
	
	//Mida del camp (en carcters o bytes).
	private Integer length			= -1;
	
	//Indica si el seu valor es pot reptir dins de la taula.
	private boolean unique			= false;
	
	//Indica si s obligatori.
	private boolean required		= false;
	
	/**
	 * Constructor.
	 * 
	 * @param property - Nom de l'atribut de l'entitat que es mapeja.
	 * @param field	- Nom del camp de la taula amb el que es vincula l'atribut.
	 * @param typeMap - Tipus de dades mapejat al que pertany l'atribut.
	 * @param precision - Precisi quan es tracta d'un camp numric. 
	 * @param scale - Escala quan es tracta d'un camp numric.
	 * @param length - Mida en carcters o bytes del camp.
	 * @param unique - Indica si el valor del camp es pot repetir.
	 * @param required - Indica si el camp s obligatori.
	 * 
	 * @throws MyFrameUOCException Si no es pot crear l'objecte mapejat.
	 */
	public FieldMap(String property, String field, TypeMap typeMap, Integer precision, Integer scale, Integer length, boolean unique, boolean required) throws MyFrameUOCException {
		if (property == null || "".equals(property)){
			//Error => No es pot crear un objecte mapejat sense un atribut.
			throw new MyFrameUOCException(MappingMsg.OBJECTEMAPEJAT_SENSE_PROPIETAT);
		}
		if (field == null || "".equals(field)){
			//Error => No es pot crear un objecte mapejat sense el camp de la taula.
			throw new MyFrameUOCException(MappingMsg.OBJECTEMAPEJAT_SENSE_CAMP);
		}
		if (typeMap == null){
			//Error => No es pot crear un objecte mapejat sense el tipus de dades mapejat.
			throw new MyFrameUOCException(MappingMsg.OBJECTEMAPEJAT_SENSE_TIPUS);
		}
		this.property = property;
		this.field = field;
		this.typeMap = typeMap;
		this.precision = precision;
		this.scale = scale;
		this.length = length;
		this.unique = unique;
		this.required = required;
	}

	/**
	 * Permet obtenir el nom de l'atribut que interv en el mapeig.
	 * 
	 * @return Un <code>String</code> amb el nom de l'atribut que interv en el mapeig.
	 */
	public String getProperty(){
		return (this.property);
	}
	
	/**
	 * Permet obtenir el nom del camp de la taula amb el que es vincula l'atribut.
	 * 
	 * @return Un <code>String</code> amb el nom del camp de la taula amb el que es vincula l'atribut.
	 */
	public String getField(){
		return (this.field);
	}
	
	/**
	 * Permet obtenir el tipus de dades mapejat.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.database.TypeMap TypeMap}</code> que 
	 * cont el tipus de dades mapejat que li correspon a l'objecte que es mapeja.
	 */
	public TypeMap getTypeMap(){
		return (this.typeMap);
	}
	
	/**
	 * Permet obtenir la precisi que t el camp numric en la taula.
	 * 
	 * @return Un <code>int</code> amb la precisi que t el camp numric en la taula.
	 */
	public Integer getPrecision(){
		return (this.precision);
	}
	
	/**
	 * Permet obtenir l'escala que t el camp numric en la taula.
	 * 
	 * @return Un <code>int</code> amb l'escala que t el camp numric en la taula.
	 */
	public Integer getScale(){
		return (this.scale);
	}
	
	/**
	 * Permet obtenir la mida en carcters o bytes que t el camp en la taula.
	 * 
	 * @return Un <code>int</code> amb la mida en carcters o bytes que t el camp en la taula.
	 */
	public Integer getLength(){
		return (this.length);
	}
	
	/**
	 * Permet conixer si el seu valor s nic.
	 * 
	 * @return <code>True</code> si el seu valor ha de ser nic. <code>False</code> en qualsevol
	 * altre cas.
	 */
	public boolean getUnique(){
		return (this.unique);
	}
	
	/**
	 * Permet conixer si s obligatori.
	 * 
	 * @return <code>True</code> si s obligatori i, per tant, no pot ser nul. <code>False</code> en
	 * qualsevol altre cas.
	 */
	public boolean getRequired(){
		return (this.required);
	}
}
