package com.uoc.myframeuoc.mapping;

import java.util.ArrayList;

import com.uoc.myframeuoc.environment.Session;
import com.uoc.myframeuoc.exceptions.MappingMsg;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.utils.MyFrameUOC;

/**
 * Classe que represneta i gestiona les claus foranes d'una entitat.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public class ForeignKeyMap extends AssociationMap {

	//Nom que t la clau forana en la taula de la base de dades.
	private String name							= null;
	
	//Correspondncia entre els camps de l'entitat origen i els de l'entitat referenciada.
	private String fields						= null;
	
	//Llista dels noms dels camps de la clau forana.
	private ArrayList<String> fieldsFK			= null;
	
	//Lista dels noms dels atributs de clau primria de l'entitat dest als quals fan referncia els camps.
	private ArrayList<String> propertiesFK		= null;
	
	/**
	 * Constructor.
	 * 
	 * @param table - Taula amb la qual s'implementa la relaci.
	 * @param property - Atribut de la relaci.
	 * @param nameEntity - Nom de l'entitat de la relaci.
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param name - Nom que t la clau forana en la taula de la base de dades.
	 * @params fields - Correspondncia entre els camps de l'entitat origen i els de l'entitat
	 * referenciada.
	 * 
	 * @throws MyFrameUOCException Si no es pot crear la clau forana.
	 */
	public ForeignKeyMap(String table, String property, String nameEntity, Session session, String name, String fields) throws MyFrameUOCException {
		super(table, property, nameEntity, session);
		
		if (name == null || "".equals(name)){
			//Error => No es pot crear una clau forana sense el seu nom.
			throw new MyFrameUOCException(MappingMsg.CLAU_FORANA_SENSE_NOM);
		}
		if (fields == null || "".equals(name)){
			//Error => No es pot crear una clau forana sense la correspondncia entre els camps origen i dest.
			throw new MyFrameUOCException(MappingMsg.CLAU_FORANA_SENSE_CORRESPONDENCIA);
		}
		this.name = name;
		this.fields = fields;
		
		this.buildFieldsProperties();
	}
	
	/*
	 * Construeix la llista dels camps de clau forana i la llista
	 * dels atributs de clau primria de l'entitat dest als quals fan
	 * referncia aquests camps.
	 */
	private void buildFieldsProperties(){
		String[] params			= null;
		String[] param			= null;
		
		this.fieldsFK = new ArrayList<String>();
		this.propertiesFK = new ArrayList<String>();
		
		params = this.fields.split(MyFrameUOC.SEPARATOR_FIELDS);
		if (params != null && params.length > 0){
			for (int i = 0; i < params.length; i++){
				param = params[i].split(MyFrameUOC.SEPARATOR_FIELD);
				if (param != null && param.length > 0){
					this.fieldsFK.add(param[0]);
					this.propertiesFK.add(param[1]);
				}
			}
		}
	}
	
	/**
	 * Permet obtenir el nom que t la clau forana en la taula de la base de dades.
	 *  
	 * @return Un <code>String</code> amb el nom que t la clau forana en la taula
	 * de la base de dades.
	 */
	public String getName(){
		return (this.name);
	}

	/**
	 * Permet obtenir la correspondncia entre els camps origen i dest d'una clau forana.
	 * 
	 * @return Un <code>String</code> amb la corresponncia entre els camps origen i dest
	 * d'una clau forana.
	 */
	public String getFields(){
		return (this.fields);
	}
	
	/**
	 * Permet obtenir els camps mapejats indicats en l'anotaci.
	 * 
	 * @return Un <code>ArrayList</code> que cont un <code>String[2]</code> on el primer
	 * element s el camp origen i el segon element s la clau primria de l'entitat dest.
	 */
	public ArrayList<String[]> getMappedFields(){
		ArrayList<String[]> mappedFields		= null;
		String[] fields							= null;
		String[] mappedField					= null;
		
		if (this.fields != null && !"".equals(this.fields)){
			mappedFields = new ArrayList<String[]>();
			fields = this.fields.split(MyFrameUOC.SEPARATOR_FIELDS);
			for (String field: fields){
				mappedField = field.split(MyFrameUOC.SEPARATOR_FIELD);
				mappedFields.add(mappedField);
			}
		}
		return (mappedFields);
	}
	
	/**
	 * Permet obtenir la llista dels noms dels camps que formen part de la clau forana.
	 * 
	 * @return Un <code>ArrayList</code> que cont els noms dels camps que forrmen
	 * part de la clau forana.
	 */
	public ArrayList<String> getFieldsFK(){
		return (this.fieldsFK);
	}
	
	/**
	 * Permet obtenir la llista dels noms dels atributs de l'entitat dest als quals
	 * es fa referncia en la clau forana.
	 * 
	 * @return Un <code>ArrayList</code> que cont els noms dels atributs de l'entitat
	 * dest als quals es fa referncia en la clau forana.
	 */
	public ArrayList<String> getPropertiesFK(){
		return (this.propertiesFK);
	}
}
