package com.uoc.myframeuoc.mapping;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;

import com.uoc.myframeuoc.annotations.Column;
import com.uoc.myframeuoc.annotations.Entity;
import com.uoc.myframeuoc.annotations.ForeignKey;
import com.uoc.myframeuoc.annotations.ManyToMany;
import com.uoc.myframeuoc.annotations.OneToMany;
import com.uoc.myframeuoc.annotations.OneToOne;
import com.uoc.myframeuoc.annotations.PrimaryKey;
import com.uoc.myframeuoc.database.Connection;
import com.uoc.myframeuoc.database.JavaType;
import com.uoc.myframeuoc.database.TypeMap;
import com.uoc.myframeuoc.environment.EntityManager;
import com.uoc.myframeuoc.environment.Session;
import com.uoc.myframeuoc.exceptions.GeneralMsg;
import com.uoc.myframeuoc.exceptions.MappingMsg;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.querymanagement.AddCondition;
import com.uoc.myframeuoc.querymanagement.AlterTable;
import com.uoc.myframeuoc.querymanagement.AlterTableType;
import com.uoc.myframeuoc.querymanagement.Condition;
import com.uoc.myframeuoc.querymanagement.ConstraintType;
import com.uoc.myframeuoc.querymanagement.CreateTable;
import com.uoc.myframeuoc.querymanagement.Delete;
import com.uoc.myframeuoc.querymanagement.Drop;
import com.uoc.myframeuoc.querymanagement.FieldValuePair;
import com.uoc.myframeuoc.querymanagement.ForeignKeyConstraint;
import com.uoc.myframeuoc.querymanagement.Insert;
import com.uoc.myframeuoc.querymanagement.LogicalOperator;
import com.uoc.myframeuoc.querymanagement.RelationalOperator;
import com.uoc.myframeuoc.querymanagement.Restriction;
import com.uoc.myframeuoc.querymanagement.Select;
import com.uoc.myframeuoc.querymanagement.Update;
import com.uoc.myframeuoc.utils.MyFrameUOC;

/**
 * Clase que representa una entitat (una classe persistent en MyFrameUOC).
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public class RegisteredEntity {
	
	//Nom de l'entitat o de la classe que es fa persistent.
	private String name									= null;
	
	//Nom de la taula amb la qual s'associa l'entitat.
	private String table								= null;
	
	//Sentncia CREATE TABLE per a crear la taula de l'entitat.
	private CreateTable preparedSqlCreateTable			= null;
	
	//Sentncia CREATE TABLE per a crear la taula de l'entitat per sense les restriccions de clau forana.
	private CreateTable preparedSqlCreateTableNoFK		= null;
	
	//Sentncia ALTER TABLE par a modificar la taula i afegir-hi les restriccions de clau forana.
	private AlterTable preparedSqlAlterTableAddFK		= null;
	
	//Sentncia INSERT per a inserir un registre en la taula sense tenir en compte els valors de les claus foranes.
	private Insert preparedSqlInsertNoFK				= null;
	
	//Sentncia UPDATE per a actualitzar noms els valors de les claus foranes d'un registre de la taula de l'entitat.
	private Update preparedSqlUpdateOnlyFK				= null;
	
	//Sentncia ALTER TABLE per a modificar la taula i treure les restriccions de clau forana.
	private AlterTable preparedSqlAlterTableDropFK		= null;
	
	//Sentncia SELECT per a llegir un objecte d'una entitat de la base de dades.
	private Select preparedSqlRead						= null;
	
	//Sentncia UPDATE per actualitzar un objecte d'una entitat en la base de dades.
	private Update preparedSqlUpdate					= null;
	
	//Sentncia DELETE per a esborrar un objecte de la base de dades.
	private Delete preparedSqlDelete					= null;
	
	//ArrayList amb els atributs mapejats que formen la clau primria.
	private ArrayList<PrimaryKeyMap> primaryKeyMaps		= null;
	
	//ArrayList amb els atributs de tipus columna mapejats.
	private ArrayList<ColumnMap> columnMaps				= null;
	
	//ArrayList amb els atributs mapejats que formen part d'una clau forana.
	private ArrayList<ForeignKeyMap> foreignKeyMaps		= null;
	
	//ArrayList amb les relacions d'un a un mapjeades de l'entitat.
	private ArrayList<OneToOneMap> oneToOneMaps			= null;
	
	//ArrayList amb les relacions d'un a molts mapjeades de l'entitat.
	private ArrayList<OneToManyMap> oneToManyMaps		= null;
	
	//ArrayList amb les relacions de mols a molts mapejades de l'entitat.
	private ArrayList<ManyToManyMap> manyToManyMaps		= null;
	
	//Sentncia DROP que permet esborrar la taula de l'entitat.
	private Drop preparedSqlDrop						= null;
	
	//Indica si ja s'ha fet el 'drop' d'una entitat.
	private boolean dropped								= false;
	
	//Cont els objectes que s'han llegit en un moment donat.
	private static ArrayList<Object> readObjects		= null;
	
	//Cont els objectes que s'han esborrat en un moment donat.
	private static ArrayList<Object> deletedObjects		= null;	
	
	//Cont els objeces que s'han inserit en un moment donat.
	private static ArrayList<Object> insertedObjects	= null;
	
	//S'utilitza per conixer quan cal esborrar els objectes de la llista d'objectes emmagatzemats.
	private static boolean isInserting					= false;
	
	//S'utilitza per conixer quan cal esborrar els objectes de la llista d'objectes llegits
	private static boolean isReading					= false;
	
	//S'utilitza per conixer quan cal esborrar els objectes de la llista d'objectes eliminats. 
	private static boolean isDeleting					= false;
	
	/**
	 * Constructor.
	 * 
	 * @param nameEntity - Nom de l'entitat que es vol crear.
	 * @param session - Sessi de MyFrameUOC que utilitza l'usuari.
	 * 
	 * @throws MyFrameUOCException Si no es pot crear una entitat.
	 */
	public RegisteredEntity(String nameEntity, Session session) throws MyFrameUOCException {
		Class<?> entityClass	= null;
		
		if (nameEntity == null || "".equals(nameEntity)){
			//Error => No es pot crear una entitat sense el seu nom.
			throw new MyFrameUOCException(MappingMsg.CREAR_ENTITAT_SENSE_NOM);
		}
		if (session == null){
			//Error => La sessi no est inicialitzada.
			throw new MyFrameUOCException(MappingMsg.CREAR_ENTITAT_SENSE_SESSIO);
		}
		
		try{
			this.name = nameEntity;

			try {
				entityClass = Class.forName(this.name);
			} 
			catch (ClassNotFoundException e) {
				//Error => S'ha produt un error en la obtenci d'una classe.
				throw new MyFrameUOCException(MappingMsg.ERROR_OBTENCIO_CLASSE, e.getMessage());
			}

			//Obtenir tota la informaci de l'entitat mitjanant les anotacions de MyFrameUOC.
			this.getEntityMetaData(session, entityClass);

			//Construir les sentncies SQL de l'entitat.
			this.prepareQueries(session);
		}
		catch (Exception e){
			session.getEntityManager().dropEntity(this);
			//Error => S'ha produt un error en la creaci d'una entitat.
			throw new MyFrameUOCException(MappingMsg.CREAR_ENTITAT_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Obt la informaci de l'entitat (mapeig dels atributs, claus primries, claus foranes, relacions, etc.) a partir
	 * de les anotacions de la classe de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC utilitzada per l'usuari.
	 * @param entityClass - Un Class<?> amb la classe de l'entitat.
	 */
	private void getEntityMetaData(Session session, Class<?> entityClass) throws MyFrameUOCException{
		java.lang.reflect.Field[] fields						= null;
		ArrayList<java.lang.reflect.Field> primaryKeyFields		= null;
		ArrayList<java.lang.reflect.Field> foreignKeyFields		= null;
		ArrayList<java.lang.reflect.Field> columnFields			= null;
		ArrayList<java.lang.reflect.Field> oneToOneFields		= null;
		ArrayList<java.lang.reflect.Field> oneToManyFields		= null;
		ArrayList<java.lang.reflect.Field> manyToManyFields		= null;
		Annotation annotation									= null;
		
		if (entityClass == null){
			//Error => No es poden obtenir les anotacions d'una classe que s nulla.
			throw new MyFrameUOCException(MappingMsg.ANOTACIONS_SENSE_CLASSE);
		}
		
		//fields = entityClass.getFields();
		fields = entityClass.getDeclaredFields();

		
		if (fields == null || fields.length == 0){
			//Error => L'entitat no t atributs.
			throw new MyFrameUOCException(MappingMsg.ENTITAT_SENSE_ATRIBUTS);
		}
		primaryKeyFields = new ArrayList<java.lang.reflect.Field>();
		foreignKeyFields = new ArrayList<java.lang.reflect.Field>();
		columnFields = new ArrayList<java.lang.reflect.Field>();
		oneToOneFields = new ArrayList<java.lang.reflect.Field>();
		oneToManyFields = new ArrayList<java.lang.reflect.Field>();
		manyToManyFields = new ArrayList<java.lang.reflect.Field>();
		
		for (java.lang.reflect.Field field: fields){
			annotation = field.getAnnotation(PrimaryKey.class);
			if (annotation != null){
				//L'anotaci de l'atribut s de clau primria.
				primaryKeyFields.add(field);
			}
			else {
				annotation = field.getAnnotation(Column.class);
				if (annotation != null){
					//L'anotaci de l'atribut s de columna.
					columnFields.add(field);
				}
				else{
					annotation = field.getAnnotation(ForeignKey.class);
					if (annotation != null){
						//L'anotaci de l'atribut s de clau forana.
						foreignKeyFields.add(field);
					}
					else{
						annotation = field.getAnnotation(OneToOne.class);
						if (annotation != null){
							//L'anotaci de l'atribut s una relaci d'un a un.
							oneToOneFields.add(field);
						}
						else{
							annotation = field.getAnnotation(OneToMany.class);
							if (annotation != null){
								//L'anotaci de l'atribut s una relaci d'un a molts.
								oneToManyFields.add(field);
							}
							else{
								annotation = field.getAnnotation(ManyToMany.class);
								if (annotation != null){
									//L'anotaci de l'atribut s una relaci de molts a molts.
									manyToManyFields.add(field);
								}
								else{
									//Error => L'entitat no t anotacions definides.
									throw new MyFrameUOCException(MappingMsg.ENTITAT_SENSE_ANOTACIONS);
								}
							}
						}
					}
				}
			}
		}
		
		//Obtenir la informaci de l'anotaci de l'entitat.
		this.getEntityAnnotationsData(entityClass);
		
		//Obtenir la informaci de les anotacions de la clau primria.
		this.getPrimaryKeyAnnotationsData(session, entityClass, primaryKeyFields);
		
		//Obtenir la informaci de les anotacions de les columnes.
		this.getColumnAnnotationsData(session, entityClass, columnFields);
		
		//Registrar realment l'entitat.
		session.getEntityManager().addEntity(this);
		
		//Obtenir la informaci de les anotacions de les claus foranes.
		this.getForeignKeyAnnotationsData(session, entityClass, foreignKeyFields);
		
		//Obtenir la informaci de les anotacions de les relacions un a un.
		this.getOneToOneAnnotationsData(session, entityClass, oneToOneFields);
		
		//Obtenir la informaci de les anotacions de les relacions un a molts.
		this.getOneToManyAnnotationsData(session, entityClass, oneToManyFields);
		
		//Obtenir la informaci de les anotacions de les relacions molts a molts.
		this.getManyToManyAnnotationsData(session, entityClass, manyToManyFields);
	}
	
	/*
	 * Obt la informaci de l'anotaci de l'entitat.
	 * 
	 * @param entityClass - Un Class<?> amb la classe de l'entitat.
	 */
	private void getEntityAnnotationsData(Class<?> entityClass) throws MyFrameUOCException{
		Entity aEntity		= null;

		aEntity = (Entity) entityClass.getAnnotation(Entity.class);

		if (aEntity == null){
			//Error => L'entitat no t definit l'anotaci @Entity.
			throw new MyFrameUOCException(MappingMsg.ANOTACIO_ENTITAT_NO_EXISTEIX);
		}
		this.table = aEntity.Table();
		if (this.table == null || "".equals(this.table)){
			//Error => La propietat Table de l'anotaci @Entity no est informada.
			throw new MyFrameUOCException(MappingMsg.ANOTACIO_ENTITAT_SENSE_TAULA);
		}
	}
	
	/*
	 * Obt la informaci de les anotacions de clau primria.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param entityClass - Un Class<?> amb la classe de l'entitat.
	 * @param fields - Un ArrayList<java.lang.reflect.Field> que cont els atributs de l'entitat que 
	 * tenen una anotaci de clau primria.
	 */
	private void getPrimaryKeyAnnotationsData(Session session, Class<?> entityClass, ArrayList<java.lang.reflect.Field> fields) throws MyFrameUOCException{
		PrimaryKey aPrimaryKey				= null;
		PrimaryKeyMap primaryKeyMap			= null;
		Integer precision					= null;
		Integer scale						= null;
		Integer length						= null;
		TypeMap typeMap						= null;
		
		if (session == null){
			//Error => No hi ha una sessi inicialitzada.
			throw new MyFrameUOCException(MappingMsg.OBTENIR_ANOTACIONS_PK_SENSE_SESSIO);
		}
		
		try{
			if (fields != null && fields.size() > 0){
				if (this.primaryKeyMaps == null){
					this.primaryKeyMaps = new ArrayList<PrimaryKeyMap>();
				}
				for (java.lang.reflect.Field fieldClass: fields){
					aPrimaryKey = fieldClass.getAnnotation(PrimaryKey.class);
					if (!"".equals(aPrimaryKey.Precision())){
						precision = Integer.valueOf(aPrimaryKey.Precision());
					}
					if (!"".equals(aPrimaryKey.Scale())){
						scale = Integer.valueOf(aPrimaryKey.Scale());
					}
					if (!"".equals(aPrimaryKey.Length())){
						length = Integer.valueOf(aPrimaryKey.Length());
					}
					typeMap = session.getConnection().getTypeMap(fieldClass.getType().getCanonicalName(), precision, scale, length);
					primaryKeyMap = new PrimaryKeyMap(fieldClass.getName(), aPrimaryKey.Field(), typeMap, 
							precision, scale, length, aPrimaryKey.Name());
					this.primaryKeyMaps.add(primaryKeyMap);
					precision = null;
					scale = null;
					length = null;
				}
			}
		}
		catch (Exception e){
			//Error => No s'ha pogut obtenir la informaci de les anotacions de clau primria.
			throw new MyFrameUOCException(MappingMsg.OBTENIR_ANOTACIONS_PK_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Obt la informaci de les anotacions de clau forana.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param entityClass - Un Class<?> amb la classe de l'entitat.
	 * @param fields - Un ArrayList<java.lang.reflect.Field> que cont els atributs de l'entitat que 
	 * tenen una anotaci de clau forana.
	 */
	private void getForeignKeyAnnotationsData(Session session, Class<?> entityClass, 
			ArrayList<java.lang.reflect.Field> fields) throws MyFrameUOCException{
		ForeignKey aForeignKey			= null;
		ForeignKeyMap foreignKeyMap		= null;
		
		if (session == null){
			//Error => No hi ha una sessi inicialitzada.
			throw new MyFrameUOCException(MappingMsg.OBTENIR_ANOTACIONS_FK_SENSE_SESSIO);
		}
		try{
			if (fields != null && fields.size() > 0){
				if (this.foreignKeyMaps == null){
					this.foreignKeyMaps = new ArrayList<ForeignKeyMap>();
				}
				for (java.lang.reflect.Field fieldClass: fields){
					aForeignKey = fieldClass.getAnnotation(ForeignKey.class);
					foreignKeyMap = new ForeignKeyMap(null, fieldClass.getName(), fieldClass.getType().getCanonicalName(), 
							session, aForeignKey.Name(), aForeignKey.Fields());
					this.foreignKeyMaps.add(foreignKeyMap);
				}
			}
		}
		catch (Exception e){
			//Error => No s'ha pogut obtenir la informaci de les anotacions de clau forana.
			throw new MyFrameUOCException(MappingMsg.OBTENIR_ANOTACIONS_FK_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Obt la informaci de les anotacions de les columnes.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param entityClass - Un Class<?> amb la classe de l'entitat.
	 * @param fields - Un ArrayList<java.lang.reflect.Field> que cont els atributs de l'entitat que 
	 * tenen una anotaci de tipus columna.
	 */
	private void getColumnAnnotationsData(Session session, Class<?> entityClass, ArrayList<java.lang.reflect.Field> fields) throws MyFrameUOCException{
		Column aColumn			= null;
		ColumnMap columnMap		= null;
		Integer precision		= null;
		Integer scale			= null;
		Integer length			= null;
		TypeMap typeMap			= null;
		
		if (session == null){
			//Error => No hi ha una sessi inicialitzada.
			throw new MyFrameUOCException(MappingMsg.OBTENIR_ANOTACIONS_COLUMNA_SENSE_SESSIO);
		}
		
		try{
			if (fields != null && fields.size() > 0){
				if (this.columnMaps == null){
					this.columnMaps = new ArrayList<ColumnMap>();
				}
				for (java.lang.reflect.Field fieldClass: fields){
					aColumn = fieldClass.getAnnotation(Column.class);
					if (!"".equals(aColumn.Precision())){
						precision = Integer.valueOf(aColumn.Precision());
					}
					if (!"".equals(aColumn.Scale())){
						scale = Integer.valueOf(aColumn.Scale());
					}
					if (!"".equals(aColumn.Length())){
						length = Integer.valueOf(aColumn.Length());
					}
					typeMap = session.getConnection().getTypeMap(fieldClass.getType().getCanonicalName(), precision, scale, length);
					columnMap = new ColumnMap(fieldClass.getName(), aColumn.Field(), typeMap, precision, scale, length, aColumn.Unique(), aColumn.Required());
					this.columnMaps.add(columnMap);
					precision = null;
					scale = null;
					length = null;
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'obtenci de la informaci de les anotacions de columnes.
			throw new MyFrameUOCException(MappingMsg.OBTENIR_ANOTACIONS_COLUMNA_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Obt la informaci de les anotacions de les relacions un a un.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param entityClass - Un Class<?> amb la classe de l'entitat.
	 * @param fields - Un ArrayList<java.lang.reflect.Field> que cont els atributs de l'entitat que 
	 * tenen una anotaci de relaci d'un a un.
	 */
	private void getOneToOneAnnotationsData(Session session, Class<?> entityClass, ArrayList<java.lang.reflect.Field> fields) throws MyFrameUOCException{
		OneToOne aOneToOne			= null;
		OneToOneMap oneToOneMap		= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.GETONETOONEANNOTATIONSDATA_SENSE_SESSIO);
			}
			if (fields != null && fields.size() > 0){
				if (this.oneToOneMaps == null){
					this.oneToOneMaps = new ArrayList<OneToOneMap>();
				}
				
				for (java.lang.reflect.Field field: fields){
					aOneToOne = field.getAnnotation(OneToOne.class);
					oneToOneMap = new OneToOneMap(aOneToOne.Table(), field.getName(), field.getType().getCanonicalName(), session,  
									aOneToOne.NameFK_To(), aOneToOne.NameFK_From(), aOneToOne.Fields_To(), aOneToOne.Fields_From());
					this.oneToOneMaps.add(oneToOneMap);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'obtenci de la informaci de les relacions un a un.
			throw new MyFrameUOCException(MappingMsg.GETONETOONEANNOTACIONSDATA_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Obt la informaci de les anotacions de les relacions d'un a molts (1 a N).
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param entityClass - Un Class<?> amb la classe de l'entitat.
	 * @param fields - Un ArrayList<java.lang.reflect.Field> que cont els atributs de l'entitat que 
	 * tenen una anotaci de relaci d'un a molts.
	 */
	private void getOneToManyAnnotationsData(Session session, Class<?> entityClass, ArrayList<java.lang.reflect.Field> fields) throws MyFrameUOCException{
		OneToMany aOneToMany		= null;
		OneToManyMap oneToManyMap	= null;
		String propertyType			= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.GETONETOMANYANNOTATIONDATA_SENSE_SESSIO);
			}
			if (fields != null && fields.size() > 0){
				if (this.oneToManyMaps == null){
					this.oneToManyMaps = new ArrayList<OneToManyMap>();
				}
				
				for (java.lang.reflect.Field field: fields){
					aOneToMany = field.getAnnotation(OneToMany.class);
					propertyType = this.getEntityCollectionType(this.name, field.getName());
					oneToManyMap = new OneToManyMap(aOneToMany.Table(), field.getName(), propertyType, session,  
									aOneToMany.NameFK_To(), aOneToMany.NameFK_From(), aOneToMany.Fields_To(), aOneToMany.Fields_From());					
					this.oneToManyMaps.add(oneToManyMap);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'obtenci de la informaci de les relacions un a un.
			throw new MyFrameUOCException(MappingMsg.GETONETOMANYANNOTATIONDATA_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Permet obtenir el tipus de dades dels objectes d'una Collection de l'atribut d'una classe.
	 * 
	 * @param nameEntity - Nom de l'entitat o classe que cont l'atribut que s una Collection i de la qual es vol conixer el tipus dels seus objectes.
	 * @param property - Nom de l'atribut de la classe 'c' que cont la Collection.
	 */
	private String getEntityCollectionType(String nameEntity, String property) throws MyFrameUOCException{
		String collectionType					= null;
		Class<?> c								= null;
		Field field								= null;
		Type type								= null;
		ParameterizedType parameterizedType		= null;
		Type[] types							= null;
		Class<?> propertyClass					= null;
		
		try{
			if (nameEntity == null || "".equals(nameEntity)){
				//Error => No s'ha indicat el nom de l'entitat.
				throw new MyFrameUOCException(MappingMsg.GETENTITYCOLLECTIONTYPE_SENSE_ENTITAT);
			}
			if (property == null || "".equals(property)){
				//Error => No s'ha indicat el nom de l'atribut de l'entitat.
				throw new MyFrameUOCException(MappingMsg.GETENTITYCOLLECTIONTYPE_SENSE_PROPIETAT);
			}
			c = Class.forName(nameEntity);
			field = c.getDeclaredField(property);
			type = field.getGenericType();
			parameterizedType = (ParameterizedType) type;
			types = parameterizedType.getActualTypeArguments();
			propertyClass = (Class<?>) types[0];
			collectionType = propertyClass.getCanonicalName();
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'obtenci del tipus de dades dels objectes d'una collecci.
			throw new MyFrameUOCException(MappingMsg.GETENTITYCOLLECTIONTYPE_ERROR, e.getMessage());
		}
		return (collectionType);
	}
	
	/*
	 * Obt la informaci de les anotacions de les relacions molts a molts.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param entityClass - Un Class<?> amb la classe de l'entitat.
	 * @param fields - Un ArrayList<java.lang.reflect.Field> que cont els atributs de l'entitat que 
	 * tenen una anotaci de relaci de molts a molts.
	 */
	private void getManyToManyAnnotationsData(Session session, Class<?> entityClass, ArrayList<java.lang.reflect.Field> fields) throws MyFrameUOCException{
		ManyToMany aManyToMany			= null;
		ManyToManyMap manyToManyMap		= null;
		String propertyType				= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.GETMANYTOMANYANNOTATIONDATA_SENSE_SESSIO);
			}
			if (fields != null && fields.size() > 0){
				if (this.manyToManyMaps == null){
					this.manyToManyMaps = new ArrayList<ManyToManyMap>();
				}
				
				for (java.lang.reflect.Field field: fields){
					aManyToMany = field.getAnnotation(ManyToMany.class);
					propertyType = this.getEntityCollectionType(this.name, field.getName());
					manyToManyMap = new ManyToManyMap(aManyToMany.Table(), field.getName(), propertyType, session,  
									aManyToMany.NameFK_To(), aManyToMany.NameFK_From(), aManyToMany.Fields_To(), aManyToMany.Fields_From());
					this.manyToManyMaps.add(manyToManyMap);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'obtenci de la informaci de les relacions un a un.
			throw new MyFrameUOCException(MappingMsg.GETMANYTOMANYANNOTATIONDATA_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Permet obtenir el nom de l'entitat.
	 * 
	 * @return Un <code>String</code> amb el nom de l'entitat.
	 */
	public String getName(){
		return (this.name);
	}
	
	/**
	 * Permet obtenir el nom de la taula que est associada a l'entitat.
	 * 
	 * @return Un <code>String</code> amb el nom de la taula associada a l'entitat.
	 */
	public String getTable(){
		return (this.table);
	}
	
	/*
	 * Construeix totes les sentncies SQL preparades que sn necessries per a emmagatzemar, llegir, actualitzar i esborrar
	 * entitats de la base de dades. 
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * 
	 */
	private void prepareQueries(Session session) throws MyFrameUOCException{		
		try{
			//Construir la sentncia CREATE TABLE per a poder crear la taula en la BD associada a l'entitat.
			this.prepareCreateTable(session);

			//Construir la sentncia CREATE TABLE per a poder cerar la taula en la BD per sense les restriccions de clau forana.
			this.pepareCreateTableNoFK(session);
			
			//Construir la sentncia ALTER TABLE que permet modificar la taula per afegir-hi les restriccions de clau forana.
			this.prepareAlterTableAddFK(session);
			
			//Cconstuir la sentncia ALTER TALBE que permet modificar la taula per treure les restriccions de clau forana.
			this.prepareAlterTableDropFK(session);
			
			//Construir la sentncia INSERT per a inserir un registre en la taula de l'entitat sense tenir en compte els valors de les claus foranes. 
			this.prepareInsertNoFK(session);
			
			//Construir la sentncia UPDATE per a actualitzar noms els valors de les claus foranes d'un registre de la taula de l'entitat.
			this.prepareUpdateOnlyFK(session);
			
			//Construir la sentncia SELECT per a llegir de la BD un objecte de l'entitat.
			this.prepareRead(session);

			//Construir la sentnca UPDATE per a actualitzar en la BD un objecte de l'entitat.
			this.prepareUpdate(session);

			//Construir la sentncia DELETE per a esborrar de la BD un objecte de l'entitat.
			this.prepareDelete(session);
			
			//Construir la sentncia DROP per a esborrar la taula de l'entitat de la BD.
			this.prepareDrop(session);
			
			//Constuir les sentncies associades a les relacions un a un.
			this.prepareOneToOneQueries(session);
			
			//Construir les sentncies associades a les relacions un a molts.
			this.prepareOneToManyQueries(session);
			
			//Construir les sentncies associades a les relacions molts a molts.
			this.prepareManyToManyQueries(session);
		}
		catch (Exception e){
			//Error => No s'han pogut obtenir les sentncies SQL preparades.
			throw new MyFrameUOCException(MappingMsg.PREPARAR_QUERIES_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies associades a les relacions molts a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareManyToManyQueries(Session session) throws MyFrameUOCException{
		try{
			this.prepareManyToManyCreateTable(session);
			this.prepareManyToManyDrop(session);
			this.prepareManyToManyInsert(session);
			this.prepareManyToManyRead(session);
			this.prepareManyToManyReadCollection(session);
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de les sentncies de les relacions molts a motls de l'entitat.
			throw new MyFrameUOCException(MappingMsg.PREPAREMANYTOMANYQUERIES_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies CREATE TABLE de les relacions de molts a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareManyToManyCreateTable(Session session) throws MyFrameUOCException{
		CreateTable createTable					= null;
		int elements;
		PrimaryKeyMap pkm						= null;
		ArrayList<String> fieldsFKTo			= null;
		ArrayList<String> fieldsFKFrom			= null;
		ArrayList<String> propertiesFKTo		= null;
		ArrayList<String> propertiesFKFrom		= null;
		Integer precision						= null;
		Integer scale							= null;
		Integer length							= null;
		ArrayList<FieldValuePair> pairs			= null;
		RegisteredEntity dest					= null;
		ArrayList<String> fields				= null;
		
		try{
			if (this.hasManyToManyMaps()){
				pairs = new ArrayList<FieldValuePair>();
				fields = new ArrayList<String>();
				for (ManyToManyMap mtmm: this.manyToManyMaps){
					createTable = CreateTable.getCreateTable(session, mtmm.getTable());
					pairs.clear();
					fields.clear();
					if (mtmm.hasFieldsFKTo()){
						fieldsFKTo = mtmm.getFieldsFKTo();
						elements = fieldsFKTo.size();
						propertiesFKTo = mtmm.getPropertiesFKTo();
						dest = mtmm.getEntity();
						for (int i = 0; i < elements; i++){
							pkm = dest.findPrimaryKeyMap(propertiesFKTo.get(i));
							if (pkm != null){
								precision = pkm.getPrecision();
								scale = pkm.getScale();
								length = pkm.getLength();
								createTable.addColumn(fieldsFKTo.get(i), pkm.getTypeMap().getMappingType(precision, scale, length));
								pairs.add(new FieldValuePair(fieldsFKTo.get(i), pkm.getField()));
								fields.add(fieldsFKTo.get(i));
							}
						}
						createTable.addFKConstraintFromPairs(mtmm.getNameFKTo(), dest.table, pairs);
						//createTable.addConstraintFromNameFields(mtmm.getNameFKTo() + "_PK", ConstraintType.PRIMARY_KEY, fields);
					}
					else{
						//Error => La relaci no t definit la correspondncia entre els camps de la FK i els atributs de l'entitat dest.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
					}
					if (mtmm.hasFieldsFKFrom()){
						fieldsFKFrom = mtmm.getFieldsFKFrom();
						elements = fieldsFKFrom.size();
						propertiesFKFrom = mtmm.getPropertiesFKFrom();
						pairs.clear();
						for (int i = 0; i < elements; i++){
							pkm = this.findPrimaryKeyMap(propertiesFKFrom.get(i));
							if (pkm != null){
								precision = pkm.getPrecision();
								scale = pkm.getScale();
								length = pkm.getLength();
								createTable.addColumn(fieldsFKFrom.get(i), pkm.getTypeMap().getMappingType(precision, scale, length));
								pairs.add(new FieldValuePair(fieldsFKFrom.get(i), pkm.getField()));
								fields.add(fieldsFKFrom.get(i));
							}
						}
						createTable.addFKConstraintFromPairs(mtmm.getNameFKFrom(), this.table, pairs);
						createTable.addConstraintFromNameFields(mtmm.getNameFKFrom() + "_PK", ConstraintType.PRIMARY_KEY, fields);
					}
					else{
						//Error => La relaci no t definit la correspondncia entre els camps de la FK i els atributs de l'entitat origen.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
					}
					mtmm.setPreparedSqlCreateTable(createTable);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la construcci de les sentncies CREATE TABLE de les relacions d'un a un.
			throw new MyFrameUOCException(MappingMsg.MANYTOMANYCREATETABLE_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies DROP TABLE de les relacions de molts a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza
	 */
	private void prepareManyToManyDrop(Session session) throws MyFrameUOCException{
		Drop drop		= null;
		
		try{
			if (this.hasManyToManyMaps()){
				for (ManyToManyMap mtmm: this.manyToManyMaps){
					drop = Drop.getDrop(session, mtmm.getTable());
					mtmm.setPreparedSqlDrop(drop);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de les sentncies DROP TABLE de les relacions d'un a molts de l'entitat.
			throw new MyFrameUOCException(MappingMsg.PREPAREMANYTOMANYDROP_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies INSERT de les relacions de molts a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareManyToManyInsert(Session session) throws MyFrameUOCException{
		Insert insert						= null;
		ArrayList<String> fieldsFKTo		= null;
		ArrayList<String> propertiesFKTo	= null;
		ArrayList<String> fieldsFKFrom		= null;
		ArrayList<String> propertiesFKFrom	= null;
		String value						= null;
		int elements;
		String param						= MyFrameUOC.PARAM_STATEMENT;
		String sep							= MyFrameUOC.SEPARATOR_PARAM_STATEMENT;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.PREPAREMANYTOMANYINSERT_SENSE_SESSIO);
			}
			if (this.hasManyToManyMaps()){
				for (ManyToManyMap mtmm: this.manyToManyMaps){
					insert = Insert.getInsert(session, mtmm.getTable());
					fieldsFKTo = mtmm.getFieldsFKTo();
					propertiesFKTo = mtmm.getPropertiesFKTo();
					fieldsFKFrom = mtmm.getFieldsFKFrom();
					propertiesFKFrom = mtmm.getPropertiesFKFrom();
					if (mtmm.hasFieldsFKTo()){
						elements = fieldsFKTo.size();
						for (int i = 0; i < elements; i++){
							value = param + "Collection" + sep + mtmm.getEntity().name + sep + propertiesFKTo.get(i) + param;
							insert.addFieldValuePair(fieldsFKTo.get(i), value);
						}
					}
					else{
						//Error => La correspondncia entre els camps FK i els atributs de l'entitat dest sn incorrectes.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
					}
					if (mtmm.hasFieldsFKFrom()){
						elements = fieldsFKFrom.size();
						for (int i = 0; i < elements; i++){
							value = param + this.name + sep + propertiesFKFrom.get(i) + param;
							insert.addFieldValuePair(fieldsFKFrom.get(i), value);
						}
					}
					else{
						//Error => La correspondncia entre els camps FK i els atributs de l'entitat origen sn incorrectes.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
					}
					mtmm.setPreparedSqlInsert(insert);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de les sentncies INSERT de les relacions un a un de l'entitat.
			throw new MyFrameUOCException(MappingMsg.PREPAREMANYTOMANYINSERT_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies SELECT de les relacions molts a molts de l'entitat que
	 * permete obtenir els registres de la collecci de la relaci.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareManyToManyReadCollection(Session session) throws MyFrameUOCException{
		Select select						= null;
		ArrayList<String> fieldsFKTo		= null;
		ArrayList<String> fieldsFKFrom		= null;
		Restriction restriction				= null;
		int elements;
		String paramValue					= MyFrameUOC.PARAM_VALUE;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.PREPAREMANYTOMANYREADCOLLECTION_SENSE_SESSIO);
			}
			if (this.hasManyToManyMaps()){
				for (ManyToManyMap mtmm: this.manyToManyMaps){
					select = Select.getSelect(session, mtmm.getTable());
					if (mtmm.hasFieldsFKTo()){
						fieldsFKTo = mtmm.getFieldsFKTo();
						elements = fieldsFKTo.size();
						for (int i = 0; i < elements; i++){
							select.addField(fieldsFKTo.get(i));
						}
					}
					else{
						//Error => La correspondncia entre els camps de la clau forana i els atributs de l'entitat dest no estn definitis.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
					}
					if (mtmm.hasFieldsFKFrom()){
						fieldsFKFrom = mtmm.getFieldsFKFrom();
						elements = fieldsFKFrom.size();
						restriction = new Restriction(RelationalOperator.EQUAL, fieldsFKFrom.get(0), paramValue);
						select.addField(fieldsFKFrom.get(0));
						for (int i = 1; i < elements; i++){
							restriction.addCondition(LogicalOperator.AND, RelationalOperator.EQUAL, fieldsFKFrom.get(i), paramValue);
							select.addField(fieldsFKFrom.get(i));
						}
					}
					else{
						//Error => La correspondncia entre els camps de la clau forna i els atributs de l'entitat origen no estn definits.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
					}
					select.addRestriction(restriction);
					mtmm.setPreparedSqlReadCollection(select);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de les sentncies SELECT de les relacions un a un.
			throw new MyFrameUOCException(MappingMsg.PREPAREMANYTOMANYREADCOLLECTION_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies SELECT de les relacions de molts a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareManyToManyRead(Session session) throws MyFrameUOCException{
		Select select						= null;
		ArrayList<String> fieldsFKTo		= null;
		ArrayList<String> fieldsFKFrom		= null;
		Restriction restriction				= null;
		int elements;
		String paramValue					= MyFrameUOC.PARAM_VALUE;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.PREPAREMANYTOMANYREAD_SENSE_SESSIO);
			}
			if (this.hasManyToManyMaps()){
				for (ManyToManyMap mtmm: this.manyToManyMaps){
					select = Select.getSelect(session, mtmm.getTable());
					if (mtmm.hasFieldsFKTo()){
						fieldsFKTo = mtmm.getFieldsFKTo();
						elements = fieldsFKTo.size();
						restriction = new Restriction(RelationalOperator.EQUAL, fieldsFKTo.get(0), paramValue);
						select.addField(fieldsFKTo.get(0));
						for (int i = 1; i < elements; i++){
							restriction.addCondition(LogicalOperator.AND, RelationalOperator.EQUAL, fieldsFKTo.get(i), paramValue);
							select.addField(fieldsFKTo.get(i));
						}
					}
					else{
						//Error => La correspondncia entre els camps de la clau forana i els atributs de l'entitat dest no estn definitis.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
					}
					if (mtmm.hasFieldsFKFrom()){
						fieldsFKFrom = mtmm.getFieldsFKFrom();
						elements = fieldsFKFrom.size();
						for (int i = 0; i < elements; i++){
							restriction.addCondition(LogicalOperator.AND, RelationalOperator.EQUAL, fieldsFKFrom.get(i), paramValue);
							select.addField(fieldsFKFrom.get(i));
						}
					}
					else{
						//Error => La correspondncia entre els camps de la clau forna i els atributs de l'entitat origen no estn definits.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
					}
					select.addRestriction(restriction);
					mtmm.setPreparedSqlRead(select);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de les sentncies SELECT de les relacions un a un.
			throw new MyFrameUOCException(MappingMsg.PREPAREMANYTOMANYREAD_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies associades a les reslacions un a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareOneToManyQueries(Session session) throws MyFrameUOCException{
		try{
			this.prepareOneToManyCreateTable(session);
			this.prepareOneToManyDrop(session);
			this.prepareOneToManyInsert(session);
			this.prepareOneToManyRead(session);
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de les sentncies de les relacions un a molts.
			throw new MyFrameUOCException(MappingMsg.PREPAREONETOMANYQUERIES_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies CREATE TABLE de les relacions d'un a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareOneToManyCreateTable(Session session) throws MyFrameUOCException{
		CreateTable createTable					= null;
		int elements;
		PrimaryKeyMap pkm						= null;
		ArrayList<String> fieldsFKTo			= null;
		ArrayList<String> fieldsFKFrom			= null;
		ArrayList<String> propertiesFKTo		= null;
		ArrayList<String> propertiesFKFrom		= null;
		Integer precision						= null;
		Integer scale							= null;
		Integer length							= null;
		ArrayList<FieldValuePair> pairs			= null;
		RegisteredEntity dest					= null;
		ArrayList<String> fields				= null;
		
		try{
			if (this.hasOneToManyMaps()){
				pairs = new ArrayList<FieldValuePair>();
				fields = new ArrayList<String>();
				for (OneToManyMap otmm: this.oneToManyMaps){
					createTable = CreateTable.getCreateTable(session, otmm.getTable());
					if (otmm.hasFieldsFKTo()){
						fieldsFKTo = otmm.getFieldsFKTo();
						elements = fieldsFKTo.size();
						propertiesFKTo = otmm.getPropertiesFKTo();
						dest = otmm.getEntity();
						pairs.clear();
						fields.clear();
						for (int i = 0; i < elements; i++){
							pkm = dest.findPrimaryKeyMap(propertiesFKTo.get(i));
							if (pkm != null){
								precision = pkm.getPrecision();
								scale = pkm.getScale();
								length = pkm.getLength();
								createTable.addColumn(fieldsFKTo.get(i), pkm.getTypeMap().getMappingType(precision, scale, length));
								pairs.add(new FieldValuePair(fieldsFKTo.get(i), pkm.getField()));
								fields.add(fieldsFKTo.get(i));
							}
						}
						createTable.addFKConstraintFromPairs(otmm.getNameFKTo(), dest.table, pairs);
						createTable.addConstraintFromNameFields(otmm.getNameFKTo() + "_PK", ConstraintType.PRIMARY_KEY, fields);
					}
					else{
						//Error => La relaci no t definit la correspondncia entre els camps de la FK i els atributs de l'entitat dest.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
					}
					if (otmm.hasFieldsFKFrom()){
						fieldsFKFrom = otmm.getFieldsFKFrom();
						elements = fieldsFKFrom.size();
						propertiesFKFrom = otmm.getPropertiesFKFrom();
						pairs.clear();
						for (int i = 0; i < elements; i++){
							pkm = this.findPrimaryKeyMap(propertiesFKFrom.get(i));
							if (pkm != null){
								precision = pkm.getPrecision();
								scale = pkm.getScale();
								length = pkm.getLength();
								createTable.addColumn(fieldsFKFrom.get(i), pkm.getTypeMap().getMappingType(precision, scale, length));
								pairs.add(new FieldValuePair(fieldsFKFrom.get(i), pkm.getField()));
							}
						}
						createTable.addFKConstraintFromPairs(otmm.getNameFKFrom(), this.table, pairs);
					}
					else{
						//Error => La relaci no t definit la correspondncia entre els camps de la FK i els atributs de l'entitat origen.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
					}
					otmm.setPreparedSqlCreateTable(createTable);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la construcci de les sentncies CREATE TABLE de les relacions d'un a un.
			throw new MyFrameUOCException(MappingMsg.ONETOMANYCREATETABLE_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies INSERT de les relacions d'un a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareOneToManyInsert(Session session) throws MyFrameUOCException{
		Insert insert						= null;
		ArrayList<String> fieldsFKTo		= null;
		ArrayList<String> propertiesFKTo	= null;
		ArrayList<String> fieldsFKFrom		= null;
		ArrayList<String> propertiesFKFrom	= null;
		String value						= null;
		int elements;
		String param						= MyFrameUOC.PARAM_STATEMENT;
		String sep							= MyFrameUOC.SEPARATOR_PARAM_STATEMENT;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.PREPAREONETOMANYINSERT_SENSE_SESSIO);
			}
			if (this.hasOneToManyMaps()){
				for (OneToManyMap otmm: this.oneToManyMaps){
					insert = Insert.getInsert(session, otmm.getTable());
					fieldsFKTo = otmm.getFieldsFKTo();
					propertiesFKTo = otmm.getPropertiesFKTo();
					fieldsFKFrom = otmm.getFieldsFKFrom();
					propertiesFKFrom = otmm.getPropertiesFKFrom();
					if (otmm.hasFieldsFKTo()){
						elements = fieldsFKTo.size();
						for (int i = 0; i < elements; i++){
							value = param + "Collection" + sep + otmm.getEntity().name + sep + propertiesFKTo.get(i) + param;
							insert.addFieldValuePair(fieldsFKTo.get(i), value);
						}
					}
					else{
						//Error => La correspondncia entre els camps FK i els atributs de l'entitat dest sn incorrectes.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
					}
					if (otmm.hasFieldsFKFrom()){
						elements = fieldsFKFrom.size();
						for (int i = 0; i < elements; i++){
							value = param + this.name + sep + propertiesFKFrom.get(i) + param;
							insert.addFieldValuePair(fieldsFKFrom.get(i), value);
						}
					}
					else{
						//Error => La correspondncia entre els camps FK i els atributs de l'entitat origen sn incorrectes.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
					}
					otmm.setPreparedSqlInsert(insert);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de les sentncies INSERT de les relacions un a un de l'entitat.
			throw new MyFrameUOCException(MappingMsg.PREPAREONETOMANYINSERT_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies READ de les relacions d'un a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareOneToManyRead(Session session) throws MyFrameUOCException{
		Select select						= null;
		ArrayList<String> fieldsFKFrom		= null;
		Restriction restriction				= null;
		int elements;
		String paramValue					= MyFrameUOC.PARAM_VALUE;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.PREPAREONETOMANYREAD_SENSE_SESSIO);
			}
			if (this.hasOneToManyMaps()){
				for (OneToManyMap otmm: this.oneToManyMaps){
					select = Select.getSelect(session, otmm.getTable());
					if (otmm.hasFieldsFKTo()){
						for (String field: otmm.getFieldsFKTo()){
							select.addField(field);
						}
					}
					else{
						//Error => La correspondncia entre els camps de la clau forana i els atributs de l'entitat dest no estn definitis.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
					}
					if (otmm.hasFieldsFKFrom()){
						fieldsFKFrom = otmm.getFieldsFKFrom();
						elements = fieldsFKFrom.size();
						restriction = new Restriction(RelationalOperator.EQUAL, fieldsFKFrom.get(0), paramValue);
						select.addField(fieldsFKFrom.get(0));
						for (int i = 1; i < elements; i++){
							restriction.addCondition(LogicalOperator.AND, RelationalOperator.EQUAL, fieldsFKFrom.get(i), paramValue);
							select.addField(fieldsFKFrom.get(i));
						}
					}
					else{
						//Error => La correspondncia entre els camps de la clau forna i els atributs de l'entitat origen no estn definits.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
					}
					select.addRestriction(restriction);
					otmm.setPreparedSqlRead(select);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de les sentncies SELECT de les relacions un a un.
			throw new MyFrameUOCException(MappingMsg.PREPAREONETOMANYREAD_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies DROP TABLES de les relacions d'un a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareOneToManyDrop(Session session) throws MyFrameUOCException{
		Drop drop		= null;
		
		try{
			if (this.hasOneToManyMaps()){
				for (OneToManyMap otmm: this.oneToManyMaps){
					drop = Drop.getDrop(session, otmm.getTable());
					otmm.setPreparedSqlDrop(drop);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de les sentncies DROP TABLE de les relacions d'un a molts de l'entitat.
			throw new MyFrameUOCException(MappingMsg.PREPAREONETOMANYDROP_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies associades a les relacions un a un de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareOneToOneQueries(Session session) throws MyFrameUOCException{
		try{
			this.prepareOneToOneCreateTable(session);
			this.prepareOneToOneInsert(session);
			this.prepareOneToOneRead(session);
			this.prepareOneToOneDrop(session);
		}
		catch (Exception e){
			//Error => S'ha produt un erro en la construcci de les sentncies de les relacions un a un.
			throw new MyFrameUOCException(MappingMsg.PREPAREONETOONEQUERIES_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies CREATE TABLE que serveixen per a crear les taules que s'utilitzen per
	 * a implementar les relacions d'un a un.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareOneToOneCreateTable(Session session) throws MyFrameUOCException{
		CreateTable createTable					= null;
		int elements;
		PrimaryKeyMap pkm						= null;
		ArrayList<String> fieldsFKTo			= null;
		ArrayList<String> fieldsFKFrom			= null;
		ArrayList<String> propertiesFKTo		= null;
		ArrayList<String> propertiesFKFrom		= null;
		Integer precision						= null;
		Integer scale							= null;
		Integer length							= null;
		ArrayList<FieldValuePair> pairs			= null;
		RegisteredEntity dest					= null;
		ArrayList<String> fields				= null;
		
		try{
			if (this.hasOneToOneMaps()){
				pairs = new ArrayList<FieldValuePair>();
				fields = new ArrayList<String>();
				for (OneToOneMap otom: this.oneToOneMaps){
					pairs.clear();
					fields.clear();
					createTable = CreateTable.getCreateTable(session, otom.getTable());
					if (otom.hasFieldsFKTo()){
						fieldsFKTo = otom.getFieldsFKTo();
						elements = fieldsFKTo.size();
						propertiesFKTo = otom.getPropertiesFKTo();
						dest = otom.getEntity();
						for (int i = 0; i < elements; i++){
							pkm = dest.findPrimaryKeyMap(propertiesFKTo.get(i));
							if (pkm != null){
								precision = pkm.getPrecision();
								scale = pkm.getScale();
								length = pkm.getLength();
								createTable.addColumn(fieldsFKTo.get(i), pkm.getTypeMap().getMappingType(precision, scale, length));
								pairs.add(new FieldValuePair(fieldsFKTo.get(i), pkm.getField()));
								fields.add(fieldsFKTo.get(i));
							}
						}
						createTable.addFKConstraintFromPairs(otom.getNameFKTo(), dest.table, pairs);
						createTable.addConstraintFromNameFields(otom.getNameFKTo() + "_UNIQUE", ConstraintType.UNIQUE, fields);
					}
					else{
						//Error => La relaci no t definit la correspondncia entre els camps de la FK i els atributs de l'entitat dest.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
					}
					if (otom.hasFieldsFKFrom()){
						fieldsFKFrom = otom.getFieldsFKFrom();
						elements = fieldsFKFrom.size();
						propertiesFKFrom = otom.getPropertiesFKFrom();
						pairs.clear();
						fields.clear();
						for (int i = 0; i < elements; i++){
							pkm = this.findPrimaryKeyMap(propertiesFKFrom.get(i));
							if (pkm != null){
								precision = pkm.getPrecision();
								scale = pkm.getScale();
								length = pkm.getLength();
								createTable.addColumn(fieldsFKFrom.get(i), pkm.getTypeMap().getMappingType(precision, scale, length));
								pairs.add(new FieldValuePair(fieldsFKFrom.get(i), pkm.getField()));
								fields.add(fieldsFKFrom.get(i));
								
							}
						}
						createTable.addFKConstraintFromPairs(otom.getNameFKFrom(), this.table, pairs);
						createTable.addConstraintFromNameFields(otom.getNameFKFrom() + "_PK", ConstraintType.PRIMARY_KEY, fields);
					}
					else{
						//Error => La relaci no t definit la correspondncia entre els camps de la FK i els atributs de l'entitat origen.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
					}
					otom.setPreparedSqlCreateTable(createTable);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la construcci de les sentncies CREATE TABLE de les relacions d'un a un.
			throw new MyFrameUOCException(MappingMsg.ONETOONECREATETABLE_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies INSERT que serveixen per a inserir un registre en les taules que
	 * s'utilitzen per a implementar les relacions d'un a un.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareOneToOneInsert(Session session) throws MyFrameUOCException{
		Insert insert						= null;
		ArrayList<String> fieldsFKTo		= null;
		ArrayList<String> propertiesFKTo	= null;
		ArrayList<String> fieldsFKFrom		= null;
		ArrayList<String> propertiesFKFrom	= null;
		String value						= null;
		int elements;
		String param						= MyFrameUOC.PARAM_STATEMENT;
		String sep							= MyFrameUOC.SEPARATOR_PARAM_STATEMENT;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.PREPAREONETOONEINSERT_SENSE_SESSIO);
			}
			if (this.hasOneToOneMaps()){
				for (OneToOneMap otom: this.oneToOneMaps){
					insert = Insert.getInsert(session, otom.getTable());
					fieldsFKTo = otom.getFieldsFKTo();
					propertiesFKTo = otom.getPropertiesFKTo();
					fieldsFKFrom = otom.getFieldsFKFrom();
					propertiesFKFrom = otom.getPropertiesFKFrom();
					if (otom.hasFieldsFKTo()){
						elements = fieldsFKTo.size();
						for (int i = 0; i < elements; i++){
							value = param + this.name + sep + otom.getProperty() + sep + otom.getEntity().name + sep + propertiesFKTo.get(i) + param;
							insert.addFieldValuePair(fieldsFKTo.get(i), value);
						}
					}
					else{
						//Error => La correspondncia entre els camps FK i els atributs de l'entitat dest sn incorrectes.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
					}
					if (otom.hasFieldsFKFrom()){
						elements = fieldsFKFrom.size();
						for (int i = 0; i < elements; i++){
							value = param + this.name + sep + propertiesFKFrom.get(i) + param;
							insert.addFieldValuePair(fieldsFKFrom.get(i), value);
						}
					}
					else{
						//Error => La correspondncia entre els camps FK i els atributs de l'entitat origen sn incorrectes.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
					}
					otom.setPreparedSqlInsert(insert);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de les sentncies INSERT de les relacions un a un de l'entitat.
			throw new MyFrameUOCException(MappingMsg.PREPAREONETOONEINSERT_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies SELECT que serveixen per a llegir un registre de les taules
	 * que s'utilitzen per a implementar les relacions d'un a un.
	 */
	private void prepareOneToOneRead(Session session) throws MyFrameUOCException{
		Select select						= null;
		ArrayList<String> fieldsFKFrom		= null;
		Restriction restriction				= null;
		int elements;
		String paramValue					= MyFrameUOC.PARAM_VALUE;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.PREPAREONETOONEREAD_SENSE_SESSIO);
			}
			if (this.hasOneToOneMaps()){
				for (OneToOneMap otom: this.oneToOneMaps){
					select = Select.getSelect(session, otom.getTable());
					if (otom.hasFieldsFKTo()){
						for (String field: otom.getFieldsFKTo()){
							select.addField(field);
						}
					}
					else{
						//Error => La correspondncia entre els camps de la clau forana i els atributs de l'entitat dest no estn definitis.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
					}
					if (otom.hasFieldsFKFrom()){
						fieldsFKFrom = otom.getFieldsFKFrom();
						elements = fieldsFKFrom.size();
						restriction = new Restriction(RelationalOperator.EQUAL, fieldsFKFrom.get(0), paramValue);
						select.addField(fieldsFKFrom.get(0));
						for (int i = 1; i < elements; i++){
							restriction.addCondition(LogicalOperator.AND, RelationalOperator.EQUAL, fieldsFKFrom.get(i), paramValue);
							select.addField(fieldsFKFrom.get(i));
						}
					}
					else{
						//Error => La correspondncia entre els camps de la clau forna i els atributs de l'entitat origen no estn definits.
						throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
					}
					select.addRestriction(restriction);
					otom.setPreparedSqlRead(select);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de les sentncies SELECT de les relacions un a un.
			throw new MyFrameUOCException(MappingMsg.PREPAREONETOONEREAD_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies DROP TABLE que serveixen per a esborrar les taules
	 * que s'utilitzen per a implementar les relacions un a un de l'entitat.
	 */
	private void prepareOneToOneDrop(Session session) throws MyFrameUOCException{
		Drop drop		= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.PREPAREONETOONEDROP_SENSE_SESSIO);
			}
			if (this.hasOneToOneMaps()){
				for (OneToOneMap otom: this.oneToOneMaps){
					drop = Drop.getDrop(session, otom.getTable());
					otom.setPreparedSqlDrop(drop);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de la sentncia prepareOneToOneDrop.
			throw new MyFrameUOCException(MappingMsg.PREPAREONETOONEDROP_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix la sentncia CREATE TABLE que s'ha d'executar per a crear la taula de l'entitat en
	 * la base de dades.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareCreateTable(Session session) throws MyFrameUOCException{
		CreateTable createTable		= null;		
		
		try{
			createTable = CreateTable.getCreateTable(session, table);

			//Afegir la definici de les columnes en la sentncia CREATE TABLE.
			this.prepareCreateTableColumns(session, createTable);

			//Afegir la definici de la restricci de clau primria.
			this.prepareCreateTablePK(createTable);

			//Afegir la definici de les restriccions de claus foranes.
			this.prepareCreateTableFK(createTable);

			this.preparedSqlCreateTable = createTable;
		}
		catch (Exception e){
			//Error => S'ha produt un erro en la preparaci de la sentncia CREATE TABLE.
			throw new MyFrameUOCException(MappingMsg.PREPARAR_CREATE_TABLE_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix la sentncia CREATE TABLE preparada que s'ha d'executar per a crear la taula
	 * de l'enitat per sense les restriccions de clau forana.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void pepareCreateTableNoFK(Session session) throws MyFrameUOCException{
		try{
			if (this.preparedSqlCreateTable != null){
				this.preparedSqlCreateTableNoFK = this.preparedSqlCreateTable.copyNoFK(session);
			}
			else{
				//Error => No es pot crear la sentncia preparedSqoCreateTableNoFK sense la sentncia preparedSqlCreateTable
				throw new MyFrameUOCException(MappingMsg.PREPAREDCREATETABLENOFK_SENSE_PREPAREDSQLCREATETABLE);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la construcci de la sentncia CREATE TABLE sense les restriccions de clau forana.
			throw new MyFrameUOCException(MappingMsg.PREPARAR_CREATE_TABLE_NOFK_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix la sentncia ALTER TABLE preparada que s'ha d'executar per a modificar
	 * la taula de l'entitat per afegir-hi les restriccions de clau forana.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareAlterTableAddFK(Session session) throws MyFrameUOCException{
		AlterTable alterTable										= null;
		ArrayList<ForeignKeyConstraint> foreignKeyConstraints		= null;
		
		try{
			if (this.preparedSqlCreateTable != null){
				alterTable = AlterTable.getAlterTable(session, this.table, AlterTableType.ADD);
				foreignKeyConstraints = this.preparedSqlCreateTable.getFKConstraints();
				if (foreignKeyConstraints != null && foreignKeyConstraints.size() > 0){
					for (ForeignKeyConstraint foreignKeyConstraint: foreignKeyConstraints){
						alterTable.addFKConstraint(foreignKeyConstraint);
					}
					this.preparedSqlAlterTableAddFK = alterTable;
				}
			}
			else{
				//Error => No es pot construir la sentncia preparedAlterTable sense la sentncia preparedSqlCreateTable.
				throw new MyFrameUOCException(MappingMsg.PREAPREDALTERTABLE_SENSE_PREPAREDSQLCREATETABLE);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la preparaci de la sentncia ALTER TABLE.
			throw new MyFrameUOCException(MappingMsg.PREPARAR_ALTER_TABLE_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Construeix la sentncia ALTER TABLE per a treure les restriccions de clau forana
	 * de la taula de l'entitat.
	 * 
	 * @param session - Sessin de MyFrameUOC que s'utilitza.
	 */
	private void prepareAlterTableDropFK(Session session) throws MyFrameUOCException{
		AlterTable alterTable				= null;
		
		
		try{
			if (session == null){
				//Error => No hi ha una sessi inicialitzada.
				throw new MyFrameUOCException(MappingMsg.PREPAREALTERTABLEDROPFK_SENSE_SESSIO);
			}
			if (this.preparedSqlAlterTableAddFK != null){
				alterTable = AlterTable.getAlterTable(session, this.table, AlterTableType.DROP);
				
				for (ForeignKeyConstraint fkc: this.preparedSqlAlterTableAddFK.getForeignKeyConstraints()){
					alterTable.addFKConstraint(fkc);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error al preparar preparedSQLAlterTableDropFK.
			throw new MyFrameUOCException(MappingMsg.PREPAREALTERTABLEDROPFK_ERROR, e.getMessage());
		}
		this.preparedSqlAlterTableDropFK = alterTable;
	}
	
	/*
	 * Afegeix en la sentncia CREATE TABLE la definici de les columnes.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param createTable - Sentncia CREATE TABLE en la qual s'afegeix la definici de
	 * les columnes. Si es passa un nul, no s'afegeix res.
	 */
	private void prepareCreateTableColumns(Session session, CreateTable createTable) throws MyFrameUOCException{
		Integer precision		= null;
		Integer scale			= null;
		Integer length			= null;
		String definition		= null;
		Connection connection	= null;
		
		connection = session.getConnection();
		
		if (createTable != null){
			//Afegir la definici de les columnes.
			if (this.primaryKeyMaps != null){
				//Afegir una columna per a cada clau primria mapejada.
				for (PrimaryKeyMap primaryKeyMap: this.primaryKeyMaps){
					precision = primaryKeyMap.getPrecision();
					scale = primaryKeyMap.getScale();
					length = primaryKeyMap.getLength();
					definition = primaryKeyMap.getTypeMap().getMappingType(precision, scale, length);
					if (primaryKeyMap.getUnique()){
						//definition += " UNIQUE";
						definition += " " + connection.getUniqueConstraint();
					}
					createTable.addColumn(primaryKeyMap.getField(), definition);
				}
			}
			if (this.columnMaps != null){
				//Afegir una columna per a cada columna mapejada.
				for (ColumnMap columnMap: this.columnMaps){
					precision = columnMap.getPrecision();
					scale = columnMap.getScale();
					length = columnMap.getLength();
					
					definition = columnMap.getTypeMap().getMappingType(precision, scale, length);
					if (columnMap.getRequired()){
						//definition += " NOT NULL";
						definition += " " + connection.getNotNullConstraint();
					}
					if (columnMap.getUnique()){
						//definition += " UNIQUE";
						definition += " " + connection.getUniqueConstraint();
					}
					createTable.addColumn(columnMap.getField(),definition);
				}
			}
		}
	}
	
	/*
	 * Afegeix en la sentncia CREATE TABLE la definici de la restricci de la clau primria.
	 * 
	 *  @param createTable - Sentncia CREATE TABLE en la qual s'afegeix la definici de la restricci
	 *  de la clau primria. Si es passa un nul, no s'afegeix res.
	 */
	private void prepareCreateTablePK(CreateTable createTable) throws MyFrameUOCException{
		ArrayList<String> fields		= null;
		String cad						= null;
		String namePK					= null;
		
		if (createTable != null){
			if (this.primaryKeyMaps != null){
				fields = new ArrayList<String>();
				for (PrimaryKeyMap primaryKeyMap: this.primaryKeyMaps){
					cad = primaryKeyMap.getName();
					if (namePK == null || "".equals(namePK)){
							namePK = cad;
					}
					else{
						if (!namePK.equals(cad)){
							//Error => El nom de la clau primria d'una entitat s nic.
							throw new MyFrameUOCException(MappingMsg.CLAU_PRIMARIA_NO_UNICA);
						}
					}
					fields.add(primaryKeyMap.getField());
				}
				createTable.addConstraintFromNameFields(namePK, ConstraintType.PRIMARY_KEY, fields);
			}
		}
	}
	
	/*
	 * Afegeix en la sentncia CREATE TABLE la definici de les restriccions de les claus foranes.
	 * 
	 * @param createTable - Sentncia CREATE TABLE en la qual s'afegeix la definici de les
	 * restriccions de les claus foranes. Si es passa un nul, no s'afegeix res.
	 */
	private void prepareCreateTableFK(CreateTable createTable) throws MyFrameUOCException{
		ArrayList<FieldValuePair> pairs				= null;
		FieldValuePair pair							= null;
		String[] fieldPairs							= null;
		String[] fields								= null;
		RegisteredEntity entity						= null;
		ArrayList<PrimaryKeyMap> primaryKeyMaps		= null;
		PrimaryKeyMap primaryKeyMap					= null;
		boolean found;
		int elements;
		int i;
		String table								= null;
		Integer precision;
		Integer scale;
		Integer length;
		
		try{
			if (createTable != null){
				if (this.hasForeignKeyMaps()){
					pairs = new ArrayList<FieldValuePair>();
					for (ForeignKeyMap foreignKeyMap: this.foreignKeyMaps){
						pairs.clear();
						fieldPairs = foreignKeyMap.getFields().split(MyFrameUOC.SEPARATOR_FIELDS);
						entity = foreignKeyMap.getEntity();
						for (String field: fieldPairs){
							fields = field.split(MyFrameUOC.SEPARATOR_FIELD);
							primaryKeyMaps = entity.getPrimaryKeyMaps();
							if (primaryKeyMaps == null || primaryKeyMaps.size() == 0){
								//Error => L'entitat dest de la clau forana no t claus primries.
								throw new MyFrameUOCException(MappingMsg.CREATE_TABLE_NO_CLAUS_PRIMARIES);
							}
							elements = primaryKeyMaps.size();
							if (elements != fieldPairs.length){
								//Error => Falten camps per definir en la clau forana de l'entitat.
								throw new MyFrameUOCException(MappingMsg.CLAU_FORANA_FALTEN_CAMPS);
							}
							i = 0;
							found = false;
							while (i < elements && !found){
								primaryKeyMap = primaryKeyMaps.get(i);
								if (primaryKeyMap.getProperty().equals(fields[1])){
									pair = new FieldValuePair(fields[0], primaryKeyMap.getField());
									precision = primaryKeyMap.getPrecision();
									scale = primaryKeyMap.getScale();
									length = primaryKeyMap.getLength();
									createTable.addColumn(fields[0], primaryKeyMap.getTypeMap().getMappingType(precision, scale, length));
									found = true;
								}
								else{
									i++;
								}
							}
							if (!found){
								//Error => L'entitat dest no t la clau primria que s'ha indicat en l'anotaci de clau forana.
								throw new MyFrameUOCException(MappingMsg.CREATE_TABLE_CLAU_PRIMARIA_NO_TROBADA);
							}
							pairs.add(pair);
						}
						table = entity.getTable();
						createTable.addFKConstraintFromPairs(foreignKeyMap.getName(), table, pairs);
					}
				}
			}
		}
		catch (Exception e){
			//Error => No s'ha pogut afegir la definici de les claus foranes.
			throw new MyFrameUOCException(MappingMsg.CREATE_TABLE_NO_CLAU_FORANA, e.getMessage());
		}
	}
	
	/*
	 * Construeix les sentncies INSERT preparades que sn necessries executar per a emmagatzemar
	 * en la base de dades un objecte de l'entitat sense tenir en compte els valors de les claus foranes.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareInsertNoFK(Session session) throws MyFrameUOCException{
		Insert insert						= null;
		String param						= MyFrameUOC.PARAM_STATEMENT;
		String sep							= MyFrameUOC.SEPARATOR_PARAM_STATEMENT;
		String value						= null;
		
		insert = Insert.getInsert(session, this.table);
		
		if (this.primaryKeyMaps != null && this.primaryKeyMaps.size() > 0){
			for (PrimaryKeyMap primaryKeyMap: this.primaryKeyMaps){
				value = param + this.name + sep + primaryKeyMap.getProperty() + param;
				insert.addFieldValuePair(primaryKeyMap.getField(), value);
			}
		}
		
		if (this.columnMaps != null && this.columnMaps.size() > 0){
			for (ColumnMap columnMap: this.columnMaps){
				value = param + this.name + sep + columnMap.getProperty() + param;
				insert.addFieldValuePair(columnMap.getField(), value);
			}
		}
		this.preparedSqlInsertNoFK = insert;
	}
	
	/*
	 * Prepara la sentncia UPDATE que permet actualitzar els valors de les claus foranes
	 * d'un registre de la taula de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareUpdateOnlyFK(Session session) throws MyFrameUOCException{
		Update update							= null;
		ArrayList<String[]> mappedFields		= null;
		Restriction restriction					= null;
		int elements;
		PrimaryKeyMap primaryKeyMap				= null;
		String param							= MyFrameUOC.PARAM_STATEMENT;
		String sep								= MyFrameUOC.SEPARATOR_PARAM_STATEMENT;
		String value							= null;
		RegisteredEntity entity					= null;
		AddCondition addCondition				= null;
		
		try{
			if (this.primaryKeyMaps == null || this.primaryKeyMaps.size() == 0){
				//Error => L'entitat ha de tenir com a mnim una clau primria.
				throw new MyFrameUOCException(MappingMsg.PREAPREDALTERTABLE_SENSE_PREPAREDSQLCREATETABLE);
			}

			if (this.foreignKeyMaps != null && this.foreignKeyMaps.size() > 0){
				update = Update.getUpdate(session, this.table);
				for (ForeignKeyMap foreignKeyMap: this.foreignKeyMaps){
					mappedFields = foreignKeyMap.getMappedFields();
					entity = foreignKeyMap.getEntity();
					for (String[] mappedField: mappedFields){
						value = param + this.name + sep + foreignKeyMap.getProperty();
						entity = foreignKeyMap.getEntity();
						//value += param + entity.getName() + sep + mappedField[1]; 
						value += sep + entity.getName() + sep + mappedField[1] + param;
						update.addFieldValuePair(mappedField[0], value);
					}
				}

				elements = this.primaryKeyMaps.size();
				primaryKeyMap = this.primaryKeyMaps.get(0);
				//value = param + this.name + sep + primaryKeyMap.getProperty();
				value = param + this.name + sep + primaryKeyMap.getProperty() + param;
				restriction = new Restriction(RelationalOperator.EQUAL, primaryKeyMap.getField(), value);
				for (int i = 1; i < elements; i++){
					primaryKeyMap = this.primaryKeyMaps.get(i);
					//value = param + this.name + sep + primaryKeyMap.getProperty();
					value = param + this.name + sep + primaryKeyMap.getProperty() + param;
					addCondition = new AddCondition(LogicalOperator.AND, RelationalOperator.EQUAL, primaryKeyMap.getField(), value);
					restriction.addCondition(addCondition);
				}
				update.addRestriction(restriction);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la creaci de la sentncia SQL preparada preparedSqlUpdateOnlyFK.
			throw new MyFrameUOCException(MappingMsg.PREPAREDSQLUPDATEONLYFK_ERROR, e.getMessage());
		}
		
		this.preparedSqlUpdateOnlyFK = update;
	}
	
	/*
	 * Construeix les sentnces SELECT preparades que sn necessries executar per a llegir
	 * de la base de dades un objecte de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareRead(Session session) throws MyFrameUOCException{
		Select select					= null;
		Restriction restriction			= null;
		Condition condition				= null;
		PrimaryKeyMap primaryKeyMap		= null;
		int elements;
		
		select = Select.getSelect(session, this.table);
		
		if (this.hasPrimaryKeyMaps()){
			primaryKeyMap = primaryKeyMaps.get(0);
			//value = param + this.name + sep + primaryKeyMap.getProperty() + param;
			condition = new Condition(RelationalOperator.EQUAL, primaryKeyMap.getField(), "?");
			restriction = new Restriction(condition);
			select.addField(primaryKeyMap.getField());
			elements = this.primaryKeyMaps.size();
			
			for (int i = 1; i < elements; i++){
				primaryKeyMap = this.primaryKeyMaps.get(i);
				select.addField(primaryKeyMap.getField());
				//value = param + this.name + sep + primaryKeyMap.getProperty() + param;
				restriction.addCondition(LogicalOperator.AND, RelationalOperator.EQUAL, primaryKeyMap.getField(), "?");
			}
			select.addRestriction(restriction);
		}
				
		if (this.hasColumnMaps()){
			for (ColumnMap columnMap: this.columnMaps){
				select.addField(columnMap.getField());
			}
		}
		
		if (this.hasForeignKeyMaps()){
			for (ForeignKeyMap foreignKeyMap: this.foreignKeyMaps){
				for (String field: foreignKeyMap.getFieldsFK()){
					select.addField(field);
				}
			}
		}
		
		this.preparedSqlRead = select;
	}
	
	/*
	 * Construeix les sentncies UPDATE preparades que sn necessries executar per a actualitzar
	 * en la base de dades un objecte de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareUpdate(Session session) throws MyFrameUOCException{
		Update update					= null;
		String value					= null;
		String param					= MyFrameUOC.PARAM_STATEMENT;
		String sep						= MyFrameUOC.SEPARATOR_PARAM_STATEMENT;
		Restriction restriction			= null;
		PrimaryKeyMap primaryKeyMap		= null;
		int elements;
		int numProperties;
		ArrayList<String> properties	= null;
		ArrayList<String> fields		= null;
		
		try{
			if (this.hasPrimaryKeyMaps()){
				update = Update.getUpdate(session, this.table);

				elements = this.primaryKeyMaps.size();
				primaryKeyMap = this.primaryKeyMaps.get(0);
				value = param + this.name + sep + primaryKeyMap.getProperty() + param;
				restriction = new Restriction(RelationalOperator.EQUAL, primaryKeyMap.getField(), value);

				update.addFieldValuePair(primaryKeyMap.getField(), value);

				for (int i = 1; i < elements; i++){
					primaryKeyMap = this.primaryKeyMaps.get(i);
					value = param + this.name + sep + primaryKeyMap.getProperty() + param;
					restriction.addCondition(LogicalOperator.AND, RelationalOperator.EQUAL, primaryKeyMap.getField(), value);

					update.addFieldValuePair(primaryKeyMap.getField(), value);
				}
				update.addRestriction(restriction);

				if (this.hasColumnMaps()){
					for (ColumnMap columnMap: this.columnMaps){
						value = param + this.name + sep + columnMap.getProperty() + param;
						update.addFieldValuePair(columnMap.getField(), value);
					}
				}
				
				if (this.hasForeignKeyMaps()){
					for (ForeignKeyMap fkm: this.foreignKeyMaps){
						properties = fkm.getPropertiesFK();
						fields = fkm.getFieldsFK();
						numProperties = properties.size();
						for (int i = 0; i < numProperties; i++){
							value = param + this.name + sep + fkm.getProperty() + sep + fkm.getEntity().getName() + sep + properties.get(i) + param;
							update.addFieldValuePair(fields.get(i), value);
						}
					}
				}
			}
			else{
				//Error => L'entitat no t claus primries.
				throw new MyFrameUOCException(MappingMsg.PREPARESQLUPDATE_SENSE_PK);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en preparar la sentncia preparedSqlUpdate.
			throw new MyFrameUOCException(MappingMsg.PREPARESQLUPDATE_ERROR, e.getMessage());
		}
		this.preparedSqlUpdate = update;
	}
	
	/*
	 * Construeix les sentncies DELETE preparades que sn necessries executar per a esborrar
	 * de la base de dades un objecte de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareDelete(Session session) throws MyFrameUOCException{
		Delete delete					= null;
		Restriction restriction			= null;
		String value					= null;
		String param					= MyFrameUOC.PARAM_STATEMENT;
		String sep						= MyFrameUOC.SEPARATOR_PARAM_STATEMENT;
		String property					= null;
		PrimaryKeyMap primaryKeyMap		= null;
		int elements;
		
		if (this.primaryKeyMaps == null || this.primaryKeyMaps.size() == 0){
			//Error => No es pot construir la sentncia DELETE preparada si l'entitat no t claus primries.
			throw new MyFrameUOCException(MappingMsg.DELETE_SENSE_CLAUS_PRIMARIES);
		}
		
		delete = Delete.getDelete(session, this.table);
	
		elements = this.primaryKeyMaps.size();
		primaryKeyMap = this.primaryKeyMaps.get(0);
		property = primaryKeyMap.getProperty();
		value = param + this.name + sep + String.valueOf(property.charAt(0)).toUpperCase() + property.substring(1, property.length()) + param; 
		restriction = new Restriction(RelationalOperator.EQUAL, primaryKeyMap.getField(), value);
		
		for (int i = 1; i < elements; i++){
			primaryKeyMap = this.primaryKeyMaps.get(i);
			property = primaryKeyMap.getProperty();
			value = param + this.name + sep + String.valueOf(property.charAt(0)).toUpperCase() + property.substring(1, property.length()) + param;
			restriction.addCondition(LogicalOperator.AND, RelationalOperator.EQUAL, primaryKeyMap.getField(), value);
		}
		
		delete.addRestriction(restriction);
		
		this.preparedSqlDelete = delete;
	}
	
	/*
	 * Construeix la sentncia DROP TABLE que permet esborrar la taula de l'entitat.
	 *  
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void prepareDrop(Session session) throws MyFrameUOCException{
		Drop drop		= null;
		
		drop = Drop.getDrop(session, this.table);
		
		this.preparedSqlDrop = drop;
	}
	
	/**
	 * Permet obtenir les columnes mapejades de l'entitat.
	 * 
	 * @return Un <code>ArrayList</code> amb les columnes mapejades de l'entitat. 
	 */
	public ArrayList<ColumnMap> getColumnMaps(){
		return (this.columnMaps);
	}
	
	/**
	 * Permet obtenir les claus primries mapejades de l'entitat.
	 * 
	 * @return Un <code>ArrayList</code> amb les claus primries mapejades de l'entitat.
	 */
	public ArrayList<PrimaryKeyMap> getPrimaryKeyMaps(){
		return (this.primaryKeyMaps);
	}
	
	/**
	 * Permet obtenir les claus foranes mapejades de l'entitat.
	 * 
	 * @return Un <code>ArrayList</code> amb les claus foranes mapejades de l'entitat.
	 */
	public ArrayList<ForeignKeyMap> getForeignKeyMaps(){
		return (this.foreignKeyMaps);
	}
	
	/**
	 * Crea la taula de l'entitat i les taules de les seves relacions.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * 
	 * @exception MyFrameUOCException Si no es poden crear les taules de l'entitat i de les seves relacions.
	 */
	public void createEntityTables(Session session) throws MyFrameUOCException{
		String query											= null;
		Connection connection									= null;
		
		
		try{
			if (session == null){
				//Error => No es poden crear taules sense una sessi inicialitzada.
				throw new MyFrameUOCException(MappingMsg.CREACIO_TAULES_SENSE_SESSIO);
			}
			
			if (this.preparedSqlCreateTable == null){
				//Error => La sentncia CREATE TABLE de l'entitat no est preparada.
				throw new MyFrameUOCException(MappingMsg.CREACIO_TAULES_SENSE_CREATE_TABLE);
			}
			
			connection = session.getConnection();
			
			if (!connection.existsTable(this.table)){
				//Si la taula de l'entitat no existeix en la BD, llavors crear-la.
				
				if (this.hasForeignKeyMaps()){
					//L'entitat t claus foranes => crear la taula sense les restriccions de clau forana.
					query = this.preparedSqlCreateTableNoFK.getQuery();
					connection.executeSQLDDL(query);

					//Crear les taules de les claus foranes de l'entitat.
					for (ForeignKeyMap foreignKeyMap: this.foreignKeyMaps){
						foreignKeyMap.getEntity().createEntityTables(session);
					}
					
					//Modificar la taula de l'entitat per afegir-hi les restriccions de clau forana.
					query = this.preparedSqlAlterTableAddFK.getQuery();
					if (query != null){
						connection.executeSQLDDL(query);
					}
				}
				else{
					connection.executeSQLDDL(this.preparedSqlCreateTable.getQuery());
				}
				
				//Crear les taules de les relacions d'un a un.
				this.createOneToOneTables(session);
				
				//Crear les taules de les relacions d'un a molts.
				this.createOneToManyTables(session);
				
				//Crear les taules de les relacions de molts a molts.
				this.createManyToManyTables(session);	
			}
		}
		catch (Exception e){
			//Error => No s'han pogut crear les taules de l'entitat.
			throw new MyFrameUOCException(MappingMsg.CREACIO_TAULES_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Crea les taules necessries per a les relacions de molts a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void createManyToManyTables(Session session) throws MyFrameUOCException{
		RegisteredEntity dest		= null;
		Connection connection		= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.CREATEMANYTOMANYTABLES_SENSE_SESSIO);
			}
			connection = session.getConnection();
			if (this.hasManyToManyMaps()){
				for (ManyToManyMap mtmm: this.manyToManyMaps){
					dest = mtmm.getEntity();
					dest.createEntityTables(session);
					if (!connection.existsTable(mtmm.getTable())){
						connection.executeSQLDDL(mtmm.getPreparedSqlCreateTable().getQuery());
					}
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la creaci de les taules de les relacions de molts a molts de l'entitat.
			throw new MyFrameUOCException(MappingMsg.CREATEMANYTOMANYTABLES_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Crea les taules necessries per a les relacions d'un a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void createOneToManyTables(Session session) throws MyFrameUOCException{
		RegisteredEntity dest		= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.CREATEONETOMANYTABLES_SENSE_SESSIO);
			}
			if (this.hasOneToManyMaps()){
				for (OneToManyMap otmm: this.oneToManyMaps){
					dest = otmm.getEntity();
					dest.createEntityTables(session);
					session.getConnection().executeSQLDDL(otmm.getPreparedSqlCreateTable().getQuery());
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la creaci de les taules de les relacions d'un a molts de l'entitat.
			throw new MyFrameUOCException(MappingMsg.CREATEONETOMANYTABLES_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Crea les taules necessries per a les relacions un a un de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 */
	private void createOneToOneTables(Session session) throws MyFrameUOCException{
		RegisteredEntity dest		= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.CREATEONETOONETABLES_SENSE_SESSIO);
			}
			if (this.hasOneToOneMaps()){
				for (OneToOneMap otom: this.oneToOneMaps){
					dest = otom.getEntity();
					dest.createEntityTables(session);
					session.getConnection().executeSQLDDL(otom.getPreparedSqlCreateTable().getQuery());
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la creaci de les taules de les relacions un a un de l'entitat.
			throw new MyFrameUOCException(MappingMsg.CREATEONETOONETABLES_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Emmagatzema un objecte nou de l'entitat en la base de dades.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param obj - Objecte de l'entitat.
	 * 
	 * @throws MyFrameUOCException Si no es pot emmagatzemar l'objecte en la base de dades. 
	 */
	public void insert(Session session, Object obj) throws MyFrameUOCException{
		Object insertedObj				= null;
		boolean deleteInsertObjects		= false;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.INSERIR_ENTITAT_SENSE_SESSIO);
			}
			if (obj == null){
				//Error => No es pot inserir un objecte nul
				throw new MyFrameUOCException(MappingMsg.INSERT_SENSE_OBJECTE);
			}
			insertedObj = this.getInsertedObject(obj);
			if (insertedObj == null){
				this.addInsertedObject(obj);
				if (!RegisteredEntity.isInserting){
					RegisteredEntity.isInserting = true;
					deleteInsertObjects = true;
				}
				if (!existsObject(session, obj)){
					//Inserir un registre en la BD sense tenir en compte els valors de les claus foranes.
					session.getConnection().executeSQLDDL(this.preparedSqlInsertNoFK.getQuery(), obj);

					//Fer les insercions o actualitzacions per a les claus foranes.
					this.insertFK(session, obj);

					//Fer les insercions per a les relacions d'un a un.
					this.insertOneToOne(session, obj);

					//Fer les insercions per a les relacions d'un a molts.
					this.insertOneToMany(session, obj);

					//Fer les insercions per a les relacions de molts a molts.
					this.insertManyToMany(session, obj);
				}
				else{
					//Error => No es pot inserir un objecte que ja existeix en la BD.
					throw new MyFrameUOCException(MappingMsg.INSERT_OBJECTE_REPETIT);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la inserci d'un objecte.
			throw new MyFrameUOCException(MappingMsg.INSERT_ERROR, e.getMessage());
		}
		finally{
			if (deleteInsertObjects){
				if (this.hasInsertedObjects()){
					RegisteredEntity.insertedObjects.clear();
				}
				RegisteredEntity.isInserting = false;
			}
		}
	}
	
	/*
	 * Realizta la inserci dels registres de les relacions de molts a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param obj - Objecte que es vol emmagatzemar en la base de dades.
	 */
	private void insertManyToMany(Session session, Object obj) throws MyFrameUOCException{
		RegisteredEntity dest				= null;
		Collection<?> collection			= null;
		ArrayList<Object> pk				= new ArrayList<Object>();
		ArrayList<String> fieldsFKTo		= null;
		ArrayList<String> propertiesFKTo	= null;
		ArrayList<String> fieldsFKFrom		= null;
		ArrayList<String> propertiesFKFrom	= null;
		int elements;
		ResultSet rs						= null;
		Connection connection				= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.INSERTMANYTOMANY_SENSE_SESSIO);
			}
			connection = session.getConnection();
			if (obj != null){
				if (this.hasManyToManyMaps()){
					for (ManyToManyMap mtmm: this.manyToManyMaps){
						dest = mtmm.getEntity();
						collection = (Collection<?>) MyFrameUOC.getObjectValue(mtmm.getProperty(), obj);
						if (collection != null && collection.size() > 0){
							for (Object objCollection: collection){
								if (!dest.existsObject(session, objCollection)){
									dest.insert(session, objCollection);
								}
								pk.clear();
								if (mtmm.hasFieldsFKTo()){
									fieldsFKTo = mtmm.getFieldsFKTo();
									elements = fieldsFKTo.size();
									propertiesFKTo = mtmm.getPropertiesFKTo();
									for (int i = 0; i < elements; i++){
										pk.add(MyFrameUOC.getObjectValue(propertiesFKTo.get(i), objCollection));
									}
								}
								else{
									//Error => Les correspondncies entre els camps de la clau forana i l'entitat dest no estan definides.
									throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
								}
								if (mtmm.hasFieldsFKFrom()){
									fieldsFKFrom = mtmm.getFieldsFKFrom();
									elements = fieldsFKFrom.size();
									propertiesFKFrom = mtmm.getPropertiesFKFrom();
									for (int i = 0; i < elements; i++){
										pk.add(MyFrameUOC.getObjectValue(propertiesFKFrom.get(i), obj));
									}
								}
								else{
									//Error => Les correspondncies entre els camps de la clau forana i l'entitat origen no estan definides.
									throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_FROM_ERROR);
								}
								rs = connection.executeSQL(mtmm.getPreparedSqlRead().getQuery(), pk);
								if (!(rs != null && rs.next())){
									connection.executeSQLDDL(mtmm.getPreparedSqlInsert().getQuery(), objCollection, obj);
								}
							}	
						}
					}
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la inserci de les relacions d'un a un de l'entitat.
			throw new MyFrameUOCException(MappingMsg.INSERTMANYTOMANY_ERROR, e.getMessage());
		}
		finally{
			if (rs != null){
				Statement stm	= null;
				try {
					stm = rs.getStatement();
					stm.close();
				} 
				catch (SQLException e) {
				}
				try {
					rs.close();
				} 
				catch (SQLException e) {
				}
			}
		}
	}
	
	/*
	 * Realitza la inserci de les relacions d'un a molts de l'entitat.
	 * 
	 *  @param session - Sessi de MyFrameUOC que s'utilitza.
	 *  @param obj - Objecte que es vol emmagatzemar en la base de dades.
	 */
	private void insertOneToMany(Session session, Object obj) throws MyFrameUOCException{
		RegisteredEntity dest		= null;
		Collection<?> collection	= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.INSERTONETOMANY_SENSE_SESSIO);
			}
			if (obj != null){
				if (this.hasOneToManyMaps()){
					for (OneToManyMap otmm: this.oneToManyMaps){
						dest = otmm.getEntity();
						collection = (Collection<?>) MyFrameUOC.getObjectValue(otmm.getProperty(), obj);
						if (collection != null && collection.size() > 0){
							for (Object objCollection: collection){
								if (!dest.existsObject(session,objCollection)){
									dest.insert(session, objCollection);
								}
								session.getConnection().executeSQLDDL(otmm.getPreparedSqlInsert().getQuery(), objCollection, obj);
							}	
						}
					}
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la inserci de les relacions d'un a un de l'entitat.
			throw new MyFrameUOCException(MappingMsg.INSERTONETOMANY_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Realitza la inserci de les relacions d'un a un de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param obj - Objecte que es vol emmagatzemar en la base de dades.
	 */
	private void insertOneToOne(Session session, Object obj) throws MyFrameUOCException{
		RegisteredEntity dest		= null;
		Object objFK				= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.INSERTONETOONE_SENSE_SESSIO);
			}
			if (obj != null){
				if (this.hasOneToOneMaps()){
					for (OneToOneMap otom: this.oneToOneMaps){
						dest = otom.getEntity();
						objFK = MyFrameUOC.getObjectValue(otom.getProperty(), obj);
						if (objFK != null){
							if (!dest.existsObject(session, objFK)){
								dest.insert(session, objFK);
							}
							session.getConnection().executeSQLDDL(otom.getPreparedSqlInsert().getQuery(), obj);
						}
					}
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la inserci de les relacions d'un a un de l'entitat.
			throw new MyFrameUOCException(MappingMsg.INSERTONETOONE_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Realitza la inserci o l'actuatlizaci de les claus foranes d'un
	 * registre en la taula de l'entitat. 
	 */
	private void insertFK(Session session, Object obj) throws MyFrameUOCException{
		Object objFK				= null;
		RegisteredEntity entityFK	= null;
		Connection connection		= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.INSERTFK_SENSE_SESSIO);
			}
			connection = session.getConnection();
			if (this.hasForeignKeyMaps()){
				for (ForeignKeyMap fkm: this.foreignKeyMaps){
					objFK = MyFrameUOC.getObjectValue(fkm.getProperty(), obj);
					if (objFK != null){
						entityFK = fkm.getEntity();
						if (entityFK.hasPrimaryKeyMaps()){
							if (!entityFK.existsObject(session, objFK)){
								entityFK.insert(session, objFK);
							}
						}
					}
				}
				connection.executeSQLDDL(this.preparedSqlUpdateOnlyFK.getQuery(), obj);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en fer la insercci de les claus foranes.
			throw new MyFrameUOCException(MappingMsg.INSERTFK_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Verifica si un objecte de l'entitat ja est emmagatzemat en la base de dades.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param obj - Objecte de l'entitat que es vol verificar si existeix en la base de dades.
	 */
	private boolean existsObject(Session session, Object obj) throws MyFrameUOCException{
		boolean existeix		= false;
		ArrayList<Object> pk	= null;
		ResultSet rs			= null;
		Connection connection	= null;
		
		try{
			if (session == null){
				//Error => No hi ha una sessi inicialitzada.
				throw new MyFrameUOCException(MappingMsg.EXISTSOBJECT_SENSE_SESSIO);
			}
			connection = session.getConnection();
			if (this.hasPrimaryKeyMaps()){
				pk = new ArrayList<Object>();
				for (PrimaryKeyMap primaryKeyMap: this.primaryKeyMaps){
					pk.add(MyFrameUOC.getObjectValue(primaryKeyMap.getProperty(), obj));
				}
				rs = connection.executeSQL(this.preparedSqlRead.getQuery(), pk);
				if (rs != null && rs.next()){
					existeix = true;
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la verificaci de l'existncia d'un objecte en la BD.
			throw new MyFrameUOCException(MappingMsg.EXISTSOBJECT_ERROR, e.getMessage());
		}
		finally{
			if (rs != null){
				Statement stm = null;
				try {
					stm = rs.getStatement();
					stm.close();
				} 
				catch (SQLException e) {
				}
				try {
					rs.close();
				} 
				catch (SQLException e) {
				}
			}
		}
		return (existeix);
	}
	
	/**
	 * Permet obtenir un objecte de l'entitat que est emmagatzemat en la base de dades.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param values - Llista dels valors de la clau primria, en ordre, de l'objecte que es vol cercar.
	 * @param objSource - Objecte extern que necessita llegir un objecte de l'entitat.
	 * 
	 * @return Un <code>Object</code> amb l'objecte de l'entitat trobat. Nul si no es troba.
	 * 
	 * @throws MyFrameUOCException Si no es pot llegir l'objecte de l'entitat de la base de dades. 
	 */
	public Object read(Session session, ArrayList<Object> values) throws MyFrameUOCException{
		Object obj						= null;
		ResultSet rs					= null;
		Class<?> c						= null;
		String setter					= null;
		String property					= null;
		String type						= null;
		String field					= null;
		boolean deleteReadObjects		= false;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.LLEGIR_ENTITAT_SENSE_SESSIO);
			}
			if (values == null || values.size() == 0){
				//Error => No hi ha valors en la restricci.
				throw new MyFrameUOCException(MappingMsg.LLEGIR_ENTITAT_SENSE_VALORS);
			}
			if (!this.hasPrimaryKeyMaps()){
				//Error => L'entitat no t claus primries.
				throw new MyFrameUOCException(MappingMsg.READ_SENSE_PK);
			}
			if (this.primaryKeyMaps.size() != values.size()){
				//Error => El nmero de claus primries no coincideix amb els passats en la llista de valors.
				throw new MyFrameUOCException(MappingMsg.LLEGIR_ENTITAT_VALORS_DIFERENTS);
			}
			
			obj = this.getReadObject(values);
			
			if (obj != null){
				return (obj);
			}
			
			if (!RegisteredEntity.isReading){
				RegisteredEntity.isReading = true;
				deleteReadObjects = true;
			}
			
			rs = session.getConnection().executeSQL(this.preparedSqlRead.getQuery(), values);
			
			if (rs != null && rs.next()){
				c = Class.forName(this.name);
				obj = c.newInstance();
				if (this.primaryKeyMaps != null && this.primaryKeyMaps.size() > 0){
					for (PrimaryKeyMap primaryKeyMap: this.primaryKeyMaps){
						property = primaryKeyMap.getProperty();
						setter = "set" + String.valueOf(property.charAt(0)).toUpperCase() + property.substring(1, property.length());
						type = primaryKeyMap.getTypeMap().getJavaType().name();
						field = primaryKeyMap.getField();
						this.setValueObject(c, obj, field, type, setter, rs);
					}
				}
				if (this.columnMaps != null && this.columnMaps.size() > 0){
					for (ColumnMap columnMap: this.columnMaps){
						property = columnMap.getProperty();
						setter = "set" + String.valueOf(property.charAt(0)).toUpperCase() + property.substring(1, property.length());
						type = columnMap.getTypeMap().getJavaType().name();
						field = columnMap.getField();
						this.setValueObject(c, obj, field, type, setter, rs);
					}
				}
				
				//Posar l'objecte com a llegit.
				this.addReadObject(obj);
				
				//Llegir la part referent a les claus foranes.
				this.readFK(session, rs, values, obj);
				
				//Llegir els registres de les relacions un a un de l'entitat.
				this.readOneToOne(session, values, obj);
				
				//Llegir els registres de les relacions un a molts de l'entitat.
				this.readOneToMany(session, values, obj);
				
				//Llegir els registres de les relacions molts a molts de l'entitat.
				this.readManyToMany(session, values, obj);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la lectura d'una instncia de l'entitat.
			throw new MyFrameUOCException(MappingMsg.LLEGIR_ENTITAT_ERROR, e.getMessage());
		}
		finally{
			if (rs != null){
				try {
					Statement stm	= null;
					stm = rs.getStatement();
					if (stm != null){
						stm.close();
					}
					rs.close();
				} 
				catch (SQLException e) {
				}
			}
			
			if (deleteReadObjects){
				if (this.hasReadObjects()){
					RegisteredEntity.readObjects.clear();
				}
				RegisteredEntity.isReading = false;
			}
		}
		return (obj);
	}
	
	/*
	 * Llegeix els objectes dels atributs de l'objecte 'obj' que sn claus foranes.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param rs - ResultSet que cont tots els camps del registre de la taula de l'objecte 'obj'.
	 * @param pkObj - Llista de valors de les claus primries de l'objecte 'obj'.
	 * @param obj - Objecte del qual es vol llegir els seus atributs que sn clau forana.
	 */
	private void readFK(Session session, ResultSet rs, ArrayList<Object> pkObj, Object obj) throws MyFrameUOCException{
		ArrayList<Object> fkObj				= null;
		Object objRead						= null;
		RegisteredEntity destEntity			= null;

		try{
			if (session == null || rs == null || pkObj == null || pkObj.size() == 0 || obj == null){
				//Error => Falta informaci per a llegir els objectes dels atributs de les claus foranes d'un objecte.
				throw new MyFrameUOCException(MappingMsg.READFK_FALTA_INFORMACIO);
			}
			fkObj = new ArrayList<Object>();
			if (this.hasForeignKeyMaps()){
				for (ForeignKeyMap fkm: this.foreignKeyMaps){
					fkObj.clear();
					for (String nameField: fkm.getFieldsFK()){
						fkObj.add(rs.getObject(nameField));
					}
					destEntity = fkm.getEntity();
					objRead = destEntity.getReadObject(fkObj);
					if (objRead == null){
						objRead = destEntity.read(session, fkObj);
					}
					MyFrameUOC.setObjectProperty(fkm.getProperty(), obj, objRead);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error al llegir els objectes dels atributs de les claus foranes d'un objecte.
			throw new MyFrameUOCException(MappingMsg.READFK_ERROR, e.getMessage());
		}
	}
	
	/*
	 * Llegeix els objectes de les relacions d'un a un de l'entitat.
	 * 
	 *  @param session - Sessi de MyFrameUOC que s'utilitza.
	 *  @param values - Llista dels valors de la clau primria de l'objecte 'obj'.
	 *  @param obj - Objecte que s'obt de la base de dades.
	 */
	private void readOneToOne(Session session, ArrayList<Object> values, Object obj) throws MyFrameUOCException{
		ResultSet rs					= null;
		Connection connection			= null;
		ArrayList<Object> pkDestObj		= null;
		Object destObj					= null;
		RegisteredEntity dest			= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.READONETOONE_SENSE_SESSIO);
			}
			connection = session.getConnection();
			
			if (this.hasOneToOneMaps()){
				pkDestObj = new ArrayList<Object>();
				
				for (OneToOneMap otom: this.oneToOneMaps){
					pkDestObj.clear();
					rs = connection.executeSQL(otom.getPreparedSqlRead().getQuery(), values);
					if (rs != null && rs.next()){
						if (otom.hasFieldsFKTo()){
							for (String field: otom.getFieldsFKTo()){
								pkDestObj.add(rs.getObject(field));
							}
						}
						else{
							//Error => La correspondncia entre els camps de la clau forana i els atributs de l'entitat dest no est definida.
							throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
						}
					}
					dest = otom.getEntity();
					destObj = dest.getReadObject(pkDestObj);
					if (destObj == null){
						destObj = dest.read(session, pkDestObj);
					}
					MyFrameUOC.setObjectProperty(otom.getProperty(), obj, destObj);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la lectura de les relacions d'un a un d'un objecte.
			throw new MyFrameUOCException(MappingMsg.READONETOONE_ERROR, e.getMessage());
		}
		finally{
			if (rs != null){
				Statement stm	= null;
				try {
					stm = rs.getStatement();
					stm.close();
				} 
				catch (SQLException e) {
				}
				try {
					rs.close();
				} 
				catch (SQLException e) {
				}
			}
		}
	}
	
	/*
	 * Llegeix els objectes de les relacions molts a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param values - Llista dels valors de la clau primria de l'objecte 'obj'.
	 * @param obj - Objecte que s'obt de la base de dades.
	 */
	private void readManyToMany(Session session, ArrayList<Object> values, Object obj) throws MyFrameUOCException{
		ResultSet rs					= null;
		Connection connection			= null;
		ArrayList<Object> pkDestObj		= null;
		Object destObj					= null;
		RegisteredEntity dest			= null;
		Collection<Object> collection	= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.READMANYTOMANY_SENSE_SESSIO);
			}
			connection = session.getConnection();
			
			if (this.hasManyToManyMaps()){
				pkDestObj = new ArrayList<Object>();
				for (ManyToManyMap mtmm: this.manyToManyMaps){
					rs = connection.executeSQL(mtmm.getPreparedSqlReadCollection().getQuery(), values);
					collection = new ArrayList<Object>();
					if (rs != null){
						while (rs.next()){
							pkDestObj.clear();
							if (mtmm.hasFieldsFKTo()){
								for (String field: mtmm.getFieldsFKTo()){
									pkDestObj.add(rs.getObject(field));
								}
							}
							else{
								//Error => La correspondncia entre els camps de la clau forana i els atributs de l'entitat dest no est definida.
								throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
							}

							dest = mtmm.getEntity();
							destObj = dest.getReadObject(pkDestObj);
							if (destObj == null){
								destObj = dest.read(session, pkDestObj);
							}
							collection.add(destObj);
						}
						MyFrameUOC.setObjectProperty(mtmm.getProperty(), obj, collection);
					}
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la lectura de les relacions d'un a un d'un objecte.
			throw new MyFrameUOCException(MappingMsg.READMANYTOMANY_ERROR, e.getMessage());
		}
		finally{
			if (rs != null){
				Statement stm	= null;
				try {
					stm = rs.getStatement();
					stm.close();
				} 
				catch (SQLException e) {
				}
				try {
					rs.close();
				} 
				catch (SQLException e) {
				}
			}
		}
	}
	
	/*
	 * Llegeix els objectes de les relacions un a molts de l'entitat.
	 * 
	 *  @param session - Sessi de MyFrameUOC que s'utilitza.
	 *  @param values - Llista dels valors de la clau primria de l'objecte 'obj'.
	 *  @param obj - Objecte que s'obt de la base de dades.
	 */
	private void readOneToMany(Session session, ArrayList<Object> values, Object obj) throws MyFrameUOCException{
		ResultSet rs					= null;
		Connection connection			= null;
		ArrayList<Object> pkDestObj		= null;
		Object destObj					= null;
		RegisteredEntity dest			= null;
		Collection<Object> collection	= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.READONETOMANY_SENSE_SESSIO);
			}
			connection = session.getConnection();
			
			if (this.hasOneToManyMaps()){
				pkDestObj = new ArrayList<Object>();
				for (OneToManyMap otmm: this.oneToManyMaps){
					rs = connection.executeSQL(otmm.getPreparedSqlRead().getQuery(), values);
					collection = new ArrayList<Object>();
					if (rs != null){
						while (rs.next()){
							pkDestObj.clear();
							if (otmm.hasFieldsFKTo()){
								for (String field: otmm.getFieldsFKTo()){
									pkDestObj.add(rs.getObject(field));
								}
							}
							else{
								//Error => La correspondncia entre els camps de la clau forana i els atributs de l'entitat dest no est definida.
								throw new MyFrameUOCException(MappingMsg.ASSOCIATIONBINARYMAP_BUILDFIELDS_TO_ERROR);
							}

							dest = otmm.getEntity();
							destObj = dest.getReadObject(pkDestObj);
							if (destObj == null){
								destObj = dest.read(session, pkDestObj);
							}
							collection.add(destObj);
						}
						MyFrameUOC.setObjectProperty(otmm.getProperty(), obj, collection);
					}
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en la lectura de les relacions d'un a un d'un objecte.
			throw new MyFrameUOCException(MappingMsg.READONETOMANY_ERROR, e.getMessage());
		}
		finally{
			if (rs != null){
				Statement stm	= null;
				try {
					stm = rs.getStatement();
					stm.close();
				} 
				catch (SQLException e) {
				}
				try {
					rs.close();
				} 
				catch (SQLException e) {
				}
			}
		}
	}
	
	/*
	 * Assigna valors a un objecte cridant als setters corresponents.
	 * 
	 * @param c - La classe de l'objecte.
	 * @param obj - Una instncia nova de la classe de l'objecte.
	 * @param field - Camp del ResultSet que s'ha de recuperar.
	 * @param type - Tiups de Java al qual pertany el tipus de dada del camp.
	 * @param setter - Nom del mtode setter de la classe de l'objecte.
	 * @param rs - ResultSet amb el registre obtingut de la base de dades.
	 */
	private void setValueObject(Class<?> c, Object obj, String field, String type, String setter, ResultSet rs) throws MyFrameUOCException{
		Class<?>[] methodArgs		= null;
		Method method				= null;
		Object[] invokeArgs			= null;
		String value				= null;
		
		try{
			if (JavaType.String.name().equals(type)){
				methodArgs = new Class[] {String.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getString(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.char_.name().equals(type)){
				methodArgs = new Class[] {char.class};
				method = c.getMethod(setter, methodArgs);
				value = rs.getString(field);
				if (value != null){
					invokeArgs = new Object[] {value.charAt(0)};
				}
				else{
					invokeArgs = new Object[] {null};
				}
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.Character.name().equals(type)){
				methodArgs = new Class[] {Character.class};
				method = c.getMethod(setter, methodArgs);
				value = rs.getString(field);
				if (value != null){
					invokeArgs = new Object[] {value.charAt(0)};
				}
				else{
					invokeArgs = new Object[] {null};
				}
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.char_array.name().equals(type)){
				methodArgs = new Class[] {char[].class};
				method = c.getMethod(setter, methodArgs);
				value = rs.getString(field);
				if (value != null){
					invokeArgs = new Object[] {value.toCharArray()};
				}
				else{
					invokeArgs = new Object[] {null};
				}
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.int_.name().equals(type)){
				methodArgs = new Class[] {int.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getInt(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.Integer.name().equals(type)){
				methodArgs = new Class[] {Integer.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getInt(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.float_.name().equals(type)){
				methodArgs = new Class[] {float.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getFloat(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.Float.name().equals(type)){
				methodArgs = new Class[] {Float.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getFloat(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.double_.name().equals(type)){
				methodArgs = new Class[] {double.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getDouble(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.Double.name().equals(type)){
				methodArgs = new Class[] {Double.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getDouble(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.BigDecimal.name().equals(type)){
				methodArgs = new Class[] {BigDecimal.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getBigDecimal(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.Date.name().equals(type)){
				methodArgs = new Class[] {java.sql.Date.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getDate(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.byte_.name().equals(type)){
				methodArgs = new Class[] {byte.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[]{rs.getByte(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.Byte.name().equals(type)){
				methodArgs = new Class[] {Byte.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getByte(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.short_.name().equals(type)){
				methodArgs = new Class[] {short.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getShort(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.Short.name().equals(type)){
				methodArgs = new Class[] {Short.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getShort(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.long_.name().equals(type)){
				methodArgs = new Class[] {long.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getLong(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.Long.name().equals(type)){
				methodArgs = new Class[] {Long.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getLong(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.boolean_.name().equals(type)){
				methodArgs = new Class[] {boolean.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getBoolean(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.Boolean.name().equals(type)){
				methodArgs = new Class[] {Boolean.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getBoolean(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.Time.name().equals(type)){
				methodArgs = new Class[] {java.sql.Time.class};
				method = c.getMethod(setter, methodArgs);
				invokeArgs = new Object[] {rs.getTime(field)};
				method.invoke(obj, invokeArgs);
			}
			else if (JavaType.BigInteger.name().equals(type)){
				methodArgs = new Class[] {java.math.BigInteger.class};
				method = c.getMethod(setter, methodArgs);
				value = rs.getString(field);
				if (value != null){
					invokeArgs = new Object[] {new BigInteger(value)};
				}
				else{
					invokeArgs = new Object[] {null};
				}
				method.invoke(obj, invokeArgs);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'assignaci dels valors.
			throw new MyFrameUOCException(MappingMsg.SETVALUESOBJECT_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Actualitza un objecte en la base de dades.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param obj - L'objecte que cont els valors actualitzats i que es volen passar a la base de dades.
	 * @throws MyFrameUOCException 
	 */
	public void update(Session session, Object obj) throws MyFrameUOCException{
		Object objOld					= null;
		ArrayList<Object> pk			= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.UPDATE_SENSE_SESSIO);
			}

			if (obj != null){
				if (this.hasForeignKeyMaps() || this.hasOneToOneMaps() || this.hasOneToManyMaps() || this.hasManyToManyMaps()){
					if (this.hasPrimaryKeyMaps()){
						pk = new ArrayList<Object>();
						for (PrimaryKeyMap pkm: this.primaryKeyMaps){
							pk.add(MyFrameUOC.getObjectValue(pkm.getProperty(), obj));
						}
						objOld = this.read(session, pk);
						if (objOld != null){
							this.delete(session, objOld);
							this.insert(session, obj);
						}
						else{
							//Error => L'objecte 'obj' no s'ha trobat en la BD.
							throw new MyFrameUOCException(MappingMsg.UPDATE_NO_EXISTEIX_OBJ);
						}
					}
					else{
						//Error => L'entitat no t claus primries.
						throw new MyFrameUOCException(MappingMsg.UPDATE_OBJ_NO_PK);
					}
				}
				else{
					session.getConnection().executeSQLDDL(this.preparedSqlUpdate.getQuery(), obj);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'actualitzaci d'un objecte.
			throw new MyFrameUOCException(MappingMsg.UPDATE_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Esborra un objecte de la base de dades.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param obj - Objecte que es vol esborrar de la base de dades.
	 * 
	 * @throws MyFrameUOCException Si no es pot esborrar l'objecte de l'entitat de la base de dades. 
	 */
	public void delete(Session session, Object obj) throws MyFrameUOCException{
		Object deletedObj					= null;
		boolean deleteDeletedObjects		= false;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.DELETE_SENSE_SESSIO);
			}
			
			if (obj != null){
				deletedObj = this.getDeletedObject(obj);
				
				if (deletedObj == null){
					if (!RegisteredEntity.isDeleting){
						RegisteredEntity.isDeleting = true;
						deleteDeletedObjects = true;
					}
					
					//Posar l'objecte en la llista d'objectes esborrats.
					this.addDeletedObject(obj);
					
					//Eliminar els registres de les relacions un a un de l'entitat.
					this.deleteOneToOne(session, obj);

					//Eliminar els registres de les relacions un a molts de l'entitat.
					this.deleteOneToMany(session, obj);
					
					//Eliminar els registres de les relacions molts a molts de l'entitat.
					this.deleteManyToMany(session, obj);
					
					//Eliminar el registre de la taula de l'entitat corresponent a l'objecte 'obj'.
					session.getConnection().executeSQLDDL(this.preparedSqlDelete.getQuery(), obj);
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'eliminaci d'un objecte de la base de dades.
			throw new MyFrameUOCException(MappingMsg.DELETE_ERROR, e.getMessage());
		}
		finally{
			if (deleteDeletedObjects){
				if (this.hasDeletedObjects()){
					RegisteredEntity.deletedObjects.clear();
				}
				RegisteredEntity.isDeleting = false;
			}
		}
	}
	
	/*
	 * S'encarrega d'esborrar els registres de les taules de les entitats dest de les relacions molts a molts de l'entitat.
	 */
	private void deleteManyToMany(Session session, Object obj) throws MyFrameUOCException{
		RegisteredEntity destEntity			= null;
		Collection<?> collection			= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.DELETEMANYTOMANY_SENSE_SESSIO);
			}
			if (obj == null){
				//Error => No es pot esborrar les relacions de molts a molts d'un objecte que s nul.
				throw new MyFrameUOCException(MappingMsg.DELETEMANYTOMANY_SENSE_OBJ);
			}
			if (this.hasManyToManyMaps()){
				for (ManyToManyMap mtmm: this.manyToManyMaps){
					destEntity = mtmm.getEntity();
					collection = (Collection<?>) MyFrameUOC.getObjectValue(mtmm.getProperty(), obj);
					if (collection != null && collection.size() > 0){
						for (Object destObj: collection){
							destEntity.delete(session, destObj);
						}
					}
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'eliminaci dels registres de les taules de les relacions molts amolts.
			throw new MyFrameUOCException(MappingMsg.DELETEMANYTOMANY_ERROR, e.getMessage());
		}
	}
	
	/*
	 * S'encarrega d'esborrar els registres de les taules de les entitats dest de les relacions un a molts de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param obj - Objecte que es vol esborrar de l'entitat.
	 */
	private void deleteOneToMany(Session session, Object obj) throws MyFrameUOCException{
		RegisteredEntity destEntity			= null;
		Collection<?> collection			= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.DELETEONETOMANY_SENSE_SESSIO);
			}
			if (obj == null){
				//Error => No es pot esborrar les relacions d'un a molts d'un objecte que s nul.
				throw new MyFrameUOCException(MappingMsg.DELETEONETOMANY_SENSE_OBJ);
			}
			if (this.hasOneToManyMaps()){
				for (OneToManyMap otmm: this.oneToManyMaps){
					destEntity = otmm.getEntity();
					collection = (Collection<?>) MyFrameUOC.getObjectValue(otmm.getProperty(), obj);
					if (collection != null && collection.size() > 0){
						for (Object destObj: collection){
							destEntity.delete(session, destObj);
						}
					}
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'eliminaci dels registres de les taules de les relacions un a un.
			throw new MyFrameUOCException(MappingMsg.DELETEONETOMANY_ERROR, e.getMessage());
		}
	}
	
	/*
	 * S'encarrega d'esborrar els registres de les taules de les entitats dest de les relacions un a un de l'entitat.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param obj - Objecte que es vol esborrar de l'entitat.
	 */
	private void deleteOneToOne(Session session, Object obj) throws MyFrameUOCException{
		RegisteredEntity destEntity			= null;
		Object destObj						= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.DELETEONETOONE_SENSE_SESSIO);
			}
			if (obj == null){
				//Error => No es pot esborrar les relacions d'un a un d'un objecte que s nul.
				throw new MyFrameUOCException(MappingMsg.DELETEONETOONE_SENSE_OBJ);
			}
			if (this.hasOneToOneMaps()){
				for (OneToOneMap otom: this.oneToOneMaps){
					destEntity = otom.getEntity();
					destObj = MyFrameUOC.getObjectValue(otom.getProperty(), obj);
					if (destObj != null){
						destEntity.delete(session, destObj);
					}
				}
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'eliminaci dels registres de les taules de les relacions un a un.
			throw new MyFrameUOCException(MappingMsg.DELETEONETOONE_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Permet esborrar una entitat (la seva taula i les taules de les entitats relacionades).
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param nameEntity - Nom de l'entitat que vol esborrar la taula d'una altra entitat.
	 * 
	 * @throws MyFrameUOCException Si no es pot esborrar l'entitat. 
	 */
	public void drop(Session session, String nameEntity) throws MyFrameUOCException{
		Connection connection			= null;
		EntityManager entityManager		= null;
		RegisteredEntity dest			= null;
		
		try{
			if (session == null){
				//Error => La sessi no est inicialitzada.
				throw new MyFrameUOCException(MappingMsg.DROP_SENSE_SESSIO);
			}

			entityManager = session.getEntityManager();
			connection = session.getConnection();
			if (connection.existsTable(this.table) && !this.dropped){
				this.dropped = true;
				//Esborrar les taules a les que apunten les claus foranes.
				if (this.hasForeignKeyMaps()){
					connection.executeSQLDDL(this.preparedSqlAlterTableDropFK.getQuery());
					for (ForeignKeyMap fkm: this.foreignKeyMaps){
						if (!fkm.getEntity().getName().equals(nameEntity)){
							fkm.getEntity().drop(session, this.name);
						}
					}
				}

				//Esborrar les taules de les relacions d'un a un de l'entitat.
				if (this.hasOneToOneMaps()){
					for (OneToOneMap otom: this.oneToOneMaps){
						connection.executeSQLDDL(otom.getPreparedSqlDrop().getQuery());
						dest = otom.getEntity();
						dest.drop(session, dest.name);
					}
				}
				
				//Esborrar les taules de les relacions d'un a molts de l'entitat.
				if (this.hasOneToManyMaps()){
					for (OneToManyMap otmm: this.oneToManyMaps){
						connection.executeSQLDDL(otmm.getPreparedSqlDrop().getQuery());
						dest = otmm.getEntity();
						dest.drop(session, dest.name);
					}
				}
				
				//Esborrar les taules de les relacions de molts a molts de l'entitat.
				if (this.hasManyToManyMaps()){
					for (ManyToManyMap mtmm: this.manyToManyMaps){
						if (connection.existsTable(mtmm.getTable())){
							connection.executeSQLDDL(mtmm.getPreparedSqlDrop().getQuery());
						}
						dest = mtmm.getEntity();
						dest.drop(session, dest.name);
					}
				}
				connection.executeSQLDDL(this.preparedSqlDrop.getQuery());
			}
			entityManager.dropEntity(this);
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'eliminaci de les taules associades a una entitat.
			throw new MyFrameUOCException(MappingMsg.DROP_ERROR, e.getMessage());
		}
	}
	
	/**
	 * Permet conixer si l'entitat t claus primries mapejades.
	 * 
	 * @return <code>True</code> si l'entitat t claus primries mapejades. 
	 * <code>False</code> en qualsevol altre cas.
	 */
	public boolean hasPrimaryKeyMaps(){
		boolean has		= false;
		
		if (this.primaryKeyMaps != null && this.primaryKeyMaps.size() > 0){
			has = true;
		}
		return (has);
	}
	
	/**
	 * Permet conixer si l'entitat t columnes mapejades.
	 * 
	 * @return <code>True</code> si l'entitat t columnes. 
	 * <code>False</code> en qualsevol altre cas.
	 */
	public boolean hasColumnMaps(){
		boolean has		= false;
		
		if (this.columnMaps != null && this.columnMaps.size() > 0){
			has = true;
		}
		return (has);
	}
	
	/**
	 * Permet conixer si l'entitat t claus foranes mapejades.
	 * 
	 * @return <code>True</code> si l'entitat t claus foranes mapejades.
	 * <code>False</code> en qualsevol altre cas.
	 */
	public boolean hasForeignKeyMaps(){
		boolean has		= false;
		
		if (this.foreignKeyMaps != null && this.foreignKeyMaps.size() > 0){
			has = true;
		}
		
		return (has);
	}
	
	/**
	 * Permet conixer si l'entitat t relacions d'un a un mapejades.
	 * 
	 * @return <code>True</code> si l'entitat t relacions d'un a un mapejades. 
	 * <code>False</code> en qualsevol altre cas.
	 */
	public boolean hasOneToOneMaps(){
		boolean has		= false;
		
		if (this.oneToOneMaps != null && this.oneToOneMaps.size() > 0){
			has = true;
		}
		return (has);
	}
	
	/**
	 * Permet conixer si l'entitat t relacions d'un a molts mapejades.
	 * 
	 * @return <code>True</code> si l'entitat t relacions d'un a molts mapejades.
	 * <code>False</code> en qualsevol altre cas.
	 */
	public boolean hasOneToManyMaps(){
		boolean has		= false;
		
		if (this.oneToManyMaps != null && this.oneToManyMaps.size() > 0){
			has = true;
		}
		return (has);
	}
	
	/**
	 * Permet conixer si l'entitat t relacions de molts a molts mapejades.
	 * 
	 * @return <code>True</code> si l'entitat t relacions de molts a molts mapejades.
	 * <code>False</code> en qualsevol altre cas.
	 */
	public boolean hasManyToManyMaps(){
		boolean has		= false;
		
		if (this.manyToManyMaps != null && this.manyToManyMaps.size() > 0){
			has = true;
		}
		
		return (has);
	}
	
	/*
	 * Permet obtenir la clau primria mapejada corresponent a un atribut de l'entitat.
	 * 
	 * @param property - Nom de l'atribut de l'entitat del qual es vol obtenir la seva clau primria mapejada.
	 */
	private PrimaryKeyMap findPrimaryKeyMap(String property){
		PrimaryKeyMap pkm		=  null;
		boolean found			= false;
		int i					= 0;
		
		if (this.hasPrimaryKeyMaps()){
			while (i < this.primaryKeyMaps.size() && !found){
				pkm = this.primaryKeyMaps.get(i);
				if (pkm.getProperty().equals(property)){
					found = true;
				}
				else{
					i++;
				}
			}
		}	
		return (pkm);
	}
	
	/*
	 * Afegeix un objecte en una llista d'objectes llegits.
	 * 
	 * @param obj - L'objecte que s'afegeix a la llista d'objectes llegits.
	 */
	private void addReadObject(Object obj){
		if (RegisteredEntity.readObjects == null){
			RegisteredEntity.readObjects = new ArrayList<Object>();
		}
		if (obj != null){
			RegisteredEntity.readObjects.add(obj);
		}
	}
	
	/*
	 * Permet conixer si una llista d'objectes llegits cont objectes.
	 * 
	 * @return True si la llista d'objectes llegits cont objectes. False en qualsevol altre cas.
	 */
	private boolean hasReadObjects(){
		boolean has = false;
		
		if (RegisteredEntity.readObjects != null && RegisteredEntity.readObjects.size() > 0){
			has = true;
		}
		
		return (has);
	}
	
	/*
	 * Permet obtenir un objecte de la llista d'objectes llegits.
	 * 
	 * @param pkObject - Llista de valors de la clau primria de l'objecte que es vol obtenir.
	 * 
	 * @return Un Object amb l'objecte obtingut de la llista d'objectes llegits. Un nul si l'objecte no est en la 
	 * llista d'objectes llegits.
	 */
	private Object getReadObject(ArrayList<Object> pkObject) throws MyFrameUOCException{
		Object obj							= null;
		ArrayList<Object> pkReadObject		= null;
		
		try{
			if (this.hasPrimaryKeyMaps()){
				if (pkObject != null && pkObject.size() > 0){
					if (this.hasReadObjects()){
						pkReadObject = new ArrayList<Object>();
						for (Object readObject: RegisteredEntity.readObjects){
							if (this.name.equals(readObject.getClass().getCanonicalName())){
								pkReadObject.clear();
								for (PrimaryKeyMap pkm: this.primaryKeyMaps){
									pkReadObject.add(MyFrameUOC.getObjectValue(pkm.getProperty(), readObject));
								}
								if (MyFrameUOC.equals(pkObject, pkReadObject)){
									return (readObject);
								}
							}
						}
					}
				}
			}
			else{
				//Error => L'entitat no t definides claus primries.
				throw new MyFrameUOCException(GeneralMsg.ENTITAT_SENSE_PK);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'obtenci d'un objecte ja llegit.
			throw new MyFrameUOCException(MappingMsg.GETOBJECTREAD_ERROR, e.getMessage());
		}
		
		return (obj);
	}
	
	/*
	 * Afegeix un objecte en la llista d'objectes esborrats.
	 * 
	 * @param obj - L'objecte que s'afegeix en la llista d'objectes esborrats.
	 */
	private void addDeletedObject(Object obj){
		if (RegisteredEntity.deletedObjects == null){
			RegisteredEntity.deletedObjects = new ArrayList<Object>();
		}
		if (obj != null){
			RegisteredEntity.deletedObjects.add(obj);
		}
	}
	
	/*
	 * Permet conixer si una llista d'objectes esborrats cont objectes.
	 * 
	 * @return True si la llista d'objectes esborrats cont objectes. False en qualsevol altre cas.
	 */
	private boolean hasDeletedObjects(){
		boolean has = false;
		
		if (RegisteredEntity.deletedObjects != null && RegisteredEntity.deletedObjects.size() > 0){
			has = true;
		}
		
		return (has);
	}
	
	/*
	 * Permet obtenir un objecte de la llista d'objectes esborrats.
	 * 
	 * @param object - L'objecte quee s vol obtenir de la llista d'objectes esborrats.
	 * 
	 * @return Un Object amb l'objecte obtingut de la llista d'objectes llegits. Un nul si l'objecte no est en la 
	 * llista d'objectes esborrats.
	 */
	private Object getDeletedObject(Object obj) throws MyFrameUOCException{
		ArrayList<Object> pkObj			= null;
		ArrayList<Object> pkReadObj		= new ArrayList<Object>();
				
		try{
			if (this.hasPrimaryKeyMaps()){
				if (obj != null){
					pkObj = new ArrayList<Object>();
					for (PrimaryKeyMap pkm: this.primaryKeyMaps){
						pkObj.add(MyFrameUOC.getObjectValue(pkm.getProperty(), obj));
					}
					if (this.hasDeletedObjects()){
						for (Object readObject: RegisteredEntity.deletedObjects){
							if (this.name.equals(readObject.getClass().getCanonicalName())){
								pkReadObj.clear();
								for (PrimaryKeyMap pkm: this.primaryKeyMaps){
									pkReadObj.add(MyFrameUOC.getObjectValue(pkm.getProperty(), readObject));
								}
								if (MyFrameUOC.equals(pkObj, pkReadObj)){
									return (readObject);
								}
							}
						}
					}
				}
			}
			else{
				//Error => L'entitat no t definides claus primries.
				throw new MyFrameUOCException(GeneralMsg.ENTITAT_SENSE_PK);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'obtenci d'un objecte ja llegit.
			throw new MyFrameUOCException(MappingMsg.GETOBJECTREAD_ERROR, e.getMessage());
		}
		
		return (null);
	}
	
	/*
	 * Afegeix un objecte en la llista d'objectes emmagatzemats.
	 * 
	 * @param obj - L'objecte que s'afegeix en la llista d'objectes emmagatzemats.
	 */
	private void addInsertedObject(Object obj){
		if (RegisteredEntity.insertedObjects == null){
			RegisteredEntity.insertedObjects = new ArrayList<Object>();
		}
		if (obj != null){
			RegisteredEntity.insertedObjects.add(obj);
		}
	}
	
	/*
	 * Permet conixer si una llista d'objectes emmagatzemats cont objectes.
	 * 
	 * @return True si la llista d'objectes emmagatzemats cont objectes. False en qualsevol altre cas.
	 */
	private boolean hasInsertedObjects(){
		boolean has = false;
		
		if (RegisteredEntity.insertedObjects != null && RegisteredEntity.insertedObjects.size() > 0){
			has = true;
		}
		
		return (has);
	}
	
	/*
	 * Permet obtenir un objecte de la llista d'objectes emmagatzemats.
	 * 
	 * @param object - L'objecte quee s vol obtenir de la llista d'objectes emmagatzemats.
	 * 
	 * @return Un Object amb l'objecte obtingut de la llista d'objectes emmagatzemats. Un nul si l'objecte no est en la 
	 * llista d'objectes emmagatzemats.
	 */
	private Object getInsertedObject(Object obj) throws MyFrameUOCException{
		ArrayList<Object> pkObj			= null;
		ArrayList<Object> pkReadObj		= new ArrayList<Object>();
				
		try{
			if (this.hasPrimaryKeyMaps()){
				if (obj != null){
					pkObj = new ArrayList<Object>();
					for (PrimaryKeyMap pkm: this.primaryKeyMaps){
						pkObj.add(MyFrameUOC.getObjectValue(pkm.getProperty(), obj));
					}
					if (this.hasInsertedObjects()){
						for (Object readObject: RegisteredEntity.insertedObjects){
							if (this.name.equals(readObject.getClass().getCanonicalName())){
								pkReadObj.clear();
								for (PrimaryKeyMap pkm: this.primaryKeyMaps){
									pkReadObj.add(MyFrameUOC.getObjectValue(pkm.getProperty(), readObject));
								}
								if (MyFrameUOC.equals(pkObj, pkReadObj)){
									return (readObject);
								}
							}
						}
					}
				}
			}
			else{
				//Error => L'entitat no t definides claus primries.
				throw new MyFrameUOCException(GeneralMsg.ENTITAT_SENSE_PK);
			}
		}
		catch (Exception e){
			//Error => S'ha produt un error en l'obtenci d'un objecte ja llegit.
			throw new MyFrameUOCException(MappingMsg.GETOBJECTREAD_ERROR, e.getMessage());
		}
		
		return (null);
	}
	
	/**
	 * Permet obtenir la sentncia preparada SELECT que serveix per a obtenir un registre de la taula
	 * de l'entitat.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.Select Select}</code> que cont
	 * la sentncia SELECT preparada que serveix per a obtenir un registre de la taula de l'entitat.
	 */
	public Select getPreparedSqlRead(){
		return (this.preparedSqlRead);
	}
}
