package com.uoc.myframeuoc.querymanagement;

import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.exceptions.QueryManagementMsg;

/**
 * Classe que representa una condici que s'afegeix a una restricci.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public class AddCondition {
	//Operador lgic utilitzat per afegir la condici en una restricci.
	private LogicalOperator logicalOperator		= null;
	
	//Condici que s'afegeix en una restricci.
	private Condition condition					= null;
	
	/**
	 * Constructor.
	 * 
	 * @param logicalOperator - Operador lgic utilitzat per afegir la condici en la restricci.
	 * @param condition - Condici que s'afegeix en la restricci.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public AddCondition(LogicalOperator logicalOperator, Condition condition) throws MyFrameUOCException {
		if (logicalOperator == null){
			//Error => No es pot afegir una condici en una restricci sense un operador lgic.
			throw new MyFrameUOCException(QueryManagementMsg.CONDICIO_AFEGIDA_SENSE_OPERADOR_LOGIC);
		}
		if (condition == null){
			//Error => No es pot afegir una condici nulla en una restricci.
			throw new MyFrameUOCException(QueryManagementMsg.CONDICIO_AFEGIDA_NULA);
		}
		this.logicalOperator = logicalOperator;
		this.condition = condition;
	}

	/**
	 * Constructor.
	 * 
	 * @param logicalOperator - Operador lgic utilitzat per afegir la condici en la restricci.
	 * @param relationalOperator - Operador relacional o de comparaci utilitzar en la condici que s'afegeix en la restricci.
	 * @param nameField - Nom del camp del parell camp-valor que participa en la condici que s'afegeix en la restricci.
	 * @param value - Valor del parell camp-valor que participa en la condici que s'afegeix en la restricci.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public AddCondition(LogicalOperator logicalOperator, RelationalOperator relationalOperator, String nameField, String value) throws MyFrameUOCException{
		Condition condition		= null;
		
		if (logicalOperator == null){
			//Error => No es pot afegir una condici en una restricci sense un operador lgic.
			throw new MyFrameUOCException(QueryManagementMsg.CONDICIO_AFEGIDA_SENSE_OPERADOR_LOGIC);
		}
		
		condition = new Condition(relationalOperator, nameField, value);
		this.logicalOperator = logicalOperator;
		this.condition = condition;
	}
	
	/**
	 * Permet obtenir l'operador lgic utlitzat per afegir la condici en una restricci.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.LogicalOperator LogicalOperator}</code>
	 * que cont l'operador lgic utilitzar per afegir la condici en una restricci.
	 */
	public LogicalOperator getLogicalOperator(){
		return (this.logicalOperator);
	}
	
	/**
	 * Permet obtenir la condici que s'afegeix a la restricci.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.Condition Condition}</code>
	 * que cont la condici que s'afegeix a la restricci.
	 */
	public Condition getCondition(){
		return (this.condition);
	}
	
	/**
	 * Permet obtenir una cadena amb l'operador lgic i la condici construda per a poder-la
	 * afegir en una restricci.
	 * 
	 * @return Un <code>String</code> amb l'operador lgic i la condici construda per a poder-la
	 * afegir en una restricci.
	 */
	public String getAddCondition(){
		String addCondition		= "";
		
		//Construir: operador_lgic condicio
		if (this.logicalOperator != null && this.condition != null){
			addCondition = this.logicalOperator.getOperator() + " " + this.condition.getCondition();
		}
		return (addCondition);
	}
}
