package com.uoc.myframeuoc.querymanagement;

import java.util.ArrayList;

import com.uoc.myframeuoc.environment.Configuration;
import com.uoc.myframeuoc.environment.SGBDType;
import com.uoc.myframeuoc.environment.Session;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.exceptions.QueryManagementMsg;

/**
 * Classe que representa i gesiona les sentncies ALTER TABLE utilitzades
 * en MyFrameUOC.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public abstract class AlterTable extends Query {

	//Restriccions de clau forana de la sentncia ALTER TABLE.
	private ArrayList<ForeignKeyConstraint> foreignKeyConstraints		= null;
	
	//Tipus de sentncia ALTER TABLE.
	private final AlterTableType type;
	
	/**
	 * Constructor.
	 * 
	 * @param table - Nom de la taula que participa en la sentncia ALTER TABLE.
	 * @param type - Tipus de sentncia ALTER TABLE.
	 * 
	 * @throws MyFrameUOCException Si no es pot crear la sentncia ALTER TABLE.
	 */
	public AlterTable(String table, AlterTableType type) throws MyFrameUOCException {
		super(table);
		
		if (type == null){
			//Error => No s'ha informat el tipus de sentncia ALTER TABLE que es vol crear.
			throw new MyFrameUOCException(QueryManagementMsg.ALTERTABLE_SENSE_TIPUS);
		}
		
		this.type = type;
	}

	/**
	 * Permet obtenir la sentncia ALTER TABLE construda.
	 * 
	 * @return Un <code>String</code> amb la sentncia ALTER TABLE construda.
	 * 
	 * @exception MyFrameUOCException Si no es pot construir la sentncia ALTER TABLE.
	 */
	@Override
	public abstract String getQuery() throws MyFrameUOCException;
	
	/**
	 * Permet obtenir les restriccions o <i>constraints</i> de clau forana de
	 * la sentncia ALTER TABLE.
	 * 
	 * @return Un <code>ArrayList</code> que cont les restriccions o <i>constraints</i>
	 * de la sentncia ALTER TABLE.
	 */
	public ArrayList<ForeignKeyConstraint> getForeignKeyConstraints(){
		return (this.foreignKeyConstraints);
	}
	
	/**
	 * Afegeix una restricci o <i>constraint</i> de clau forana en la sentncia ALTER TABLE.
	 * 
	 * @param foreignKeyConstraint - Un <code>{@link com.uoc.myframeuoc.querymanagement.ForeignKeyConstraint ForeignKeyConstraint}</code>
	 * que cont la restricci o <i>constraint</i> de clau forana que s'afegir en la sentncia ALTER TABLE. Si es passa un nul,
	 * llavors no s'afegeix res.
	 */
	public void addFKConstraint(ForeignKeyConstraint foreignKeyConstraint){
		if (foreignKeyConstraint != null){
			if (this.foreignKeyConstraints == null){
				this.foreignKeyConstraints = new ArrayList<ForeignKeyConstraint>();
			}
			this.foreignKeyConstraints.add(foreignKeyConstraint);
		}
	}

	/**
	 * Afegeix una restricci o <i>constraint</i> de clau forana en la sentncia ALTER TABLE.
	 * 
	 * @param name - Nom de la restricci o <i>constraint</i> de clau forana.
	 * @param table - Nom de la taula a la qual es fa referncia en la restricci o <i>constraint</i>.
	 * @param pairs - Un <code>ArrayList</code> que cont els parells camp-valor o camp-camp que 
	 * participen en la restricci o <i>constraint</i>.
	 * 
	 * @throws MyFrameUOCException Si no es pot afegir la restricci o <i>constraint</i> de clau forana
	 * en al sentncia ALTER TABLE. 
	 */
	public void addFKConstraintFromPair(String name, String table, ArrayList<FieldValuePair> pairs) throws MyFrameUOCException{
		ForeignKeyConstraint foreignKeyConstraint		= null;
		
		foreignKeyConstraint = new ForeignKeyConstraint(name, table, pairs);
		this.addFKConstraint(foreignKeyConstraint);
	}
	
	/**
	 * Permet obtenir una sentncia ALTER TABLE inicialitzada.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param table - Nom de la taula que participa en la sentncia ALTER TABLE.
	 * @param type - Tipus de sentncia ALTER TABLE.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.AlterTable AlterTable}</code>
	 * que cont la sentncia ALTER TABLE inicialitzada.
	 * 
	 * @throws MyFrameUOCException Si no es pot obtenir una sentncia ALTER TABLE inicialitzada. 
	 */
	public static AlterTable getAlterTable(Session session, String table, AlterTableType type) throws MyFrameUOCException{
		AlterTable alterTable			= null;
		Configuration configuration		= null;
		SGBDType sgbd					= null;
		
		if (session == null){
			//Error => No es pot crear una sentncia ALTER TABLE sense una sessi inicialitzada.
			throw new MyFrameUOCException(QueryManagementMsg.ALTER_TABLE_SENSE_SESSIO);
		}
		configuration = session.getConfiguration();
		sgbd = configuration.getSGBD();
		
		if (SGBDType.Oracle.name().equals(sgbd.name())){
			alterTable = new AlterTableOracle(table, type);
		}
		else if (SGBDType.MySQL.name().equals(sgbd.name())){
			alterTable = new AlterTableMySQL(table, type);
		}
		
		return (alterTable);
	}
	
	/**
	 * Permet validar les propietats de la sentncia ALTER TABLE.
	 * 
	 * @throws MyFrameUOCException Si les propietats de la sentncia ALTER TABLE sn incorrectes. 
	 */
	public void validateQuery() throws MyFrameUOCException{
		if (this.foreignKeyConstraints == null || this.foreignKeyConstraints.size() == 0){
			//Error => No hi ha restriccions de claus foranes en la sentncia ALTER TABLE.
			throw new MyFrameUOCException(QueryManagementMsg.ALTER_TABLE_SENSE_CLAUS_FORANES);
		}
	}

	/**
	 * Permet obtenir el tipus de sentncia ALTER TABLE.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.AlterTableType AlterTableType}</code> que
	 * cont el tipus de sentncia ALTER TABLE.
	 */
	public AlterTableType getType() {
		return type;
	}
}
