package com.uoc.myframeuoc.querymanagement;

import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.exceptions.QueryManagementMsg;

/**
 * Classe que representa una condici d'una restricci d'una sentncia SQL.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public class Condition {

	//Operador relacional o de comparaci utilitzat en la condici.
	private RelationalOperator relationalOperator		= null;
	//Parell de camp-valor (o valor-valor) utilitzat en la condici.
	private FieldValuePair fieldValuePair				= null;
	
	/**
	 * Constructor.
	 * 
	 * @param relationalOperator - Operador relacional o de comparaci que s'utilitza en la condici.
	 * @param fieldValuePair - Parell de camp-valor (o de valor-valor) es compara en la doncici.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public Condition(RelationalOperator relationalOperator, FieldValuePair fieldValuePair) throws MyFrameUOCException {
		if (relationalOperator == null){
			//Error => No es pot crear una condici sense un operador relacional.
			throw new MyFrameUOCException(QueryManagementMsg.CONDICIO_SENSE_OPERADOR_RELACIONAL);
		}
		
		if (fieldValuePair == null){
			//Error => No es pot crear una condici sense un parell camp-valor.
			throw new MyFrameUOCException(QueryManagementMsg.CONDICIO_SENSE_PARELL);
		}
		this.relationalOperator = relationalOperator;
		this.fieldValuePair = fieldValuePair;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param relationalOperator - Operador relacional o de comparaci.
	 * @param nameField	- Nom del camp.
	 * @param value - Valor.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public Condition(RelationalOperator relationalOperator, String nameField, String value) throws MyFrameUOCException{
		Field field				= null;
		FieldValuePair pair		= null;
		
		if (relationalOperator == null){
			//Error => No es pot crear una condici sense un operador relacional.
			throw new MyFrameUOCException(QueryManagementMsg.CONDICIO_SENSE_OPERADOR_RELACIONAL);
		}
		
		field = new Field(nameField);
		pair = new FieldValuePair(field, value);
		this.relationalOperator = relationalOperator;
		this.fieldValuePair = pair;
	}
	
	/**
	 * Permet obtenir l'operador relacional o de comparaci utilitzat en la condici.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.RelationalOperator RelationalOperator}</code>
	 * que cont l'operador relacional o de comparaci utilitzat en la condici.
	 */
	public RelationalOperator getRelationalOperator(){
		return (this.relationalOperator);
	}
	
	/**
	 * Permet obtenir el parell camp-valor que participa en la condici.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.FieldValuePair FieldValuePair}</code>
	 * que cont el parell de camp-valor que participa en la condici.
	 */
	public FieldValuePair getFieldValuePair(){
		return (this.fieldValuePair);
	}

	/**
	 * Permet obtenir la condici construda en una cadena.
	 * 
	 * @return Un <code>String</code> amb la condicin construda.
	 */
	public String getCondition(){
		String condition		= "";
		
		//Construir: operan1 operador_relacional operand2
		if (this.relationalOperator != null && this.fieldValuePair != null){
			condition = this.fieldValuePair.getField().getName() + " " + this.relationalOperator.getOperator() + " " + this.fieldValuePair.getValue();
		}
		return (condition);
	}
}
