package com.uoc.myframeuoc.querymanagement;

import java.util.ArrayList;

import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.exceptions.QueryManagementMsg;

/**
 * Classe que representa i gestiona les restriccions o <i>constraints</i> que es defineixen
 * en una taula d'una base de dades.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public class Constraint {

	//Nom de la restricci o constraint.
	private String name							= null;
	
	//Tipus de restricci o constraint.
	private ConstraintType constraintType		= null;
	
	//Camps que formen part de la restricci o constraint.
	private ArrayList<Field> fields				= null;
	
	/**
	 * Constructor.
	 * 
	 * @param name - Nom de la restricci o <i>constraint</i>.
	 * @param constraintType - Un <code>{@link com.uoc.myframeuoc.querymanagement.ConstraintType ConstraintType}</code>
	 * amb el tipus de restricci o <i>constraint</i>.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public Constraint(String name, ConstraintType constraintType) throws MyFrameUOCException {
		
		if (name == null || "".equals(name)){
			//Error => No es pot crear una restricci sense nom.
			throw new MyFrameUOCException(QueryManagementMsg.CONSTRAINT_SENSE_NOM);
		}
		if (constraintType == null){
			//Error => No es pot crear una restricci sense el tipus de constraint.
			throw new MyFrameUOCException(QueryManagementMsg.CONSTRAINT_SENSE_TIPUS);
		}
		
		this.name = name;
		this.constraintType = constraintType;
	}

	/**
	 * Constructor.
	 * 
	 * @param name - Nom de la restricci o <i>constraint</i>.
	 * @param constraintType - Un <code>{@link com.uoc.myframeuoc.querymanagement.ConstraintType ConstraintType}</code>
	 * amb el tipus de restricci o <i>constraint</i>.
	 * @param fields - Un <code>ArrayList</code> amb els camps que formen part de la restricci o <i>constraint</i>.
	 * @throws MyFrameUOCException 
	 */
	@SuppressWarnings("unchecked")
	public Constraint(String name, ConstraintType constraintType, ArrayList<Field> fields) throws MyFrameUOCException{
		if (name == null || "".equals(name)){
			//Error => No es pot crear una restricci sense nom.
			throw new MyFrameUOCException(QueryManagementMsg.CONSTRAINT_SENSE_NOM);
		}
		if (constraintType == null){
			//Error => No es pot crear una restricci sense el tipus de constraint.
			throw new MyFrameUOCException(QueryManagementMsg.CONSTRAINT_SENSE_TIPUS);
		}
	
		if (fields == null || fields.size() == 0){
			//Error => No s'han indicat campos per afegir en la restricci.
			throw new MyFrameUOCException(QueryManagementMsg.CONSTRAINT_SENSE_CAMPS);
		}
		this.name = name;
		this.constraintType = constraintType;
		this.fields = (ArrayList<Field>) fields.clone();
	}
	
	/**
	 * Permet obtenir el nom de la restricci o <i>constraint</i> que s'afegeix
	 * en la definici d'una taula.
	 * 
	 * @return
	 */
	public String getName(){
		return (this.name);
	}
	
	/**
	 * Permet obtenir el tipus de restricci o <i>constraint</i> que s'afegeix en la
	 * definici d'una taula de la base de dades.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.ConstraintType ConstraintType}</code> que
	 * cont el tipus de restricci o <i>constraint</i>.
	 */
	public ConstraintType getConstraintType(){
		return (this.constraintType);
	}
	
	/**
	 * Permet obtenir els camps que forment part de la restricci o <i>constraint</i>.
	 * 
	 * @return Un <code>ArrayList</code> amb els camps que formen part de la restricci o <i>constraint</i>.
	 */
	public ArrayList<Field> getFields(){
		return (this.fields);
	}
	
	/**
	 * Afegeix un camp perqu formi part de la restricci o <i>constraint</i>.
	 * 
	 * @param field - Camp que s'afegeix a la restricci o <i>constraint</i>. Si es passa un nul, llavors
	 * no s'afegeix res.
	 */
	public void addField(Field field){
		if (field != null){
			if (this.fields == null){
				this.fields = new ArrayList<Field>();
			}
			this.fields.add(field);
		}
	}
	
	/**
	 * Afegeix un camp perqu formi part de la restricci o <i>constraint</i>.
	 * 
	 * @param name - Nom del camp que s'afegeix a la restricci o <i>constraint</i>. Si es passa un nul,
	 * llavors no s'afegeix res.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public void addField(String name) throws MyFrameUOCException{
		Field field		= null;
		
		if (name != null && !"".equals(name)){
			field = new Field(name);
			this.addField(field);
		}
	}
	
	/**
	 * Permet obtenir la restricci o <i>constraint</i> construda.
	 * 
	 * @return Un <code>String</code> amb la restricci o <i>constraint</i> construda.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public String getConstraint() throws MyFrameUOCException{
		String constraint		= null;
		int elements			= 0;
		
		elements = this.fields.size();
		if (this.fields == null || elements == 0){
			//Error => La restricci o constraint no t camps.
			throw new MyFrameUOCException(QueryManagementMsg.CONSTRAINT_SENSE_CAMPS);
		}
		
		constraint = "CONSTRAINT " + this.name + " " + this.constraintType.getConstraint() + "(";
		constraint += this.fields.get(0).getName();
		for (int i = 1; i < elements; i++){
			constraint += ", " + this.fields.get(i).getName();
		}
		constraint += ")";
		
		return (constraint);
	}
}
