package com.uoc.myframeuoc.querymanagement;

import java.util.ArrayList;

import com.uoc.myframeuoc.environment.Configuration;
import com.uoc.myframeuoc.environment.SGBDType;
import com.uoc.myframeuoc.environment.Session;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.exceptions.QueryManagementMsg;

/**
 * Classe que representa i gestiona les sentncies de creaci de taules (CREATE TABLE).
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public abstract class CreateTable extends Query {

	//Columnes que es defineixen en la taula.
	private ArrayList<FieldValuePair> columns							= null;
	
	//Restriccions o constraints que es defineixen en la taula.
	private ArrayList<Constraint> constraints							= null;
	
	//Restriccions o constraints de clau forana que es defineixen en la taula.
	private ArrayList<ForeignKeyConstraint> foreignKeyConstraints		= null;
	
	/**
	 * Constructor.
	 * 
	 * @param table - Nom de la taula que es crea.
	 * 
	 * @throws MyFrameUOCException
	 */
	protected CreateTable(String table) throws MyFrameUOCException {
		super(table);
	}

	/**
	 * Permet obtenir les columnes que es defineixen en la taula.
	 * 
	 * @return Un <code>ArrayList</code> amb les columnes que es defineixen en la taula.
	 */
	public ArrayList<FieldValuePair> getColumns(){
		return (this.columns);
	}
	
	/**
	 * Permet obtenir les restriccions o <i>constraints</i> que es defineixen en la taula.
	 * 
	 * @return Un <code>ArrayList</code> amb les restriccions o <i>constraints</i> que es
	 * defineixen en la taula.
	 */
	public ArrayList<Constraint> getConstraints(){
		return (this.constraints);
	}
	
	/**
	 * Valida les propietats de la sentncia CREATE TABLE.
	 */
	@Override
	public void validateQuery() throws MyFrameUOCException {
		super.validateQuery();
		
		if (this.columns == null || this.columns.size() == 0){
			//Error => La sentncia CREATE TABLE no t columnes definides.
			throw new MyFrameUOCException(QueryManagementMsg.CREATE_TABLE_SENSE_COLUMNES);
		}
	}

	/**
	 * Permet obtenir la sentncia CREATE TABLE que serveix per a crear una taula.
	 * 
	 * @return Un <code>String</code> amb la sentncia CREATE TABLE construda.
	 * 
	 * @exception MyFrameUOCException Si no es pot obtenir la sentncia CREATE TABLE.
	 */
	@Override
	public abstract String getQuery() throws MyFrameUOCException;
	
	/**
	 * Afegeix una nova columna en la definici de la taula a partir d'un parell camp-valor en el qual
	 * el camp fa referncia al nom de la columna en la taula, i el valor representa el tipus de dades
	 * de la columna.
	 * 
	 * @param pair - Parell amb el nom de la columna i el tipus de dades. Si es passa un nul,
	 * no s'afegeix res.
	 */
	public void addColumn(FieldValuePair pair){
		if (pair != null){
			if (this.columns == null){
				this.columns = new ArrayList<FieldValuePair>();
			}
			this.columns.add(pair);
		}
	}
	
	/**
	 * Afegeix una nova columna en la definici de la taula. 
	 * 
	 * @param nameField - Nom de la columna. 
	 * @param tipusDades - Tipus de dades.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public void addColumn(String nameField, String tipusDades) throws MyFrameUOCException{
		FieldValuePair pair		= null;
		
		if (nameField != null && tipusDades != null){
			pair = new FieldValuePair(nameField, tipusDades);
			this.addColumn(pair);
		}
	}
	
	/**
	 * Afegeix una nova restricci o <i>constraint</i> a la definici de la taula.
	 * 
	 * @param constraint - Restricci o <i>constraint</i> que s'afegeix a la definici de la taula. Si
	 * es passa un nul, no s'afegeix res.
	 */
	public void addConstraint(Constraint constraint){
		if (constraint != null){
			if (this.constraints == null){
				this.constraints = new ArrayList<Constraint>();
			}
			this.constraints.add(constraint);
		}
	}
	
	/**
	 * Afegeix una nova restricci o <i>constraint</i> a la definici de la taula.
	 * 
	 * @param name - Nom de la restricci o <i>constraint</i>.
	 * @param constraintType - Un <code>{@link com.uoc.myframeuoc.querymanagement.ConstraintType ConstraintType}</code>
	 * amb el tipus de restricci o <i>constraint</i>.
	 * @param fields - Un <code>ArrayList</code> amb objectes de tipus {@link com.uoc.myframeuoc.querymanagement.Field Field}
	 * que contenen els camps que formen part de la restricci o <i>constraint</i>.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public void addConstraintFromFields(String name, ConstraintType constraintType, ArrayList<Field> fields) throws MyFrameUOCException{
		Constraint constraint	= null;
		
		constraint = new Constraint(name, constraintType, fields);
		this.addConstraint(constraint);
	}
	
	/**
	 * Afegeix una nova restricci o <i>constraint</i> a la definici de la taula.
	 * 
	 * @param name - Nom de la restricci o <i>constraint</i>.
	 * @param constraintType - Un <code>{@link com.uoc.myframeuoc.querymanagement.ConstraintType ConstraintType}</code>
	 * amb el tipus de restricci o <i>constraint</i>.
	 * @param nameFields - Un <code>ArrayList</code> amb objectes de tipus <code>java.lang.String</code> que contenen el
	 * nom dels camps que formen part de la restricci o <i>constraint</i>.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public void addConstraintFromNameFields(String name, ConstraintType constraintType, ArrayList<String> nameFields) throws MyFrameUOCException{
		ArrayList<Field> fields		= null;
		Field field					= null;
		
		if (nameFields == null || nameFields.size() == 0){
			//Error => No es pot afegir una restricci sense camps.
			throw new MyFrameUOCException(QueryManagementMsg.CONSTRAINT_SENSE_CAMPS);
		}
		
		fields = new ArrayList<Field>();
		for (String nameField: nameFields){
			field = new Field(nameField);
			fields.add(field);
		}
		this.addConstraintFromFields(name, constraintType, fields);
	}
	
	/**
	 * Afegeix una restricci o <i>constraint</i> de tipus clau forana en la sentncia CREATE TABLE.
	 * 
	 * @param foreignKeyConstraint - Un <code>{@link com.uoc.myframeuoc.querymanagement.ForeignKeyConstraint ForeignKeyConstraint}</code>
	 * que cont la restricci o <i>constraint</i> de tipus clau forana que es vol afegir a la
	 * sentncia CREATE TABLE. Si es passa un nul, no s'afegeix res.
	 */
	public void addFKConstraint(ForeignKeyConstraint foreignKeyConstraint){
		if (foreignKeyConstraint != null){
			if (this.foreignKeyConstraints == null){
				this.foreignKeyConstraints = new ArrayList<ForeignKeyConstraint>();
			}
			this.foreignKeyConstraints.add(foreignKeyConstraint);
		}
	}
	
	/**
	 * Afegeix una restricci o <i>constraint</i> de tipus clau forana en la sentncia CREATE TABLE.
	 * 
	 * @param name - Nom de la clau forana.
	 * @param table - Taula a la que es fa referncia.
	 * @param fieldValurPairs - Parells de camp-valor o camp-camp que intervenen en la clau forana.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public void addFKConstraintFromPairs(String name, String table, ArrayList<FieldValuePair> fieldValuePairs) throws MyFrameUOCException{
		ForeignKeyConstraint foreignKeyConstraint		= null;
		
		foreignKeyConstraint = new ForeignKeyConstraint(name, table, fieldValuePairs);
		this.addFKConstraint(foreignKeyConstraint);
	}
	
	/**
	 * Permet obtenir les restriccions o <i>constraints</i> de clau forana que t la
	 * sentncia CREATE TABLE.
	 * 
	 * @return Un <code>ArrayList</code> amb les restriccions o <i>constraints</i> de clau forana
	 * que t la sentncia CREATE TABLE.
	 */
	public ArrayList<ForeignKeyConstraint> getFKConstraints(){
		return (this.foreignKeyConstraints);
	}
	
	/**
	 * Permet obtenir una sentncia de CREATE TABLE.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param table - Nom de la taula que participa en la sentncia CREATE TABLE.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.CreateTable}</code> que cont
	 * la sentncia CREATE TABLE inicialitzada.
	 * 
	 * @throws MyFrameUOCException Si no es pot crear la sentncia CREATE TABLE. 
	 */
	public static CreateTable getCreateTable(Session session, String table) throws MyFrameUOCException{
		CreateTable createTable			= null;
		Configuration configuration		= null;
		SGBDType sgbd					= null;
		
		if (session == null){
			//Error => La sessi no est inicialitzada.
			throw new MyFrameUOCException(QueryManagementMsg.CREATE_TABLE_SENSE_SESSIO);
		}
		
		configuration = session.getConfiguration();
		sgbd = configuration.getSGBD();
		
		if (SGBDType.MySQL.name().equals(sgbd.name())){
			//El SGBD que s'utilitza s MySQL.
			createTable = new CreateTableMySQL(table);
			
		}
		else if (SGBDType.Oracle.name().equals(sgbd.name())){
			//El SGBD que s'utilitza s Oracle.
			createTable = new CreateTableOracle(table);
		}
		return (createTable);
	}
	
	/**
	 * Permet copiar una sentncia CREATE TABLE per sense les restriccions o <i>constraints</i>
	 * de clau forana.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.CreateTable CreateTable}</code>
	 * que cont la mateix sentncia CREATE TABLE per sense les restriccions o <i>constraints</i>
	 * de clau forana.
	 * 
	 * @throws MyFrameUOCException Si no es pot copiar la sentncia CREATE TABLE sense les
	 * restriccions o <i>constraints</i> de clau forana. 
	 */
	@SuppressWarnings("unchecked")
	public CreateTable copyNoFK(Session session) throws MyFrameUOCException{
		CreateTable createTable		= null;
		
		createTable = CreateTable.getCreateTable(session, this.getTable());
		createTable.columns = (ArrayList<FieldValuePair>) this.columns.clone();
		createTable.constraints = (ArrayList<Constraint>) this.constraints.clone();
		
		return (createTable);
	}
}
