package com.uoc.myframeuoc.querymanagement;

import java.util.ArrayList;

import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.exceptions.QueryManagementMsg;

/**
 * Classe que representa i gestiona les restriccions de clau forana que 
 * s'utilitzen en MyFrameUOC.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public class ForeignKeyConstraint {

	//Nom de la clau forana en la taula de la base de dades.
	private String name										= null;
	
	//Taula a la qual es referencia.
	private String table									= null;
	
	//Parells de camp-valor o camp-camp que intervenen en la clau forana.
	private ArrayList<FieldValuePair> fieldValuePairs		= null;
	
	
	/**
	 * Constructor.
	 * 
	 * @param name - Nom de la clau forana.
	 * @param table - Taula a la qual es fa referncia.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public ForeignKeyConstraint(String name, String table) throws MyFrameUOCException {
		if (name == null || "".equals(name)){
			//Error => No es pot crear una clau forana sense nom.
			throw new MyFrameUOCException(QueryManagementMsg.CLAUFORANA_SENSE_NOM);
		}
		if (table == null || "".equals(table)){
			//Error => No es pot crear una clau forana sense taula de referncia.
			throw new MyFrameUOCException(QueryManagementMsg.CLAUFORANA_SENSE_TAULA);
		}
		this.name = name;
		this.table = table;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param name - Nom de la clau forana.
	 * @param table - Taula a la qual es fa referncia.
	 * @param fieldValuePairs - Un <code>ArrayList</code> que cont parells de camp-valor o camp-camp
	 * que intervenen en la clau forana.
	 * 
	 * @throws MyFrameUOCException 
	 */
	@SuppressWarnings("unchecked")
	public ForeignKeyConstraint(String name, String table, ArrayList<FieldValuePair> fieldValuePairs) throws MyFrameUOCException{
		if (name == null || "".equals(name)){
			//Error => No es pot crear una clau forana sense nom.
			throw new MyFrameUOCException(QueryManagementMsg.CLAUFORANA_SENSE_NOM);
		}
		if (table == null || "".equals(table)){
			//Error => No es pot crear una clau forana sense taula de referncia.
			throw new MyFrameUOCException(QueryManagementMsg.CLAUFORANA_SENSE_TAULA);
		}
		if (fieldValuePairs == null || fieldValuePairs.size() == 0){
			//Error => La clau forana no t parells camp-camp.
			throw new MyFrameUOCException(QueryManagementMsg.CLAUFORANA_SENSE_CAMPS);
		}
		this.name = name;
		this.table = table;
		this.fieldValuePairs = (ArrayList<FieldValuePair>) fieldValuePairs.clone();
	}
	
	/**
	 * Permet obtenir el nom de la clau forana.
	 * 
	 * @return Un <code>String</code> amb el nom de la clau forana.
	 */
	public String getName(){
		return (this.name);
	}
	
	/**
	 * Permet obtenir la taula de referncia de la clau forana.
	 * 
	 * @return Un <code>String</code> amb la taula de referncia de la clau forana.
	 */
	public String getTable(){
		return (this.table);
	}
	
	/**
	 * Permet obtenir els parells de camp-valor o camp-camp que intervenen
	 * en la clau forana.
	 * 
	 * @return Un <code>ArrayList</code> que cont els parells de camp-valor o camp-camp
	 * que intervenen en la clau forana.
	 */
	public ArrayList<FieldValuePair> getFieldValuePairs(){
		return (this.fieldValuePairs);
	}
	
	/**
	 * Afegeix un parell camp-valor o camp-camp en la clau forana.
	 * 
	 * @param pair - Un <code>{@link com.uoc.myframeuoc.querymanagement.FieldValuePair FieldValuePair}</code> que cont
	 * el parell de camp-valor o camp-camp que s'afegir a la clau forana. Si es passa un nul, llavors
	 * no s'afegeix res.
	 */
	public void addFieldValuePair(FieldValuePair pair){
		if (pair != null){
			if (this.fieldValuePairs == null){
				this.fieldValuePairs = new ArrayList<FieldValuePair>();
			}
			this.fieldValuePairs.add(pair);
		}
	}
	
	/**
	 * Afegeix un parell camp-valor o camp-vamp en la clau forana.
	 * 
	 * @param name - Nom de del camp.
	 * @param value - Valor o nom del camp de la taula de referncia.
	 * 
	 * @throws MyFrameUOCException 
	 */
	public void addFieldValuePair(String name, String value) throws MyFrameUOCException{
		FieldValuePair pair		= null;
		
		pair = new FieldValuePair(name, value);
		this.addFieldValuePair(pair);
	}
	
	/**
	 * Permet obtenir la restricci o <i>constraint</i> construda de la clau forana d'una
	 * sentncia CREATE TABLE.
	 * 
	 * @return Un <code>String</code> amb la restricci o <i>constraint</i> construda de la
	 * clau forana d'una sentncia CREATE TABLE. 
	 * 
	 * @throws MyFrameUOCException 
	 */
	public String getForeignKeyConstraint() throws MyFrameUOCException{
		String constraint		= null;
		String referencia		= null;
		int elements			= 0;
		FieldValuePair pair		= null;
		
		if (this.fieldValuePairs == null || this.fieldValuePairs.size() == 0){
			//Error => La restricci de clau forana no t camps.
			throw new MyFrameUOCException(QueryManagementMsg.CLAUFORANA_SENSE_CAMPS);
		}
		constraint = "CONSTRAINT " + this.name + " FOREIGN KEY(";
		elements = this.fieldValuePairs.size();
		pair = this.fieldValuePairs.get(0);
		constraint += pair.getField().getName();
		referencia = "REFERENCES " + this.table + "(" + pair.getValue();
		for (int i = 1; i < elements; i++){
			pair = this.fieldValuePairs.get(i);
			constraint += ", " + pair.getField().getName();
			referencia += ", " + pair.getValue();
		}
		constraint += ")";
		referencia += ")";
		constraint += " " + referencia;
		return (constraint);
	}
}
