package com.uoc.myframeuoc.querymanagement;

import java.util.ArrayList;

import com.uoc.myframeuoc.environment.Configuration;
import com.uoc.myframeuoc.environment.SGBDType;
import com.uoc.myframeuoc.environment.Session;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.exceptions.QueryManagementMsg;

/**
 * Classe que representa una sentncia INSERT.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public abstract class Insert extends Query {

	//Parells camp-valor de la sentncia INSERT.
	private ArrayList<FieldValuePair> fieldValuePairs		= null;
	
	/**
	 * Constructor.
	 * 
	 * @param queryCommand - Tipus de comanda.
	 * @param table - Taula que interv en la consulta SQL.
	 * 
	 * @throws MyFrameUOCException Si no es pot crear la sentncia INSERT. 
	 */
	protected Insert(String table) throws MyFrameUOCException {
		super(table);
	}

	/**
	 * Permet obtenir el conjunt de parells camp-valor de la sentncia INSERT.
	 * 
	 * @return Un <code>ArrayList</code> amb els parells camp-valor de la sentncia INSERT.
	 */
	public ArrayList<FieldValuePair> getFieldValuePairs(){
		return (this.fieldValuePairs);
	}
	
	/**
	 * Permet afegir una nova parella de camp-valor dins de la sentncia INSERT.
	 * 
	 * @param pair - Parell de camp-valor. Si el parell es nul, llavors no s'afegeix a
	 * la sentncia INSERT.
	 */
	public void addFieldValuePair(FieldValuePair pair){
		if (pair != null){
			if (this.fieldValuePairs == null){
				this.fieldValuePairs = new ArrayList<FieldValuePair>();
			}
			this.fieldValuePairs.add(pair);
		}
	}

	/**
	 * Permet afegir una nova parella de camp-valor dins de la sentncia INSERT.
	 * 
	 * @param nameField - Nom del camp.
	 * @param value - Valor.
	 * 
	 * @throws MyFrameUOCException Si no es pot afegir una nova parella de camp-valor dins de la sentncia INSERT.
	 */
	public void addFieldValuePair(String nameField, String value) throws MyFrameUOCException{
		Field field				= null;
		FieldValuePair pair		= null;
		
		field = new Field(nameField);
		pair = new FieldValuePair(field, value);
		
		this.addFieldValuePair(pair);
	}
	
	/**
	 * Permet obtenir la sentncia INSERT construda.
	 * 
	 * @return Un <code>String</code> amb la sentncia INSERT construda.
	 * 
	 * @exception MyFrameUOCException Si no es pot obtenir la sentncia INSERT construda.
	 */
	@Override
	public abstract String getQuery() throws MyFrameUOCException;

	/**
	 * Valida les propietats de la sentncia INSERT.
	 * 
	 * @throws MyFrameUOCException Si no es poden validar les propietats de la sentncia INSERT.
	 */
	@Override
	public void validateQuery() throws MyFrameUOCException {
		super.validateQuery();
		
		if (this.fieldValuePairs == null || this.fieldValuePairs.size() == 0){
			//Error => No hi ha parells de camp-valor afegits.
			throw new MyFrameUOCException(QueryManagementMsg.NO_PARELLS_CAMP_VALOR);
		}
	}
	
	/**
	 * Permet obtenir una sentncia INSERT inicialitzada.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param table - Nom de la taula que participa en la sentncia INSERT.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.Insert Insert}</code> que
	 * cont la sentncia INSERT inicialitzada.
	 * 
	 * @throws MyFrameUOCException Si no es pot obtenir una sentncia INSERT inicialitzada. 
	 */
	public static Insert getInsert(Session session, String table) throws MyFrameUOCException{
		Insert insert					= null;
		Configuration configuration		= null;
		SGBDType sgbd					= null;
		
		if (session == null){
			//Error => La sessi no est incialitzada.
			throw new MyFrameUOCException(QueryManagementMsg.INSERT_SENSE_SESSIO);
		}
		configuration = session.getConfiguration();
		sgbd = configuration.getSGBD();
		
		if (SGBDType.Oracle.name().equals(sgbd.name())){
			insert = new InsertOracle(table);
		}
		else if (SGBDType.MySQL.name().equals(sgbd.name())){
			insert = new InsertMySQL(table);
		}
		return (insert);
	}
}
