package com.uoc.myframeuoc.querymanagement;

import java.util.ArrayList;

import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.exceptions.QueryManagementMsg;

/**
 * Classe que representa una restricci d'una consulta.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public class Restriction {

	//Primera condici de la restricci (l'ha que est al costat del WHERE).
	private Condition condition							= null;
	
	//Condicions que s'afegeixen a la restricci.
	private ArrayList<AddCondition> addConditions		= null;
	
	/**
	 * Constructor.
	 * 
	 * @param condition - Un <code>{@link com.uoc.myframeuoc.querymanagement.Condition Condition}</code>
	 * amb la condici de la restricci.
	 * 
	 * @throws MyFrameUOCException Si no es pot crear la restricci. 
	 */
	public Restriction(Condition condition) throws MyFrameUOCException{
		if (condition == null){
			//Error => No es pot crear una restricci sense una condici.
			throw new MyFrameUOCException(QueryManagementMsg.RESTRICCIO_SENSE_CONDICIO);
		}
		this.condition = condition;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param relationalOperator - Operador relacional o de comparaci utilitzat en la primera condici de la restricci.
	 * @param nameField - Nom del camp del parell camp-valor que participa en la primera condici de la restricci.
	 * @param value - Valor del parell camp-valor que participa en la primera condici de la restricci.
	 * 
	 * @throws MyFrameUOCException Si no es pot crear la restricci.
	 */
	public Restriction(RelationalOperator relationalOperator, String nameField, String value) throws MyFrameUOCException{
		Condition condition		= null;
		
		condition = new Condition(relationalOperator, nameField, value);
		
		this.condition = condition;
	}
	
	/**
	 * Afegeix una condici a la restricci.
	 * 
	 * @param addCondition - Un <code>{@link com.uoc.myframeuoc.querymanagement.AddCondition AddCondition}</code>
	 * amb la condici que s'afegeix a la restricci. Si es passa un nul, no s'afegeix res.
	 */
	public void addCondition(AddCondition addCondition){
		if (addCondition != null){
			if (this.addConditions == null){
				this.addConditions = new ArrayList<AddCondition>();
			}
			this.addConditions.add(addCondition);
		}
	}
	
	/**
	 * Afegeix una condici en la restricci.
	 * 
	 * @param logicalOperator - Operador lgic utilitzat per afegir la condici en la restricci.
	 * @param relationalOperator - Operador relacionals o de comparaci utilitzat en la condici.
	 * @param nameField - Nom del camp del parell camp-valor que participa en la condici que s'afegeix en la restricci.
	 * @param value - Valor del parell camp-valor que participa en la condici que s'afegeix en la restricci.
	 * 
	 * @throws MyFrameUOCException Si no es pot afegir la condici en la restricci.
	 */
	public void addCondition(LogicalOperator logicalOperator, RelationalOperator relationalOperator, String nameField, String value) throws MyFrameUOCException{
		AddCondition addCondition		= null;
		
		addCondition = new AddCondition(logicalOperator, relationalOperator, nameField, value);
		this.addCondition(addCondition);
	}
	
	/**
	 * Permet obtenir la condici de la restricci.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.Condition Condition}</code>
	 * amb la condici de la restricci.
	 */
	public Condition getCondition(){
		return (this.condition);
	}
	
	/**
	 * Permet obtenir les condiciones que s'han afegit a la restricci.
	 * 
	 * @return Un <code>ArrayList</code> amb les restriccions que s'han afegit a la restricci.
	 */
	public ArrayList<AddCondition> getAddConditions(){
		return (this.addConditions);
	}
	
	/**
	 * Permet obtenir una cadena amb la restricci construda amb totes les seves condicions. Aquesta
	 * cadena es podr afegir com a restricci en una sentncia SQL.
	 * 
	 * @return Un <code>String</code> amb la restricci construda amb totes les seves condicioins.
	 */
	public String getRestriction(){
		String restriction		= "";
		
		if (this.condition != null){
			restriction = this.condition.getCondition();
			if (this.addConditions != null && this.addConditions.size() > 0){
				//Afegir la resta de condicions a la restricci.
				for (AddCondition addCondition: this.addConditions){
					restriction += " " + addCondition.getAddCondition();
				}
			}
		}
		return (restriction);
	}
}
