package com.uoc.myframeuoc.querymanagement;

import java.util.ArrayList;

import com.uoc.myframeuoc.environment.Configuration;
import com.uoc.myframeuoc.environment.SGBDType;
import com.uoc.myframeuoc.environment.Session;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.exceptions.QueryManagementMsg;

/**
 * Classe que representa una sentncia SELECT.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public abstract class Select extends RestrictionQuery {

	//Camps que es projecten en la sentncia SELECT.
	private ArrayList<Field> fields		= null;
	
	/**
	 * Constructor.
	 * 
	 * @param table - Nom de la taula que interv en la consulta.
	 * 
	 * @throws MyFrameUOCException Si no es pot crear la sentncia SELECT.
	 */
	protected Select(String table) throws MyFrameUOCException {
		super(table);
	}

	/**
	 * Permet obtenir la sentncia SELECT construda.
	 * 
	 * @return Un <code>String</code> amb la sentncia SELECT construda.
	 * 
	 * @exception MyFrameUOCException Si no es pot obtenir la sentncia SELECT construda.
	 */
	@Override
	public abstract String getQuery() throws MyFrameUOCException;
	
	/**
	 * Permet obtenir els camps que es projecten en la sentncia SELECT.
	 * 
	 * @return Un <code>ArrayList</code> amb els camps que es projecten
	 * en la sentncia SELECT.
	 */
	public ArrayList<Field> getFields(){
		return (this.fields);
	}
	
	/**
	 * Afegeix un camp a projectar en la sentncia SELECT. 
	 * 
	 * @param field - Camp que s'afegeix a la sentncia SELECT. Si es passa un nul,
	 * llavors no s'afegeix res.
	 */
	public void addField(Field field){
		if (field != null){
			if (this.fields == null){
				this.fields = new ArrayList<Field>();
			}
			this.fields.add(field);
		}
	}

	/**
	 * Afegeix un camp a projectar en la sentncia SELECT.
	 * 
	 * @param nameField - Nom del camp.
	 * 
	 * @throws MyFrameUOCException Si no es pot afegir el camp a projectar en la sentncia SELECT.
	 */
	public void addField(String nameField) throws MyFrameUOCException{
		Field field		= null;
		
		field = new Field(nameField);
		
		this.addField(field);
	}
	
	/**
	 * Permet validar les propietats de la sentncia SELECT.
	 */
	@Override
	public void validateQuery() throws MyFrameUOCException {
		super.validateQuery();
		
		if (this.fields == null || this.fields.size() == 0){
			//Error => La sentncia SELECT no t camps per a projectar.
			throw new MyFrameUOCException(QueryManagementMsg.SELECT_SENSE_CAMPS);
		}
	}	
	
	/**
	 * Permet obtenir una sentncia SELECT inicialitzada.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param table - Nom de la taula que participa en la sentncia SELECT.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.Select Select}</code> que
	 * cont la sentncia SELEC inicialitzada.
	 * 
	 * @throws MyFrameUOCException Si no es pot obtenir una sentncia SELECT inicialitzada. 
	 */
	public static Select getSelect(Session session, String table) throws MyFrameUOCException{
		Select select					= null;
		Configuration configuration		= null;
		SGBDType sgbd					= null;
		
		if (session == null){
			//Error => La sessi no est inicialitzada.
			throw new MyFrameUOCException(QueryManagementMsg.SELECT_SENSE_SESSIO);
		}
		configuration = session.getConfiguration();
		sgbd = configuration.getSGBD();
		
		if (SGBDType.Oracle.name().equals(sgbd.name())){
			select = new SelectOracle(table); 
		}
		else if (SGBDType.MySQL.name().equals(sgbd.name())){
			select = new SelectMySQL(table);
		}
		return (select);
	}
}
