package com.uoc.myframeuoc.querymanagement;

import java.util.ArrayList;

import com.uoc.myframeuoc.environment.Configuration;
import com.uoc.myframeuoc.environment.SGBDType;
import com.uoc.myframeuoc.environment.Session;
import com.uoc.myframeuoc.exceptions.MyFrameUOCException;
import com.uoc.myframeuoc.exceptions.QueryManagementMsg;

/**
 * Classe que representa una sentncia UPDATE.
 * 
 * @author Jordi Garcia Casanova.
 *
 */
public abstract class Update extends RestrictionQuery {

	//Parells de camp-valor utilitzats en la clusula SET de la sentncia UPDATE.
	private ArrayList<FieldValuePair> fieldValuePairs		= null;
	
	/**
	 * Constructor.
	 * 
	 * @param table - Nom de la taula associada a a ls entncia UPDATE.
	 * 
	 * @throws MyFrameUOCException Si no es pot crear la sentncia UPDATE.
	 */
	protected Update(String table) throws MyFrameUOCException {
		super(table);
	}

	/**
	 * Permet obtenir la sentncia UPDATE construda.
	 * 
	 * @return Un <code>String</code> amb la sentncia UPDATE construda.
	 * 
	 * @throws MyFrameUOCException Si no es pot obtenir la sentncia UPDATE construda.
	 */
	@Override
	public abstract String getQuery() throws MyFrameUOCException;
	
	/**
	 * Valida les propietats de la sentncia UPDATE.
	 * 
	 * @throws MyFrameUOCException Si no es pot validar la sentncia UPDATE.
	 */
	@Override
	public void validateQuery() throws MyFrameUOCException {
		super.validateQuery();
		
		if (this.fieldValuePairs == null || this.fieldValuePairs.size() == 0){
			//Error => No s'han definit els parells camp-valor de la sentncia UPDATE.
			throw new MyFrameUOCException(QueryManagementMsg.UPDATE_SENSE_PARELLS);
		}
	}

	/**
	 * Permet obtenir els parells camp-valor de la sentncia UPDATE.
	 * 
	 * @return Un <code>ArrayList</code> amb els parells camp-valor de la
	 * sentncia UPDATE.
	 */
	public ArrayList<FieldValuePair> getFieldValuePairs(){
		return (this.fieldValuePairs);
	}

	/**
	 * Afegeix un nou parell camp-valor a la sentncia UPDATE.
	 * 
	 * @param pair - Un <code>{@link com.uoc.myframeuoc.querymanagement.FieldValuePair FieldValuePair}</code> 
	 * amb el parell camp-valor que es vol afegir a la sentncia UPDATE. Si es passa un valor nul,
	 * llavors no s'afegeix res.
	 */
	public void addFieldValuePair(FieldValuePair pair){
		if (pair != null){
			if (this.fieldValuePairs == null){
				this.fieldValuePairs = new ArrayList<FieldValuePair>();
			}
			this.fieldValuePairs.add(pair);
		}
	}
	
	/**
	 * Afegeix un nou parell camp-valor en la sentncia UPDATE.
	 * 
	 * @param nameField - Nom del camp del parell camp-valor.
	 * @param value - Valor del parell camp-valor.
	 * 
	 * @throws MyFrameUOCException Si no es pot afegir un nou parell camp-valor en la sentncia UPDATE.
	 */
	public void addFieldValuePair(String nameField, String value) throws MyFrameUOCException{
		FieldValuePair pair		= null;
		
		pair = new FieldValuePair(nameField, value);
		this.addFieldValuePair(pair);
	}
	
	/**
	 * Permet obtenir una sentncia UPDATE inicialitzada.
	 * 
	 * @param session - Sessi de MyFrameUOC que s'utilitza.
	 * @param table - Nom de la taula que participa en la sentncia UPDATE.
	 * 
	 * @return Un <code>{@link com.uoc.myframeuoc.querymanagement.Update Update}</code> que
	 * cont la sentncia UPDATE inicialitzada.
	 * 
	 * @throws MyFrameUOCException Si no es pot obtenir una sentncia UPDATE inicialitzada. 
	 */
	public static Update getUpdate(Session session, String table) throws MyFrameUOCException{
		Update update					= null;
		Configuration configuration		= null;
		SGBDType sgbd					= null;
		
		if (session == null){
			//Error => La sessi no est inicialitzada.
			throw new MyFrameUOCException(QueryManagementMsg.UPDATE_SENSE_SESSIO);
		}
		
		configuration = session.getConfiguration();
		sgbd = configuration.getSGBD();
		if (SGBDType.Oracle.name().equals(sgbd.name())){
			update = new UpdateOracle(table);
		}
		else if (SGBDType.MySQL.name().equals(sgbd.name())){
			update = new UpdateMySQL(table);
		}
		return (update);
	}
}
